/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("jazz.util.i18n");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("jazz.app.context");
dojo.require("com.ibm.asq.common.web.client.internal.ConfigurationManagementClient");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");


(function() {    


var Viewlet 						= com.ibm.team.dashboard.web.ui.Viewlet;
var DashboardConstants              = com.ibm.team.dashboard.web.ui.DashboardConstants;
var getCurrentPA            	  	= jazz.app.context.get;
var ConfigurationManagementClient 	= com.ibm.asq.common.web.client.internal.ConfigurationManagementClient;
var I18nUtil 						= jazz.util.i18n;
var ServiceResponseHandler 			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet",  Viewlet, 
{   

	//Override from Viewlet
	//AVT issue: Alt is missing for the icons added in the viewlets. These images are fetched from the registry
	init: function() {
		var context = this.getSite().context;
		var title = context.getTitle();
		if(title == ""){
			title = "Image";//$NON-NLS
		}
		if(context && context._iconImage){
			if(context._iconImage.alt == ""){
				context._iconImage.alt = title; 
			} 
		}
		if(context && context._headerIcon){
			var child = context._headerIcon.childNodes && context._headerIcon.childNodes.length > 0 
								? context._headerIcon.childNodes[0] : null;
			if(child && child.alt ==""){
				child.alt = title; 
			}
		}
	},
	
	 /**
     * Check if the viewlet is in a shared dashboard.  The dashboard scope is not the same as the
	 * viewlet scope.  Viewlet scope is defined in the applies-to attribute or user selection if
	 * the viewlet is defined as applies-to all and scope-sensitive.
	 */
	isSharedDashboard: function() {
        var site = this.getSite();
        // internal attribute of com.ibm.team.dashboard.web.ui.internal.ViewletSiteAdapter
        var dashboardScope =  site.context.properties["com.ibm.team.dashboard.scope"];
	    return dashboardScope != DashboardConstants.SCOPE_CONTRIBUTOR;
	},
	
	createNoScopeMsg: function() {	
        var msgSpan = document.createElement("span");
        msgSpan.innerHTML =  com.ibm.asq.common.web.client.internal.Constants.getMessage("viewletnotConfigured");
        if (this.getSite().isDashboardEditing()) {
            var editSpan = document.createElement("span");
            var nodes = I18nUtil.addMessage(editSpan,  com.ibm.asq.common.web.client.internal.Constants.getMessage("selectArea"));
            this.connect(nodes[0], "onclick", this.handleToggleSettings);
            msgSpan.appendChild(editSpan);
        }
        return msgSpan;
    },
    
    handleToggleSettings: function(event) {
        dojo.stopEvent(event);
        this.getSite().toggleSettings();
    },
    
	getProjectArea: function() {		
		var scopeItem = this.getScopeItem();    // Report via saved query (remote)
        if (scopeItem && scopeItem.itemId) {
            return scopeItem;
        }
        
		var getCurrentPA = jazz.app.context.get;
		var ProjectArea = getCurrentPA(); 
		if(ProjectArea && ProjectArea.itemId)  //local environment
    		return ProjectArea;    	    	
               
        return null;
	},
	
	getURLContext: function(){		 
		   if(dojo.isFunction(this.getWebURL)){
			      return this.getWebURL();
			   };
	},
	
	getFormattedUrl:function(){
		var formattedURL = this.getURLContext()+"/web/console/"+ encodeURIComponent(this.getProjectArea().name);
		if(this.configurationItemId){
			formattedURL += "/" + this.configurationItemId;
		}
		return formattedURL;
	},
		
	isRemote: function(){
		if(this.site && this.site.viewletAdapter)
			return !this.site.viewletAdapter.local;
	},
	
	/**
	 * Determine if the viewlet selected project area context is the same as the QM current project area
	 */
    projectIsAppCurrent: function() {
        if (this.isRemote()) {
            return false;
        }
        var isCurrent = false;
        var currentProjectArea = getCurrentPA();
        var viewletprojectArea = this.getProjectArea();
        if (currentProjectArea && viewletprojectArea) {
            isCurrent = currentProjectArea.itemId == viewletprojectArea.itemId;
        }
        return isCurrent;
    },
    
   //  Make sure uninitialize doesn't call destroy back again and result in recursive destroys
   destroy: function() {
		if(this.isDestroyed() == false)
			this.inherited(arguments);
	},
  
	// Destroy all of the widgets used by this widget.
	uninitialize: function() {
		if(this.domNode != null)
			com.ibm.asq.common.web.ui.internal.util.Utility.destroyWidgetsFromNode(this.domNode);  
	},
	
	handleRefreshWithDefaultConfig: function(succesCallback, errorCallback){
		var viewletprojectArea = this.getProjectArea();
		if(!viewletprojectArea){
			if(errorCallback){
				errorCallback();
			}
			return;
		}
		var currentPAItemId = "";
		var currentPA = getCurrentPA();
		if(currentPA){
			currentPAItemId = currentPA.itemId;
		}
		
		this.configurationItemId = "";
		if(!currentPAItemId || currentPAItemId != viewletprojectArea.itemId){
			var self = this;
			var handler = {
				success: function(config) {
					if(config && config.length > 0){
						self.configurationItemId = config[0].itemId;						
					}
					if(succesCallback){
						succesCallback();
					}
				},
				failure: function(e){
					if(e && errorCallback){
						errorCallback(e);
					}
				}
			};
	        var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
			ConfigurationManagementClient.getDefaultConfiguration(responseHandler, viewletprojectArea.itemId);
		}else{
			if(succesCallback){
				succesCallback();
			}
		}
	}
	
});

})();
