/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.navigator.SavedQueriesModel");

dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.AbstractModel");

(function() {
var NodeTypes = com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes;
var FiltersQueriesClient = com.ibm.asq.common.web.client.internal.FiltersQueriesClient;
var folderIconClass = "folder-icon";
var sorter = function(q1, q2){
    var name1 = q1.label ? q1.label.toLowerCase() : q1.label;
    var name2 = q2.label ? q2.label.toLowerCase() : q2.label;
    return (name1 > name2) ? 1 : (name1 == name2) ? 0 : -1;
};


dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.navigator.SavedQueriesModel",
              com.ibm.asq.common.web.ui.internal.widgets.navigator.AbstractModel,
{   
    savedQueries: null,
    includePersonalQueries: true,
    
    // provider method to return the saved queries
    savedQueriesValuesProvider: function(item, onComplete) {
        // must be implemented by consumer
    },
    
    // dijit.tree.Model  API
    getRoot: function(/*function(items)*/ onComplete){
        if(!this.root) {
            if (this.savedQueries) {
                // saved queries are already initialized
                var scopesFoldersItems = this._makeScopesFoldersItems(this.savedQueries);
                this.root = {
                    type: NodeTypes.ROOT_TYPE,
                    label: "",
                    itemId: FiltersQueriesClient.MODELROOT,
                    children: scopesFoldersItems
                };
            } else {
                // saved queries are not yet initialized
                this.root = {
                    type: NodeTypes.ROOT_TYPE,
                    label: "",
                    itemId: FiltersQueriesClient.MODELROOT,
                    children: []
                };
                var handler = dojo.hitch(this, function(allQueries){
                    this.root.children = this._makeScopesFoldersItems(allQueries);
                    onComplete(this.root);
                });
                this.savedQueriesValuesProvider(this.root, handler);
                return;
            }
        }
        onComplete(this.root);
    },

    // create the top level folder nodes which for now should be just My Queries and Shared Queries
    _makeScopesFoldersItems: function(allSavedQueries) {
        this.savedQueries = allSavedQueries;
        var scopeItems = [];
        for (var scopeKey in this.savedQueries) {
            var queryScope = this.savedQueries[scopeKey];
            // skip personal queries if specified not to include
            if (queryScope.itemId == FiltersQueriesClient.privateScopeId && !this.includePersonalQueries) {
                continue;
            }
            var iconClass = folderIconClass;
            if (queryScope.itemId == FiltersQueriesClient.privateScopeId) {
                iconClass = "saved-queries-icon-personal-queries";
            } else if (queryScope.itemId == FiltersQueriesClient.publicScopeId) {
                iconClass = "saved-queries-icon-shared-queries";
            }
            var scopeItem = {
                type: NodeTypes.SAVED_QUERY_SCOPE_CONTAINER_TYPE,
                itemId: queryScope.itemId,
                label: queryScope.label,
                iconClass: iconClass,
                children: null // item children initialized upon expanding this tree node
            };
            if (queryScope.folders) {
                scopeItem.folders = queryScope.folders;  // children folders of the query object
            }
            if (queryScope.queries) {
                scopeItem.queries = queryScope.queries;  // children queries of the query object
            }
            scopeItems.push(scopeItem);
        }
        return scopeItems;
    },
    
    
    // dijit.tree.Model  API
    mayHaveChildren: function(/*dojo.data.Item*/ item){
        if (item.children) {
            // item children is already initialized
            return Boolean(item.children && item.children.length > 0);
        } else {
            // children is not initialized, check if the query object has children folders or queries
            return Boolean(item.folders || item.queries);
        }
    },
   
    // dijit.tree.Model  API
    getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
        // parent has no children
        if (!this.mayHaveChildren(parentItem)) {
            onComplete([]);
            return;
        }
        var children = parentItem.children;
        if (!children) {
            // initialize and cache the children
            children = parentItem.children = this.makeChildren(parentItem);
        }
        onComplete(children);
    },

    // dijit.tree.Model  API
    newItem: function(item, parent){
        var wrapper = {
            itemId: item.itemId,
            label: item.label,
            parent: parent
        };
        
        if (item.queries) {
            // item contains queries i.e. this is a folder
            wrapper.type = NodeTypes.SAVED_QUERY_CONTAINER_TYPE;
            wrapper.iconClass = folderIconClass;
            wrapper.queries = item.queries;
            wrapper.children = null;

        } else {
            wrapper.type = NodeTypes.SAVED_QUERY_TYPE;
            wrapper.iconClass = "saved-queries-icon-none";
            wrapper.query = item.query;
            wrapper.children = [];
            if (item.query.inMenu) {
                wrapper.stateIconClass = "saved-queries-icon-in-menu";
            }
        }
        return wrapper;
    },
    
    makeChildren: function(parent, /* not used */items) {
        var children = [];
        var folders = [];
        var queries = [];

        if (parent.folders) {
            for (var key in parent.folders) {
                folders.push(this.newItem(parent.folders[key], parent));
            }
        }
        if (parent.queries) {
            for (var key in parent.queries) {
                queries.push(this.newItem(parent.queries[key], parent));
            }
        }
        children = folders.sort(sorter).concat(queries.sort(sorter));
        return children; //wrapper;
    }
});

})();