/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rdm.web.util.domutils");

dojo.require("dojo.DeferredList");
dojo.require("dojo.string");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.rdm.web.editor.EditorConstants");
dojo.require("com.ibm.rdm.web.common.Constants");

var EditorConstants = com.ibm.rdm.web.editor.EditorConstants;

var TAGS = ['p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'div', 'ul', 'ol',
'dl', 'pre', 'hr', 'blockquote', 'address', 'fieldset', 'table',
'form', 'a', 'br', 'span', 'bdo', 'map', 'object', 'img', 'u',
'tt', 'i', 'b', 'big', 'small', 'em', 'strong', 'dfn', 'code',
'q', 'samp', 'kbd', 'var', 'cite', 'abbr', 'acronym', 'sub', 'sup',
'input', 'select', 'textarea', 'label', 'button', 'ins', 'del'];

com.ibm.rdm.web.util.domutils = {
	
	createTr: function(trClass, childNode) {
		var tr = document.createElement("tr");
		if (trClass) {
			tr.setAttribute("class", trClass);
		}
		if (childNode) {
			tr.appendChild(childNode);
		}
		return tr;
	},

	createTd: function(tdClass, childNode) {
		var td = document.createElement("td");
		if (tdClass) {
			td.setAttribute("class", tdClass);
		}
		if (childNode) {
			td.appendChild(childNode);
		}
		return td;
	},
	
	createTrTd: function(trClass, tdClass, childNode) {
		var td = this.createTd(tdClass, childNode);
		var tr = this.createTr(trClass, td);
		return tr;
	},

	createSpannedTextNode: function(spanClassAttr, text) {
		var span = document.createElement("span");
		span.className = spanClassAttr;
		text = document.createTextNode(text);
		span.appendChild(text);
		return span;
	},

	createAnchor: function(label, /*String?*/href, /*String?*/cssClassName) {
		var link = document.createElement("a");
		link.setAttribute("target", "_blank");
		if (href) {
			link.setAttribute("href", href);
		}
		if (cssClassName) {
			link.className = cssClassName;
		}
		link.appendChild(document.createTextNode(label));
		return link;
	},
	
	escapeQuotes: function(text) {
		if (!text) {
			return "";
		}
		var encodedText = text.toString();
		encodedText = encodedText.replace(/"/g, '\\"');
		return encodedText;
	},
	
	getXMLEncodedText: function(text) {
			if (text == null || text == undefined) {
				return "";
			}
			var encodedText = text.toString();
			encodedText = encodedText.replace(/&/g, "&amp;");
			encodedText = encodedText.replace(/</g, "&lt;");
			encodedText = encodedText.replace(/>/g, "&gt;");
			encodedText = encodedText.replace(/"/g, "&quot;");
			encodedText = encodedText.replace(/'/g, "&#039;");
		
		// Valid characters to include in XML document, from http://www.w3.org/TR/REC-xml/#NT-Char
		//
		// x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]
		
		// Remove characters that are invalid to be used in XML.  Javascript represents values in the range
		// [#x10000-#x10FFFF] as surrogate pairs, so we allow through values in the ranges
		// D800-DBFF (high surrogate) and DC00-DFFF (low surrogate), so surrogate pairs representing code points
		// U+10000...U+10FFFF are also passed through without change.  This does leave us open to the
		// slight possibility that invalid surrogate pair values will be passed (a high or low surrogate in isolation or
		// in reversed order), but it is unlikely that such sequences would ever get this far, since for that to happen they
		// would have somehow had to get into a javascript string first.
		encodedText = encodedText.replace(/[^\u0009\u000A\u000D\u0020-\uFFFD]/g, "");
		return encodedText;
	},
	
	getXMLDecodedText: function(text) {
		if (!text) {
			return "";
		}
		var decodedText = text.toString();
		decodedText = decodedText.replace(/&lt;/g, "<");
		decodedText = decodedText.replace(/&gt;/g, ">");
		decodedText = decodedText.replace(/&quot;/g, "\"");
		decodedText = decodedText.replace(/&#039;/g, "\'");
		// Need to decode the escaped '&' last otherwise we may inadvertently
		// decode characters that were already encoded in the original text.
		// eg: if the original text contained the escaped " as &quot; this
		// encoder would produce &amp;quot; and decoding would the &amp; first
		// would yield &quot; which will be matched by a subsequent expression
		// and decoded again.
		decodedText = decodedText.replace(/&amp;/g, "&");
		return decodedText;
	},
	
	getHTMLEncodedText: function(text) {
		if (!text) {
			return "";
		}
		var encodedText = text;
		var pre = document.createElement("pre");
		// use a text area so that linebreaks are preserved
		var textNode = document.createElement("textarea");
		if (dojo.isIE) {textNode.innerText = encodedText;}
		else {textNode.textContent = encodedText;}
		pre.appendChild(textNode);
		encodedText = textNode.innerHTML;
		dojo.destroy(pre);
		dojo.destroy(textNode);
		delete pre;
		delete textNode;
		return encodedText;
	},

	getElementsByTagNameNS: function(domNode, unqualifiedTagName, namespace) {
		
		// This stub method is replaced by a browser-specific function during load of
		// this module.
				
		// summary:
		//		E.g. assuming you've got a domNode like this ...
		//		  <entry>
		//		    ...
		//		    <rdf:Description
		//		        xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
		//		        xmlns:dcterms="http://purl.org/dc/terms/"
		//		        rdf:about="/jazz/resources/rdm/resources/res.media_1">
		//		    ...
		//		    </rdf:Description>
		//		    ...
		//		  </entry>
		//		You can use:
		//		    com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
		//		        atomEntryNode, "Description",
		//		        "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
		//		to get the "rdf:Description" nodes.
		//
		// All three parameters must be provided.  Do no use this method to read
		// elements that do not have namespaces formally defined in the XML.
		//
		// Unlike an earlier implementation, there is no need for a separate variant
		// for reading elements that have a local override of the default namespace.
		// The full namespace URL (not the prefix) is used for parsing, so the
		// namespace of each node is taken into account, regardless of where in the
		// document the namespace is declared.
		//
		// domNode:		A node to start searching from.
		// unqualifiedTagName: The "local" part of the element name.
		// namespace:	The full namespace URI for elements.
		// returns:		An array of matching elements, or an empty array if there are
		//				no matches.
	},

	getSingleElementByTagNameNS: function(domNode, unqualifiedTagName, namespace) {
		
		// This stub method is replaced by a browser-specific function during load of
		// this module.
						
		// summary:
		//		Find a single domNode with a given tagName and namespace.
		//		Note that on IE in particular, this method mat be somewhat inefficient,
		//		especially if there is quite a large tree below domNode.
		// domNode:
		//		A root node to search within.
		// unqualifiedTagName:
		//		Simple tag string, without any namespace prefix.
		// namespace:
		//		The namespace for the element.  This is the full namespace identifier
		//		("http://purl.org/dc/terms/", NOT "dcterms").
		// returns:
		//		The first node below domNode that matches the given unqualifiedTagName
		//		and namespace.  null or undefined is returned if no matching element
		//		is found.

	},
	
	getAttributeNS: function(domNode, unqualifiedAttributeName, namespace) {
		
		// This stub method is replaced by a browser-specific function during load of
		// this module.
		
		// summary:
		//		E.g. assuming you've got a domNode like this ...
		//		    <rdf:Description
		//		        xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
		//		        xmlns:dcterms="http://purl.org/dc/terms/"
		//		        rdf:about="/jazz/resources/rdm/resources/res.media_1">
		//		    ...
		//		    </rdf:Description>
		//		You can use:
		//		    com.ibm.rdm.web.util.domutils.getAttributeNS(
		//		        descriptionNode, "about",
		//		        "http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
		//		to get at the "rdf:about" attribute value:
		//		        "/jazz/resources/rdm/resources/res.media_1"
	},
	
	
	getEntryBindingValueByName: function(entryNode, bindingNameAttribute) {
		// summary:
		//		Gets the plain text for a stored procedure result binding value.
		// returns:
		//		Plain text of the value contained in the binding.
		var bindingNodes = entryNode.getElementsByTagName("binding");
		for (var i=0; i<bindingNodes.length; i++) {
			var bindingAttribute = bindingNodes[i].getAttribute("name");
			if (bindingAttribute == bindingNameAttribute) {
				if (dojo.isIE && dojo.isIE < 9) {
					return bindingNodes[i].text;
				} else {
					return bindingNodes[i].textContent;
				}
			}
		}
		return "";
	},
	
	getEntryBindingValueByNameDecoded: function(entryNode, bindingNameAttribute) {
		return this.getXMLDecodedText(this.getEntryBindingValueByName(entryNode, bindingNameAttribute));
	},
	
	getXmlEntryBindingValueByName: function(entryNode, bindingNameAttribute) {
		// summary:
		//		Gets the XML text for a stored procedure result binding value.
		// returns:
		//		An XML fragment that is the text of the value with the
		//		given binding name.  Empty string returned if the binding is
		//		not found.
		var bindingNodes = entryNode.getElementsByTagName("binding");
		for (var i=0; i<bindingNodes.length; i++) {
			var bindingNode = bindingNodes[i];
			var bindingAttribute = bindingNode.getAttribute("name");
			if (bindingAttribute == bindingNameAttribute) {
				// Content is wrapped in <literal> tags so get inside them
				var literals = bindingNode.getElementsByTagName("literal");
				var containerNode = (literals && literals.length > 0) ? literals[0] : bindingNode;
				return this.getXmlFromNode(containerNode);
			}
		}
		return "";
	},
	
		
	//////////////////////////////////////////////////////////////////////////////////////////
	// Removes all of the nodes in a given element
	removeChildNodes: function(element) {
		while( element.childNodes && element.childNodes.length > 0 ) {
			element.removeChild( element.firstChild );
		}
	},
	
	//Destroys all the child nodes of a given element
	destroyChildNodes: function(element) {
		while( element.childNodes && element.childNodes.length > 0 ) {
			dojo.destroy(element.firstChild);
		}
	},

	getCssPropertyValue: function(node, cssPropertyName) {
		// summary:
		//		Gets a CSS property value for a particular DOM node.
		// example:
		//		In CSS we might have the following defined:
		//		|.com-ibm-rdm-web-header .bannerRight {
		//		|    "position: absolute;"
		//		|...
		//		In javascript we could get access to this value via the following
		//		(in this example, rightDiv has CSS class "bannerRight"):
		//		|var rightDiv = dojo.byId("header-banner-right");
		//		|var position = domutils.getCssPropertyValue(rightDiv, "position"));
		//		The end result would be that position would contain the string
		//		value "absolute".
		var value = null;
		var convertToCamelCase = function (cssProperty){
			// e.g. convertToCamelCase("margin-top") ==> "marginTop"
			for (
				var exp = /-([a-z])/;
				exp.test(cssProperty);
				cssProperty = cssProperty.replace(exp, RegExp.$1.toUpperCase())
			){}
			return cssProperty;
		};
		if (dojo.isIE) {
			var domProp = convertToCamelCase(cssPropertyName);
			value = node.currentStyle[domProp];
		} else {
			var css = document.defaultView.getComputedStyle(node, "");
			value = css.getPropertyValue(cssPropertyName);
		}
		return value;
	},

	getCssIntPropertyValue: function(node, cssPropertyName, /*Integer?*/defaultValue) {
		// summary:
		//		Gets a CSS Integer property value for a particular DOM node.
		// returns:
		//		An integer value for the passed CSS property cssPropertyName in
		//		the passed node.  This returned value MAY be NaN, unless
		//		defaultValue is passed, in which case it will return that value
		//		instead of NaN if an appropriate integer value cannot be found.
		// example:
		//		In CSS we might have the following defined:
		//		|.com-ibm-rdm-web-header .bannerRight {
		//		|    "max-width: 608px;"
		//		|...
		//		In javascript we could get access to this value via the following
		//		(in this example, rightDiv has CSS class "bannerRight"):
		//		|var rightDiv = dojo.byId("header-banner-right");
		//		|var maxWidth = domutils.getCssIntPropertyValue(rightDiv, "max-width");
		//		The end result would be that maxWidth would contain the int
		//		value 608.
		var value = this.getCssPropertyValue(node, cssPropertyName);
		// Note by SPC: I experimented to see what would happen if "width" was NOT
		// specified in the CSS ... in this case if we called getCssPropertyValue
		// for the "width" as the 'cssPropertyName' the 'value' [from previous
		// code line] would be set to "auto" for IE.  In Firefox, however, it would
		// be set to some reasonable value (like "86px), but the MDC shows that it
		// might be set to something like "" depending on which property we were
		// asking for.  Therefore calling parseInt in either case would return NaN.
		var valueAsInt = parseInt(value, 10);
		return ((defaultValue && isNaN(valueAsInt))? defaultValue: valueAsInt);
	},
	
	createHourGlassCursor: function() {
		// summary: Create an hourglass cursor
		window.document.body.style.cursor = 'wait';
	},
	
	createDefaultCursor: function() {
		// summary: set the default cursor
		window.document.body.style.cursor = 'default';
	},


	cleanHTMLString: function (/*String*/html, /*Object?*/ args) {
		// summary:
		//		Function that returns a string containing
		//		only the text from the supplied html string.
		// args:
		//		Optional parameter that can contain values for:
		//		truncateTo - defines the length of the returned string.  Optional.
		//		minTruncateTo -	if supplied, an attempt is made to truncate the
		//			string at a word break somewhere between minTruncateTo
		//			and truncateTo. Must be less than truncateTo. Optional.
		//		separator  - a char to place in between the text of each node.  Optional.
		//		truncationSubstitution - a string containg a message into which the string
		//			is substituted if it is truncated.  Typicaly something
		//			like ""${0}..." to append an ellipsis.  Optional.
		//		    each of these properties is also optional.
		// example:
		//		If str were set to:
		//			<table border="1" cellpadding="1" cellspacing="1" style="width: 200px;">
		//				<tbody>
		//					<tr>
		//						<td>  once</td>
		//						<td>  twice</td>
		//					</tr>
		//					<tr>
		//						<td>  three</td>
		//						<td>  times</td>
		//					</tr>
		//					<tr>
		//						<td>  a</td>
		//						<td>  lady</td>
		//					</tr>
		//				</tbody>
		//			</table>
		//		and you invoked this function using:
		//		    leanHTMLString(str, {separator: " ", truncateTo: 10, trim: true});
		//		it would result in returning:
		//		    "once twice"

		var separator = "";
		if (args && args.separator) {
			separator = args.separator;
		}

		// put the supplied HTML into a DOM node
		var wrapper = dojo.create("div", {innerHTML: html}),
			cleanedString = "",
			keepWalking = true,
			walkDOM = function(node, func) {
				// Sumamry:
				//   This helper function recursively walks
				//   over the DOM node and invokes "func" for
				//   each child, until func returns false.
				if (func(node)) {
					node = node.firstChild;
					while (node && keepWalking) {
						walkDOM(node, func);
						node = node.nextSibling;
					}
				} else {
					keepWalking = false;
				}
			};
		// iterate over the DOM node, supplying a function
		// to do the cleaning...
		walkDOM(wrapper, function(node){
			if (node.nodeType === Node.TEXT_NODE) {
				var trimmed = (dojo.trim(node.nodeValue));
		
				if (args) {
					// keep going until we reach the desired length, but always go at least one
					// character too far so that we get further truncation and the trucation ellipsis added below.
					if (args.truncateTo && cleanedString.length <= args.truncateTo) {
						if (trimmed !== "") {
							cleanedString += (trimmed + separator);
						}
						return true;
					} else {
						return false;
					}
				} else {
					if (trimmed !== "") {
						cleanedString += (trimmed + separator);
					}
					return true;
				}
			}
		
			return !(args && args.truncateTo && (cleanedString.length > args.truncateTo));
		});
		
		// Although the above loop terminated when it exceeded the desired length, it may still leave the
		// result longer than desired, so further truncate the string if asked to, and apply the substitution
		// if necessary.
		if (args && args.truncateTo && (args.truncateTo < cleanedString.length)) {
			cleanedString = cleanedString.substring(0, args.truncateTo);
			if (args.minTruncateTo) {
				// further truncate to a word break if possible without going below minTruncateTo
				var lastSpaceIndex = cleanedString.lastIndexOf(" ");
				if (lastSpaceIndex >= args.minTruncateTo) {
					cleanedString = cleanedString.substring(0, lastSpaceIndex);
				}
			}
			if (args.truncationSubstitution) {
				// apply the ellipsis substiution.
				cleanedString = dojo.string.substitute(args.truncationSubstitution, [cleanedString]);
			}
		}
		return cleanedString;
	},
	
	getPseudoGeneratedText: function(/*String or Node*/ source){
		var wrapper = source,
			args = {
				truncateTo: com.ibm.rdm.web.common.Constants.GENERATED_TITLE_TRUNCATION_LENGTH,
				separator: " "
			};
		if(typeof source === 'string'){
			wrapper = dojo.create("div", {innerHTML: source});
		}
		var firstP = dojo.query('p', wrapper)[0];
		if(firstP) {
			return dojo.string.trim(this.cleanHTMLString(firstP.innerHTML, args));
		}
		// WI#: 82714 Edgecase of no paragraph tag... only possible via API/OSLC (CKE forces a para on save)
		return dojo.string.trim(this.cleanHTMLString(wrapper.innerHTML, args));
	},

	findAncestorByClass: function (node, className) {
		while (node) {
			var parentNode = node.parentNode;
			if (parentNode) {
				if (dojo.hasClass(parentNode, className)) {
					return parentNode;
				} else {
					node = parentNode;
				}
			} else {
				return null;
			}
		}
		return null;
	},

	findAncestorByClassInclusive: function (node, className, limitNode) {
		// summary:
		//		Finds the closest ancestor of a given node that has the given class.
		//		The result may be the supplied node, if it has the class itself.
		// node:
		//		A node to search from.
		// className:
		//		The class to search for.
		// limitNode:
		//		Optional.  If provided, the search does not extend higher than this node.
		// returns:
		//		node, or its closes ancestor that has the className.  Returns undefined if
		//		no matching ancestor is found.
		while(node) {
			if (dojo.hasClass(node, className)) {
				return node;
			}
			if (limitNode && (limitNode === node)) {
				break;
			}
			node = node.parentNode;
		}
		// no match found.  return undefined.
	},
   
	findAncestorByAttr: function (/* Element */node, /* String */attr, /*?DOMNode*/ limitNode) {
		// summary:
		//		give a node (1st argument) find its closest ancestor
		//		that has an attribute "attr".
		// limitNode:
		//		Optional.  If provided, the search does not extend higher than this node.
		try {
			while (node) {
				if (	(dojo.isIE || (typeof node.getAttributeNode === 'function'))
						&& dojo.attr(node, attr) !== null) {
					return node;
				}
				if (limitNode && (limitNode === node)) {
					break;
				}
				node = node.parentNode;
			}
		} catch (E) {
			// I don't know what causes this on IE, but one of the
			// DOM elements doesn't like dojo.attr on occasion.
		}
		// no match found.  return undefined.
	},

	findAncestorByTag: function (/* Element */node, /* String */tag, /* ?Element */limitNode) {
		// summary:
		//		Given a node (1st argument) find its closest ancestor
		//		with a given tag.
		// limitNode:
		//		If provided, the search does not extend higher up the DOM that this node.
		this.findAncestorByTags(node, [tag], limitNode);
	},
	
	findAncestorByTags: function (/* Element */node, /* array */tags, /* ?Element */limitNode) {
		// summary:
		//		Given a node (1st argument) find its closest ancestor
		//		with a given tags.
		// limitNode:
		//		If provided, the search does not extend higher up the DOM that this node.
		while (node) {
			// look for element type node with matching tag name
			if (node.nodeType === 1 && dojo.indexOf(tags, node.tagName.toLowerCase()) !== -1) {
				return node;
			}
			if (limitNode && (node === limitNode) ) {
				break;
			}
			node = node.parentNode;
		}
	},
	
	changeNodeTag: function(/*Element*/node, /*String*/newTag){
		// summary:
		//		Converts the given node to a new one with the given tag.
		//		All child nodes and attributes are preserved.  If the node
		//		has a parent, the node is replaced on the parent.
		// node:
		//		The source node.
		// newTag:
		//		The tag to be used for the new node.
		// returns:
		//		The new node.
		var newNode = dojo.create(newTag),
			attributes = node.attributes,
			childNode = node.firstChild,
			attribute, i, nextChild;
		
		// copy all child nodes - elements and text nodes.
		while (childNode) {
			nextChild = childNode.nextSibling;
			newNode.appendChild(childNode);
			childNode = nextChild;
		}
		// copy all attributes.
		if (attributes.length) {
			var allAttrs = {};
			for (i = 0; i < attributes.length; i++) {
				attribute = attributes[i];
				allAttrs[attribute.name] = attribute.value;
			}
			dojo.attr(newNode, allAttrs);
		}
		
		// replace node on parent.
		if (node.parentNode) {
			dojo.place(newNode, node, "replace");
		}
		return newNode;
	},
	
	getTextContent: function(/* Element */node) {
		// summary:
		//		Gets the text content of a node - factors out the browser-specifics
		return dojox.xml.parser.textContent(node);
	},
	
	setTextContent: function(/* Element */node, /*String*/text) {
		// summary:
		//		Sets the text content of a node - factors out the browser-specifics
		(dojo.isIE && dojo.isIE < 9) ? node.text = text : node.textContent = text;
	},
	
	getXmlFromNode: function(/* Element */ node) {
		// summary:
		//		Gets the content of an element in the form of an XML fragment string.
		// node:
		//		An element node.
		// returns:
		//		String representation of the element contents.
		var xmlText = "";
		var childNodes = node.childNodes;
		var length = childNodes.length;
		for (var j = 0 ; j < length ; j++) {
			xmlText += dojox.xml.parser.innerXML(childNodes[j]);
		}
		return xmlText;
	},
	
	isDataEncodedImage: function(imageNode) {
		// summary:
		//		Tests whether or not the given image node is encoded as data.
		//		If an image is encoded as data, the src URI will be of the form
		//			data:[<MIME-type>][;charset=<encoding>][;base64],<data>
		return imageNode.src.indexOf("data:image") === 0;
	},
	
	isBase64Image: function(imageNode) {
		// summary:
		//		Tests whether or not the given image node is a base64 image.  FF4 and greater
		//		encodes copied images as base64 in order to support copy/paste of image data.
		//		If an image is encoded as data, the src URI will be of the form
		//			data:[<MIME-type>][;charset=<encoding>][;base64],<data>
		
		if (this.isDataEncodedImage(imageNode)) {
			var index = imageNode.src.indexOf(',');
			var imageInfo = imageNode.src.substring(0, index);
			index = imageInfo.lastIndexOf(';');
			if (index !== -1) {
				var lastSegment = imageInfo.substring(index + 1);
				if (lastSegment === "base64") {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
	},
	
	createImageFixAspect: function(src, maxDimension, imageClass){
		// summary:
		//		Creates a DOM node that is an image tag.  When the image loads
		//		it is resized such that it does not exceed maxDimension in
		//		either height or width, and its aspect ration is maintained.
		// maxDimension:
		//		Maximun height and width to display the image, in pixels.
		// imageClass:
		//		A css class name to apply to the image dom node.
		var img = document.createElement("img");
		if (imageClass) {
			dojo.addClass(img, imageClass);
		}
		// set an initial size, so image cannot appear ridiculously big if there
		// is a delay in handling the onload functions.
		dojo.style(img, "width", maxDimension+"px");
		dojo.style(img, "height", maxDimension+"px");

		var self = this;
		
		// a handler to set the size of the image once it has loaded
		var onImageLoad = function(event) {
			if (dojo.isIE) {
				// in IE we don't have access to .naturalWidth and .naturalHeight, so
				// create a new image not on display, and when it loads, find its size.
				// This load should be fast as it will be from browser cache.
				var sizerImage = new Image();
				dojo.connect(sizerImage, "load", function(){
					var naturalWidth = sizerImage.width;
					var naturalHeight = sizerImage.height;
					self._resizeImage(img, maxDimension, naturalWidth, naturalHeight);
				});
				sizerImage.setAttribute("src", src);
			} else {
				var naturalWidth = img.naturalWidth;
				var naturalHeight = img.naturalHeight;
				self._resizeImage(img, maxDimension, naturalWidth, naturalHeight);
			}
		};
		dojo.connect(img, "load", onImageLoad);
		img.setAttribute("src", src);
		return img;
	},
	
	waitImageLoaded: function(imageNode) {
		// summary:
		//		Gets a deferred that resolves when the given image is loaded.  This is intended to
		//		be used for processing images that have just been added to the DOM, and so are not yet loaded.
		//		Until the image is loaded, its size and position are not known.
		// returns:
		//		A deferred that resolves when the image has been loaded.  The image node is
		//		given as a parameter to the callback.
		var deferred = new dojo.Deferred();
		var connectHandle = dojo.connect(imageNode, "load", function(){
			dojo.disconnect(connectHandle);
			deferred.callback(imageNode);
		});
		return deferred;
	},
	
	waitAllImagesLoaded: function(imageNodes) {
		// summary:
		//		Gets a deferred that resolves when all the given images have been loaded.
		//		This is intended to be used for processing images that have jsut been added to
		//		the DOM, and so are not yet loaded.  Until an image is loaded, its size and
		//		position are not known.  In the case of multiple images, the position of one may
		//		affect others, so it is sometimes necessary to wait for several to load.
		// returns:
		//		A deferred that resolves when the images are loaded.  The parameter to the
		//		callback is an array of images that were loaded.  If there was some error
		//		in waiting for one of the images, that image node is not included in the
		//		array.
		var deferreds = [];
		dojo.forEach(imageNodes, function(image){
			deferreds.push(this.waitImageLoaded(image));
		}, this);
		var deferredList = new dojo.DeferredList(deferreds);
		deferredList.addCallback(this, function(results) {
			var images = [];
			for (var i = 0; i < results.length; i++) {
				if (results[i][0]) {
					images.push(results[i][1]);
				}
			}
			return images;
		});
		return deferredList;
	},
	
	
	
	_resizeImage: function(imgNode, maxDimension, naturalWidth, naturalHeight) {
		// summary:
		//		sets the width and height of a node (image), such that it fits in
		//		a square of maxDimension, and maintaining aspect ratio.
		// maxDimension:
		//		The maximum height and/or width to apply.
		// naturalWidth:
		//		The natural width of the image.
		// naturalHeight:
		//		The natural height of the image.
		
		var width = maxDimension;
		var height = maxDimension;
		if (naturalWidth > naturalHeight) {
			// landscape
			height = Math.round( maxDimension * (naturalHeight / naturalWidth) );
		} else if (naturalWidth < naturalHeight) {
			// portrait
			width = Math.round( maxDimension * (naturalWidth / naturalHeight) );
		} /* else square */
		
		dojo.style(imgNode, "width", width+"px");
		dojo.style(imgNode, "height", height+"px");
	},

	makeAnchorString: function(href, /* optional */text) {
		// summary:
		//		Build an <a> tag string with the given href and text values.
		// href:
		//		An href value.
		// text:
		//		Text to be added in the anchor tag.  If not provided, the href value is used.
		text = this.getHTMLEncodedText( text ? text : href );
		return '<a href="' + href + '" >' + text + '</a>';
	},
	
	
	truncateRichText: function(htmlString, len, postscript1, postscript2) {
		// summary:
		//		Truncate a rich text string, so that the text content in it is
		//		restricted to a given length.
		// htmlString: String
		//		The html string to be truncated.
		// len: int
		//		The desired text content length of the string after truncation.
		// postscript1: String
		//		Optional. A string to add to the end of the truncate text - immediately after the last character
		//		and before the closing HTML tags.  This will tend to be styled similar to the end of the text.
		// postscript2: String
		//		Optional.  A second postscript to add at the end of the truncated text.  This is added after some
		//		inline tags are closed, so can be styled independently of the text (although it is still inside
		//		some tags, so care is required.  May contain inline markup tags if required.
		
		var ind = 0; //index from the front
		var l = len; //index from the end
		var queue = []; // the front part of the result
		var stack = []; // the end part of the result
		var lastTextCharacterIndex = 0; // One beyond the last text character in queue
		var p;
		while (ind < htmlString.length){
			if (htmlString.charAt(ind) == '<'){
				if (htmlString.charAt(ind + 1) != '/'){
					// it is an open tag
					p = htmlString.indexOf(">", ind);
					var openTag = htmlString.substring(ind,p+1);
					ind = p;
					var tagName = openTag.match(/\w+/);
					queue.push(openTag);
					if (htmlString.charAt(p-1) != '/'){
						// handle tags such as <br/>
						stack.push("</" + tagName+">");
					}
				}else{
					// it is a close tag
					p = htmlString.indexOf(">", ind);
					var closeTag = htmlString.substring(ind, p+1);
					ind = p;
					var topEle = stack.pop();
					if (topEle == closeTag){
						queue.push(topEle);
					}
				}
			}else{
				// it is a normal text character.
				var textChar = htmlString.charAt(ind);
				queue.push(textChar);
				if (/\S/.test(textChar)) {
					// non-whitespace character
					lastTextCharacterIndex = queue.length;
				}
				l--;
				if (l==0) {
					break;
				}
			}
			ind++;
		}
		// generate the output, inserting the postscript after the last characters  If the
		// text has not been truncated, there will be tags at the end of the queue, following
		// the last text characters.  If the text has been truncated, there will be no trailing
		// tags in the queue, and all trailing tag closures are in the stack.
		
		var trailingTags = queue.splice(lastTextCharacterIndex, (queue.length - lastTextCharacterIndex)); // removes from queue, and returns removed section.
		var resultHTML = queue.join("");
		
		if (postscript1) {
			resultHTML += postscript1;
		}
		
		// get all the trailing tags in a single array, so we can work out whether to close any tags before
		// postscript2.  Stack is in reverse of the order we need for output so sort that out as well.
		stack.reverse();
		trailingTags = trailingTags.concat(stack);
		
		// Some trailing tags can mess up the style of the postscript, so close them before the postscript.  This only
		// handles some tags.  Other text features that have been added by styles rather than markup have to be handled
		// by our caller applying styles to the postscript.
		var finished = false;
		if (postscript2) {
			while (!finished && trailingTags.length > 0) {
				var tag = trailingTags[0];
				switch(tag) {
					case "</i>":
					case "</u>":
					case "</del>":
					case "</sup>":
					case "</sub>":
					case "</span>":
						// An inline tag that we should close to end styling before the postscript
						resultHTML += tag;
						trailingTags.splice(0, 1);
						break;
					default:
						// some other tag - probably a block tag that we should close after the postscript
						finished = true;
						break;
				}
			}

			resultHTML += postscript2;
		}
		resultHTML += trailingTags.join("");
		return resultHTML;
	},
	
	getFlattenedText: function(htmlString) {
		// summary:
		//		Get just the plain text of a string without markup.  Note that, unlike cleanHTMLString above, this
		//		method does not insert a separator between the text of each node, so may not be an appropriate way
		//		to get plain text for html that includes more than one block.  However, it is simpler - it does not
		//		iterate over the DOM in JavaScript, so should generally be faster.
		// htmlString: string
		//		HTML string containing text to be flattened.
		// returns:
		//		The text content without any markup.
		
		//create a flattened version of the text
		var div = document.createElement("DIV");
		div.innerHTML = htmlString;
		var flattenedText = dojo.isIE ? (div.innerText) : (div.textContent);
		flattenedText = dojo.string.trim(flattenedText);
		return flattenedText;
	},
	
	getPrimaryHtmlContent: function(htmlText) {
		var modifiedHtmlText = htmlText;
		if (dojo.isIE) {
			// workaround for 30416, on IE <br></br> pattern is changed to
			// </br></br> when accessing inner html
			modifiedHtmlText = modifiedHtmlText.replace(/<br>/g, "");
		}
		var htmlDiv = document.createElement("div");
		htmlDiv.innerHTML = modifiedHtmlText;
		if (htmlDiv.childNodes.length > 0) {
			var child = htmlDiv.firstChild;
			var xhtmlDivElement = null;
			if (child.attributes) {
				xhtmlDivElement = child.getAttribute("xmlns");
			}
			if (xhtmlDivElement) {
				// remove the xhtml div that wraps the html text
				var newDiv = document.createElement("div");
				var nextSibling = htmlDiv.firstChild.firstChild;
				while (nextSibling) {
					var nextNode = nextSibling;
					nextSibling = nextSibling.nextSibling;
					newDiv.appendChild(nextNode);
				}
				htmlDiv = newDiv;
			}
		}

		modifiedHtmlText = htmlDiv.innerHTML;
		dojo.destroy(htmlDiv);
		return modifiedHtmlText;
	},
	
	wrapForXHTMLValidation: function(htmlText) {
		var contents = '<div xmlns="' + com.ibm.rdm.web.util.queryutils.XHTML_NAMESPACE + '">\n' + htmlText + '\n</div>';
		return contents;
	},
	
	lowerCaseTags: function(htmlText) {
		// summary:
		//		Ensures any (of the defined) html tags are lowercase.
		//		Required for xhtml validation. IE8 is really the main culprit, but it is fairly
		//		safe to use this function on any html with escaped innerhtml.
		// htmlText:
		//		An HTML string.
		// returns:
		//		An HTML string with lowercased tag names.
		var i = 0,
			max = TAGS.length,
			tag;
		for(i; i < max; i++) {
			tag = TAGS[i];
			htmlText = htmlText.replace(new RegExp("<" + tag.toUpperCase(), 'g'), "<" + tag)
				.replace(new RegExp("/" + tag.toUpperCase() + ">", 'g'), "/" + tag + ">");
		}
		return htmlText;
	},
	
	convertLinksToAbsolute: function(htmlText, relativeBaseUrl) {
		// summary:
		//		Converts any relative links in the given html to absolute links.
		// htmlText:
		//		An HTML fragment (need not have a single root node).
		// relativeBaseUrl:
		//		A URL that any relative URLs are relative to.  e.g. for links in comments
		//		this is the URL of the comments service.
		// returns:
		//		An HTML fragment similar to htmlText, but with any relative URL links in it
		//		converted to absolute values.
		
		var baseUrl;
		var wrapperNode = dojo.create('div');
		wrapperNode.innerHTML = htmlText;
		
		dojo.query('a[href]', wrapperNode).forEach(function(linkNode){
			var linkHref = linkNode.getAttribute("href");
			// If it is a relative link, resolve it relative to the base that our caller told us.
			// e.g. baseUrl = https://server:port/rm/comments/
			//		href = ../resource/12345
			// 		=> https://server:port/rm/comments/../resource/12345
			//		=> https://server:port/rm/resource/12345
			if (linkHref.indexOf(':') < 0) {
				// relative link
				// ensure we have a baseURL that ends in a / character.  Only need to do this first time through.
				if (!baseUrl) {
					baseUrl = (relativeBaseUrl.charAt(relativeBaseUrl.length-1) != '/' ) ?
						relativeBaseUrl += '/' : relativeBaseUrl;
				}
				var url = new dojo._Url(baseUrl, linkHref);
				var absHref = url.toString();
				linkNode.setAttribute("href", absHref);
			}
		});
		return wrapperNode.innerHTML;
	},
	
	// Gets the next element sibling, from a node.
	getNextElementSibling: function(node) {
		if (dojo.isIE && dojo.isIE < 9) {
			do {
				node = node.nextSibling;
			} while (node && node.nodeType != 1);
			return node;
		} else {
			return node.nextElementSibling;
		}
	},
	
	// Gets the previous element sibling, from a node.
	getPreviousElementSibling: function(node) {
		if (dojo.isIE && dojo.isIE < 9) {
			do {
				node = node.previousSibling;
			} while (node && node.nodeType != 1);
			return node;
		} else {
			return node.previousElementSibling;
		}
	},
	
	getChildrenElements: function(node) {
		var nodes = [];
		dojo.forEach(node.childNodes, function(childNode) {
			if (childNode.nodeType === 1) {
				nodes.push(childNode);
			}
		});
		return nodes;
	},
	
	triggerDomEvent: function(node, eventName) {
		// summary:
		//		Triggers a dom event, as if it had been triggered by the user.
		// node:
		//		A dom node on which to trigger the event.
		// eventName:
		//		Name of the event to trigger with the "on" prefix.  e.g. "onclick"
		if (dojo.isIE < 9) {
			node.fireEvent(eventName);
		} else {
			eventName = eventName.slice(2);
			var event = document.createEvent("HTMLEvents");
			event.initEvent(eventName, true, true);
			node.dispatchEvent(event);
		}
	},
	
	triggerKeyPressEvent: function(node, keyCode, keyChar, modifiers) {
		// summary:
		//		Triggers a DOM keypress event on the given node.  It seems that it is not
		//		possible (at least in some browsers) to set keyChar, so we do it using a
		//		syntheticKeyChar property on the event object, and expect our key handlers
		//		to be able to understand.
		// node:
		//		A DOM node on which to trigger the event.
		// keyCode:
		//		The key code integer to include in the event.
		// keyChar:
		//		The keyChar string to include in the event.  If there is no keyChar, use
		//		empty string.
		// modifiers:
		//		Optional. An object of the form:
		//			{
		//				shift:		// Optional, Boolean
		//				ctrl:		// Optional, Boolean
		//				alt:		// Optional, Boolean
		//				meta:		// Optional, Boolean
		//				keyDown:	// Optional, Boolean	Indicates to create keydown event, rather than keypress.
		//			}
		//		Any modifier that is not provided is assumed to be false.
		//
		
		var event, shift = false, ctrl = false, alt = false, meta = false, eventType = "keypress";
		if (modifiers) {
			shift = !! modifiers.shift;
			ctrl = !! modifiers.ctrl;
			alt = !! modifiers.alt;
			meta = !! modifiers.meta;
			if (modifiers.keyDown) {
				eventType = "keydown";
			}
		}
		
		if (dojo.isIE) {
			if (dojo.isIE < 9) {
				// IE 8
				event = document.createEventObject();
				event.keyCode = keyCode;
				event.syntheticKeyChar = keyChar;
				event.shiftKey = shift;
				event.ctrlKey = ctrl;
				event.altKey = alt;
				event.metaKey = meta;
				event.synthetic = true;
				node.fireEvent("on"+eventType, event);
			} else {
				//IE9 or later
				event = document.createEvent("KeyboardEvent");
				Object.defineProperty(event, "keyCode", {
					get: function() {
						return this._keyCode;
						}
					});
				event._keyCode = keyCode;
				event.syntheticKeyChar = keyChar;
				var modifiersArray = [];
				if (shift) {
					modifiersArray.push("Shift");
				}
				if (ctrl) {
					modifiersArray.push("Control");
				}
				if (alt) {
					modifiersArray.push("Alt");
				}
				if (meta) {
					modifiersArray.push("Meta");
				}
				modifiers = modifiersArray.join(" ");
				event.initKeyboardEvent(eventType, true, true, window, "", 0, modifiers, false, "");
				event.synthetic = true;
				node.dispatchEvent(event);
			}
		} else if (dojo.isFF) {
			event = document.createEvent("KeyboardEvent");
			event.initKeyEvent(eventType, true, true, window, ctrl, alt, shift, meta, keyCode, 0);
			event.synthetic = true;
			event.syntheticKeyChar = keyChar;
			node.dispatchEvent(event);
		} else if (dojo.isSafari) {
			// Note by SPC on 9/23/1013:
			// We can't use the Chrome based solution for this as the second line of
			// the following returns true on Chrome and false on Safari:
			//     event = document.createEvent("KeyboardEvent");
	//     Object.getOwnPropertyDescriptor(event, 'keyCode').configurable
			// and therefore an exception is thrown when attempting to use the Chrome
			// base path on Safari.
			// It took me a while to find an alternative, but I eventually stumbled on
			// the following 'createEvent("Events")' based solution by reading the
			// "Summary" section for the following:
			//     https://bugs.dojotoolkit.org/ticket/9528
			event = document.createEvent("Events");
			event.initEvent(eventType, true, true);
			event.keyCode = keyCode;
			event.shiftKey = shift;
			event.metaKey = ctrl;
			event.altKey = alt;
			event.synthetic = true;
			event.syntheticKeyChar = keyChar;
			node.dispatchEvent(event);
		} else {
			// Chrome
			event = document.createEvent("KeyboardEvent");
			Object.defineProperty(event, "keyCode", {
				get: function() {
					return this._keyCode;
				}
			});
			event._keyCode = keyCode;
			event.syntheticKeyChar = keyChar;
			event.initKeyboardEvent(eventType, true, true, window, "", 0, ctrl, alt, shift, meta);
			event.synthetic = true;
			node.dispatchEvent(event);
		}
	},
	
	selectTextAndFocusNode: function(field, start, end) {
		var doSelect = function() {
			field.focus();
			if (field.createTextRange) {
				var selRange = field.createTextRange();
				selRange.collapse(true);
				selRange.moveStart('character', start);
				selRange.moveEnd('character', end-start);
				selRange.select();
			} else if (field.setSelectionRange) {
				field.setSelectionRange(start, end);
			} else if (field.selectionStart) {
				field.selectionStart = start;
				field.selectionEnd = end;
			}
		};
		// Webkit resets carat position as part of focus event,
		// defer our function until after it has been fired
		field.focus(setTimeout(function(){doSelect();},1));
	},

	isParserErrorFoundInDocument: function(doc) {
		// summary:
		//		Return true if and only if the passed doc contains a parser error.
		// description:
		//		Please see the following:
		//		    http://www.w3schools.com/dom/dom_errors_crossbrowser.asp
		//		for an explanation of the tests performed in this function.

		var isParserError = false;
		if (dojo.isIE) {
			// Note: The MSDN documentation implies that doc.parseError should
			// always be present, but we'll be extra cautious an not assume
			// that this is true.
			isParserError = (
				doc.parseError ?
				doc.parseError.errorCode !== 0 :
				false
			);
		}
		else if (dojo.isFF) {
			isParserError = (
				doc.documentElement ?
				doc.documentElement.nodeName === "parsererror" :
				false
			);
		}
		return isParserError;
	},
	
	createMarginLabeledElement: function(/* String|DomNode */label, /* DomNode */content,
				/* ?Integer */marginPad, /* ?Integer */minMargin, /* ?Integer */maxMargin) {
		// summary:
		//		Creates a DOM node that wraps a label and given node, such that the label sits to the left of
		//		the content.  This allows the content to left-align next to the label.
		// label:
		//		A string or DomNode.  If a string is supplied it is wrapped in a span.
		// content:
		//		A content node.  Must be a block display node so that margin can be applied to it.
		// marginPad:
		//		Optional. Clearance between the label and the content.  If not supplied a default is used.
		// minMargin, maxMargin:
		//		Optional.  The minumum and maximum sizes to allow for the margin.
		// returns:
		//		A node that wraps the other nodes and is styled so that the content is labeled.
		if (dojo.isString(label)) {
			var labelTextNode = document.createTextNode(label);
			label = dojo.create('span');
			label.appendChild(labelTextNode); 
		}
		
		var outerNode = dojo.create('div');
		dojo.style(outerNode, 'position', 'relative');
		dojo.style(label, {
			position: 'absolute',
			top: 0,
			left: 0
		});
		outerNode.appendChild(label);
		outerNode.appendChild(content);
		
		// set the sizes in a timeout because we need to wait until our caller
		// adds the returned node to the DOM before reading the sizes.
		window.setTimeout(function(){
			var margin = label.offsetWidth,
				minHeight = label.offsetHeight;
			margin += marginPad || 6;
			if (minMargin && minMargin > margin) {
				margin = minMargin;
			}
			if (maxMargin && maxMargin < margin) {
				margin = maxMargin;
			}
			dojo.style(content, 'marginLeft', margin+'px');
			dojo.style(outerNode, 'minHeight', minHeight+'px');
		}, 0);
		
		return outerNode;
	},
	
	clearSelection: function() {
		if (window.getSelection) {
			var sel = window.getSelection();
			if (sel.empty) {
				sel.empty();
			} else if (sel.removeAllRanges) {
				sel.removeAllRanges();
			}
		} else if (document.selection) {
			document.selection.empty();
		}
	}
};

(function(){
	// We need different parsing functions for the different browsers.  Make the switch once
	// at load time by setting functions in domutils, rather than checking the browser on every parse
	// operation.  So we define the functions first, then wire up the appropriate ones for each browser.
	
	// The following functions are defined for each browser variant:
	//	getElementsByTagNameNS(domNode, unqualifiedTagName, namespace)
	//  getSingleElementByTagNameNS(domNode, unqualifiedTagName, namespace)
	//  getAttributeNS(domNode, unqualifiedAttributeName, namespace)
	
	// summary:
	//		E.g. assuming you've got a domNode like this ...
	//		  <entry>
	//		    ...
	//		    <rdf:Description
	//		        xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	//		        xmlns:dcterms="http://purl.org/dc/terms/"
	//		        rdf:about="/jazz/resources/rdm/resources/res.media_1">
	//		    ...
	//		    </rdf:Description>
	//		    <rdf:Description>
	//		        ...
	//		    </rdf:Description>
	//		    ...
	//		  </entry>
	//		You can use:
	//		    com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
	//		        atomEntryNode, "Description",
	//		        "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	//		to get an array of all Description nodes.
	//
	//		You can use:
	//		    com.ibm.rdm.web.util.domutils.getSingleElementByTagNameNS(
	//		        atomEntryNode, "Description",
	//		        "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	//		to get the first Description node.
	//
	//		You can use:
	//		    com.ibm.rdm.web.util.domutils.getAttributeNS(
	//		        descriptionNode, "about",
	//		        "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	//		to get the rdf:about attribute on a Description node.
	
	// ============== Standard versions - for Firefox, IE9, Webkit ==============
	
	var getElementsByTagNameNS_Standard = function(domNode, unqualifiedTagName, namespace){
		if (dojo.isFunction(domNode.getElementsByTagNameNS)) {
			return domNode.getElementsByTagNameNS(namespace, unqualifiedTagName);
		}
		return [];
	};
	
	var getSingleElementByTagNameNS_Standard = function(domNode, unqualifiedTagName, namespace) {
		if (dojo.isFunction(domNode.getElementsByTagNameNS)) {
			return domNode.getElementsByTagNameNS(namespace, unqualifiedTagName)[0];
		}
	};
	
	var getAttributeNS_Standard = function(domNode, unqualifiedAttributeName, namespace) {
		return domNode ? domNode.getAttributeNS(namespace, unqualifiedAttributeName) : "";
	};


	// ============== IE7 and IE8 versions ==============
	
	// These are versions based on getElementsByTagName('*') followed by a manual filter.
	// Work on IE7 and IE8.  There are slightly fasters alternatives available, but they
	// are simgnificantly more complex.
	
	var getElementsByTagNameNS_Filter = function(domNode, unqualifiedTagName, namespace) {
		// On IE7 use getElementsByTagName('*') then filter the nodes that match.
		var selectedElements = [];
		var nodes = domNode.getElementsByTagName('*');
		var length = nodes.length;
		for (var i = 0 ; i < length ; i++) {
			var node = nodes.item(i);
			// We must check both baseName and localName attributes, since
			// either might exist in different situations in IE9.
			if (node.baseName == unqualifiedTagName
				&& node.namespaceURI == namespace) {
				selectedElements.push(node);
			}
		}
		return selectedElements;
	};

	var getSingleElementByTagNameNS_Filter = function(domNode, unqualifiedTagName, namespace) {
		// On IE7 use getElementsByTagName('*') then search for the first node that matches.
		var nodes = domNode.getElementsByTagName('*');
		var length = nodes.length;
		for (var i = 0 ; i < length ; i++) {
			var node = nodes.item(i);
			if (node.baseName == unqualifiedTagName
				&& node.namespaceURI == namespace) {
				return node;
			}
		}
		return null;
	};
	
	var getAttributeNS_Filter = function(domNode, unqualifiedAttributeName, namespace) {
		if (domNode) {
			var allAttributes = domNode.attributes;
			var index = allAttributes.length;
			while (index--) {
				var attribute = allAttributes[index];
				if (attribute.baseName == unqualifiedAttributeName
					&& attribute.namespaceURI == namespace) {
					return attribute.value;
				}
			}
		}
		return "";
	};
	
	// ============== IE9 versions ==============
	
	// These are versions based on getElementsByTagName('*') followed by a manual filter.
	// These include additional checks for the localName attribute, as
	// in some cases, this is provided in place of the baseName attribute
	// in IE9. These are provided as separate functions, since we don't want
	// to spend cycles in IE7 or IE8 unnecessarily.

	var getElementsByTagNameNS_FilterIE9 = function(domNode, unqualifiedTagName, namespace) {
		var selectedElements = [];
		var nodes = domNode.getElementsByTagName('*');
		var length = nodes.length;
		for (var i = 0 ; i < length ; i++) {
			var node = nodes.item(i);
			// We must check both baseName and localName attributes, since
			// either might exist in different situations in IE9.
			if ((  node.baseName == unqualifiedTagName
				|| node.localName == unqualifiedTagName)
				&& node.namespaceURI == namespace) {
				selectedElements.push(node);
			}
		}
		return selectedElements;
	};

	var getSingleElementByTagNameNS_FilterIE9 = function(domNode, unqualifiedTagName, namespace) {
		var nodes = domNode.getElementsByTagName('*');
		var length = nodes.length;
		for (var i = 0 ; i < length ; i++) {
			var node = nodes.item(i);
			if ((  node.baseName == unqualifiedTagName
				|| node.localName == unqualifiedTagName)
				&& node.namespaceURI == namespace) {
				return node;
			}
		}
		return null;
	};
	
	var getAttributeNS_FilterIE9 = function(domNode, unqualifiedAttributeName, namespace) {
		if (domNode) {
			var allAttributes = domNode.attributes;
			var index = allAttributes.length;
			while (index--) {
				var attribute = allAttributes[index];
				if ((  attribute.baseName == unqualifiedAttributeName
					|| attribute.localName == unqualifiedAttributeName)
					&& attribute.namespaceURI == namespace) {
					return attribute.value;
				}
			}
		}
		return "";
	};
	
	// ============== Wiring up code - executed once during initial load ==============
	// Replaces the getElementsByTagNameNS etc functions in domutils with an appropriate implementation
	// depending on the browser.
	
	// Wire up the appropriate functions for each browser.
	if (dojo.isIE && dojo.isIE < 10) {
		if(dojo.isIE > 8) {
			// IE9
			com.ibm.rdm.web.util.domutils.getElementsByTagNameNS = getElementsByTagNameNS_FilterIE9;
			com.ibm.rdm.web.util.domutils.getSingleElementByTagNameNS = getSingleElementByTagNameNS_FilterIE9;
			com.ibm.rdm.web.util.domutils.getAttributeNS = getAttributeNS_FilterIE9;
		} else {
			// IE7 & IE8
			com.ibm.rdm.web.util.domutils.getElementsByTagNameNS = getElementsByTagNameNS_Filter;
			com.ibm.rdm.web.util.domutils.getSingleElementByTagNameNS = getSingleElementByTagNameNS_Filter;
			com.ibm.rdm.web.util.domutils.getAttributeNS = getAttributeNS_Filter;
		}
	} else {
		// Firefox, Webkit
		com.ibm.rdm.web.util.domutils.getElementsByTagNameNS = getElementsByTagNameNS_Standard;
		com.ibm.rdm.web.util.domutils.getSingleElementByTagNameNS = getSingleElementByTagNameNS_Standard;
		com.ibm.rdm.web.util.domutils.getAttributeNS = getAttributeNS_Standard;
	}
	
})();
})();
