/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.MessageBox");


dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.ExplorerView");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.CategoryModelProvider");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.SavedQueriesModelProvider");

(function() {
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var FiltersQueriesClient = com.ibm.asq.common.web.client.internal.FiltersQueriesClient;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var ListingViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ListingViewHeaderLayout;
var ExplorerView     = com.ibm.asq.common.web.ui.internal.widgets.navigator.ExplorerView;
var AbstractFilteredTableView = com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView;
var AbstractFilteredPagingTableView = com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredPagingTableView;
var Constants        = com.ibm.rqm.planning.web.client.internal.Constants;
var ModelNodeTypes   = com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes;
var CommonPreferenceClient = com.ibm.asq.common.web.client.internal.CommonPreferenceClient;
var TreeView         = com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView;
var CommonUtil = com.ibm.asq.common.web.ui.internal.util.Utility;

var CatProviderTitle = Constants.getCommonMessage("labelCatFilter");
var SQProviderTitle  = Constants.getCommonMessage("labelQueriesFilter");
var CatBCLabel       = Constants.getCommonMessage("labelCatBreadCrumbLabel");
var SQBCLabel        = Constants.getCommonMessage("labelSavedQueryBreadCrumbLabel");
var LoadingLabel     = Constants.getMessage("labelLoading");

var CategoriesKey    = "CATEGORIES";
var SavedQueriesKey  = "SAVED_QUERIES";

var FilterTypeQuery  = "filter-query";
var FilterTypeAttr   = "filter-attribute";

function name_sort(v1, v2)
{
    var name1 = v1.name ? v1.name.toLowerCase() : v1.name;
    var name2 = v2.name ? v2.name.toLowerCase() : v2.name;
    return (name1 > name2) ? 1 : (name1 == name2) ? 0 : -1;
};

/*
 * Shows the saved queries of a table viewer in a externally presentable panel.
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.ExternalQueriesPanel", 
             [dijit._Widget, dijit._Templated],
{
    templateString: '<div style="margin:5px;overflow:auto;border:solid 1px #D8D8D8"></div>',

    width: 460,
    height: 420,
    includePersonalQueries: true,
    
    postCreate: function() {
        var isRemote = this.parent.tableArgs && this.parent.tableArgs.useFF3NoBindingCSS ? this.parent.tableArgs.useFF3NoBindingCSS: false ;
        
        dojo.style(this.domNode, {
            width: this.width+'px',
            height: this.height+'px'
        });
        this.tree = new TreeView({
            showHeader: false,
            state: "open",
            isRemote: isRemote,
            onLoad: dojo.hitch(this, function(onComplete){
                onComplete(this.model);
            }),
            // handle tree item selection
            onClick: dojo.hitch(this, 'onClick'),
			onDblClick: dojo.hitch(this, 'onDblClick')
        });
        dojo.place(this.tree.domNode, this.domNode);
        this.tree.model = this._getModel();
        this.tree.loadContent();
        
    },
     _getModel: function() {
        var model = this.model = new com.ibm.asq.common.web.ui.internal.widgets.navigator.SavedQueriesModel({
            savedQueriesValuesProvider: dojo.hitch(this, "getAllSavedQueries"),
            includePersonalQueries: this.includePersonalQueries
        });
        return model;
    },
    clearSelection: function() {
        this.selection = null;
        this.tree.set('selectedItem', null);
    },
    getAllSavedQueries: function(item, onComplete) {
        var filterProvider = this.parent.tableViewer.filterAttrProvider;
        filterProvider.getAllSavedQueries(onComplete, dojo.hitch(this, '_onError', item, onComplete), true);
    },
    _onError: function(item, onComplete) {
        onComplete(item);
    },
    onClick: function(item) {
        this.selection = item;
        this.onSelect(item);
    },
	onDblClick: function(item, node, evt){
		this.selection = item;
        this.onSelect(item);
		if(dojo.isFunction(this.handleDoubleClick)){
			this.handleDoubleClick(item);
		}
	},
    onSelect: function() {}                    
});

/*
 * A chooser dialog that presents a table's content filters and its saved queries.
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.ExternalFiltersQueriesChooser", 
             [dijit._Widget, dijit._Templated],
{
    templateString: '<div>' +
                    '  <div  dojoAttachPoint="filterByNode">' +
                    '    <span class="label">${labelFilterBy}</span>' +
                    '    <input id=${id}-filter type="radio" name="display-type" style="margin-left:1em;" dojoAttachPoint="attrRadioNode" dojoAttachEvent="onclick:_showFilters">' +
                    '    </input>' +
                    '    <label for=${id}-filter>${labelAttributes}</label>' +
                    '    <input id=${id}-query type="radio" name="display-type" style="margin-left:1em;" dojoAttachPoint="queryRadioNode" dojoAttachEvent="onclick:_showQueries">' +
                    '    </input>' +
                    '    <label for=${id}-query>${labelQueries}</label>' +
                    '  </div>' +
                    '  <div dojoType="jazz.ui.MessageBox" dojoAttachPoint="messageBox" style="margin:5px;width:462px"></div>' +
                    '  <div dojoAttachPoint="filtersNode" style="overflow-y:auto;overflow-x:hidden;">' +
                    '  </div>' +
                    '  <div style="display:none;" dojoAttachPoint="queriesNode">' +
                    '  </div>' +
                    '</div>',

    labelFilterBy: Constants.getMessage('labelFilterBy'),
    labelAttributes:  Constants.getMessage('labelAttributes'),
    labelQueries:  Constants.getMessage('labelSavedQueries'),
    widgetsInTemplate: true,
    showFilters: true,
    disableFilters: false,
    title: null,
    TypeQuery: FilterTypeQuery,
    TypeAttribute: FilterTypeAttr,
    defaultQuery: null,
    includePersonalQueries: true,      // include personal queries by default
    
    postCreate: function() {
        this.defaultAttrFilters = null;
        
        //Assign the width to the dialog, depending the screen size.
        var screenHeight = window.screen.availHeight;
        var dialogHeight = screenHeight * .5;
        this.filtersNode.style.maxHeight = dialogHeight.toString() + "px";
        
        // if there's a default query, initialize the chooser panels with those values
        if(this.defaultQuery) {
            if(this.defaultQuery.type == this.TypeQuery) {
                this.showFilters = false; 
            }
            else {
                // sanitize the query filters before using them. This removes
                // filters that don't really have values
                this.defaultAttrFilters = this.parent._sanitizeQueryArgs(this.defaultQuery.filters);
            }
        }
        
        if(this.disableFilters){
        	this.showFilters = false;
        	dojo.style(this.filterByNode, "display", "none");
        } else {
        	this.attrRadioNode.checked = this.showFilters;
        	this.queryRadioNode.checked = !this.showFilters;
        }
        
        // initialize the radio button selection
        var init = this.showFilters ? '_showFilters' : '_showQueries';
        this[init]();
    },
    
    // Consumer can override this method to show a message box above the filter panels.
    // Expected to return the args for jazz.ui.MessageBox.
    getMessageBox: function() {
        return null;
    },
    
    showMessageBox: function(filterType) {
        var messageArgs = this.getMessageBox();
        if (messageArgs) {
            if (messageArgs.filterType) {
                // message box is only shown for specific filter type
                if (messageArgs.filterType == filterType) {
                    this.messageBox.showMessage(messageArgs);
                } else {
                    this.messageBox.hideMessage();
                }
            } else {
                // general message is always shown
                this.messageBox.showMessage(messageArgs);
            }
        }
    },
    
    _showFilters: function() {
        this.showMessageBox(FilterTypeAttr);
        if (!this.filters) {
            this.filters = this.parent.tableViewer.getContentFiltersExternalDisplay({
                defaultValues: this.defaultAttrFilters,
                onValueChanged: dojo.hitch(this, '_onContentChanged')
            });
            dojo.place(this.filters.domNode, this.filtersNode);
        }
        dojo.style(this.queriesNode, "display", "none");
        dojo.style(this.filtersNode, "display", "");
        this.type = this.TypeAttribute;
        if(this.queries) {
            this.queries.clearSelection();
        }
        if (this.dialog) {
            this.dialog._yesButton.disabled = true;
        }      
    },
    
    _showQueries: function() {
        this.showMessageBox(FilterTypeQuery);
        if (!this.queries) {
            this.queries = new com.ibm.rqm.planning.web.ui.internal.view.table.ExternalQueriesPanel({
                parent:this.parent,
                onSelect: dojo.hitch(this, '_onQuerySelected'),
				handleDoubleClick: dojo.hitch(this, "_onQueryDbClicked"),
				includePersonalQueries: this.includePersonalQueries
            });
            dojo.place(this.queries.domNode, this.queriesNode);
        }
        dojo.style(this.queriesNode, "display", "");
        dojo.style(this.filtersNode, "display", "none");
        this.type = this.TypeQuery;
        if(this.filters) {
            this.filters.clearFilters();
        }
        if (this.dialog) {
            this.dialog._yesButton.disabled = true;
        }      
    },                    

    open: function() {
        var dialog = this.dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog({
            primaryTitle: this.title || Constants.getMessage('titleSelectFilters'),
            width: "520px",
            okCallback: dojo.hitch(this, function() {
                // package the query value to return to the caller.
                // it's a 'union' of label, itemId, filters, based on the 'type' attribute
                var queryValue = {
                    'type': this.type
                };
                if (this.type == this.TypeQuery) {
                    var value = this.queries.selection;
                    queryValue.label = value.label;
                    queryValue.itemId = value.itemId;
                    queryValue.query = value.query;
                }
                else {
                    queryValue.filters = this.filters.getSelectedFilters();
                }
                this.onOk(queryValue);
            })
        });
        dialog.contentContainer.appendChild(this.domNode);
        this.dialog._yesButton.disabled = true;
        dialog.open();
    },
    
    _onQuerySelected: function(item) {
		if(this.dialog){
			this.dialog._yesButton.disabled = (item.type != ModelNodeTypes.SAVED_QUERY_TYPE);
		}
    },
    
    _onQueryDbClicked: function(item) {
		if(this.dialog && (item.type == ModelNodeTypes.SAVED_QUERY_TYPE)){
			this._handleDoubleClick();
		}
    },
    
	_handleDoubleClick: function(){
		if(this.dialog){
			this.dialog._yesAction();
		}
	},
	
    _onContentChanged: function() {
        // if we load default filter values into the filter attributes panel, it
        // would generate a value changed event. We have not created the dialog
        // yet, ignore it. 
        if (this.dialog) {
            this.dialog._yesButton.disabled = false;
        }      
    }                        
});

// subclass of the TableListener that notify the filter provider to invalidate the saved query
// on changes in sorting and group by.
//
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableListener", 
              com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    tableColumnSortChanged: function(tableViewer, sortColumns) {
        var filterProvider = tableViewer.filterAttrProvider;
        if (filterProvider && dojo.isFunction(filterProvider.onPerformColumnSort)) {
            filterProvider.onPerformColumnSort(sortColumns);
        }
    },
    
    // Summary:
    //   If content Group By is changed in the table header, this method is called.
    tableContentGroupByChanged: function(tableViewer, colIndex) {
        var filterProvider = tableViewer.filterAttrProvider;
        if (filterProvider && dojo.isFunction(filterProvider.onPerformGroupBy)) {
            filterProvider.onPerformGroupBy(colIndex);
        }
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableViewMixin", 
             [dijit._Widget, dijit._Templated],
{
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" style="width:99%;" tabindex="-1">' +
                    '</div>',
    
    showExplorer: true,
    exploreSavedQueries: true,
    exploreArtifactsLabel: "", // specific artifact list view's label for exploring artifact, i.e., explore test plan, etc.
    explorableArtifactType: null, // the specific artifact type, i.e., Constants.MODEL_TYPE_PLAN
    supportArtifactExploring: false,
    resizeableExplorerPanel: true,
    breadcrumbLabel: "",
    _enableSavedQuery: null,
    showFloatingStatusBar: true,
    // main query page's explorer panel's initial width & height
    explorerInitHeight: 470,
    explorerInitWidth: 250,
     
    postCreate: function() {
        // conditionally create the main content. If it's a main query page, create
        // the header layout.
        var showExplorer = this.showQueryFilters && this.showExplorer;
        
        // if there's no view header & no explorer, the domNode is the parent
        this._tableViewerNode = this.contentNode = this.domNode;
        
        if(this.isQueryPage) {
            this.viewHeaderNode = new ListingViewHeaderLayout({
                isMinMarginLeft: showExplorer,
                // allow info status messages to be closable by the user in the list views.
                // These usually are messages about artifacts being changed.
                infoStatusClosable: true,
                showBreadcrumb: showExplorer,
                containerWrapper: showExplorer   // create extra wrapper node for the container node if explorer is used
            });
            // the content node is the view header's container.
            this._tableViewerNode = this.contentNode = this.viewHeaderNode.containerNode;
        }
        if (showExplorer) {
            var explorer = new ExplorerView({
                height: this.explorerInitHeight,
                initExplorerWidth: (this.isQueryPage) ? this.explorerInitWidth : 220,
                resizeableExplorer: this.resizeableExplorerPanel,
				initOpenFiltersView: this.initCloseFiltersView ? !this.initCloseFiltersView : true
            });
            this.explorerView = explorer;
            
            // initialize the position of the view header & explorer
            // host the view header in the explorer's container
            //
            if (this.viewHeaderNode) {
                dojo.place(this.viewHeaderNode.domNode, this.domNode);
                
                // initially hide the explorer sidebar, until the table is actually drawn before showing it.
                dojo.style(explorer.domNode, "visibility", "hidden");
  
                // 'rewire' the viewHeader to insert the explorer around the container node
                dojo.place(explorer.domNode, this.viewHeaderNode.containerWrapperNode);
                dojo.addClass(this.viewHeaderNode.containerWrapperNode, "explorer-style");
                dojo.place(this.viewHeaderNode.containerNode, explorer.container);
                
                // 'rewire' the explorer's container to the one in the view header
                // override the explorer' detail container's loading state change method
                // to use the header's.
                explorer.container = this.contentNode;
                explorer.onDetailContainerLoadingChange = dojo.hitch(this, function(isLoading) {
                    var methodName = isLoading ? "setProgressStart" : "setProgressEnd";
                    dojo.style(this.contentNode, "display", isLoading ? "none" : "");
                    
                    // call the header progress start/end method.
                    this.viewHeaderNode[methodName](LoadingLabel);
                });
            }
            else {
                dojo.place(explorer.domNode, this.contentNode);
                this._tableViewerNode = this.contentNode = explorer.container;
            }
            this.initializeExplorerProviders();
            
            // wait until the table view is initialized first before showing/opening the 
            // explorer view. This is when the table view's initRunQuery method is called.
            //
            this.connect(this, "initRunQuery", function(){
                dojo.style(explorer.domNode, "visibility", "visible");
                
                // open the explorer's view based on the preference.
                if ((this.initOpenExplorer || this.isQueryPage) && !this.doneInitializeExplorer) {
                    this.doneInitializeExplorer = true;
                    var initOpenPref = CommonPreferenceClient.getStoredUserPreferences(Constants.USER_UI_CUSTOMISE_QUERYLISTS_EXPLORER_INITOPEN_CATEGORY.key);
                    if (this.initOpenExplorer || (initOpenPref && initOpenPref.value == true)) {
                        this.explorerView.set('viewOpen', true);
                    }
                }
            });
        }
        else if(this.viewHeaderNode) {
            // not showing the explorer and we have the header,
            // place the header in our display.
            dojo.place(this.viewHeaderNode.domNode, this.domNode);
        }
    },
        
    // Override this method to add artifact specific explorer providers. In addition to
    // the default explorer providers, categories and saved queries, specific artifact list
    // views can add more providers.
    //
    constructArtifactExplorerProviders: function(providers) {
        // add more providers by create an keyed entry in the providers record. The provider can optionally
        // provide a position, such as 0 to be shown on top, or 3 to be shown below the standard panels.
        //
        // for example:
        // 
        // var provider = new ExplorerProvider({});
        // provider.position = 0; put on top
        // providers['explorerX'] = provider
    },
    
    initializeExplorerProviders: function() {
        if (this.showQueryFilters) {
            var modelProviders = {};
            this.constructArtifactExplorerProviders(modelProviders);
            
            if (this.exploreCategory) {
                // set the category explorer as default provider (first)
                this.categoryModelProvider = this._constructCategoryModelProvider();
                this.categoryModelProvider.position = 1;
                modelProviders[CategoriesKey] = this.categoryModelProvider;
            }
            if (this.exploreSavedQueries) {
                this.savedQueriesModelProvider = this._constructSavedQueriesModelProvider();
                this.savedQueriesModelProvider.position = 2;
                modelProviders[SavedQueriesKey] = this.savedQueriesModelProvider;
            }
            this.explorerView.set("modelProviders", modelProviders);
        }
    },
    
    _onQueryComplete: function(item, resultsDTO) {
        var currentProvider = this.explorerView.get("currentProvider");
        
        // update the label of the item to include the count of the results.
        // but only for the category value or the saved query nodes. All other
        // node types are not applicable.
        if (currentProvider && currentProvider.updateItemSubLabel && 
            item && (item.type == ModelNodeTypes.CATEGORY_VALUE_TYPE || item.type == ModelNodeTypes.SAVED_QUERY_TYPE)) {
            // update the selected item's label with the query result count.
            currentProvider.updateItemSubLabel(item, resultsDTO.totalSize + "");
        }
    },
    
    _constructSavedQueriesModelProvider: function(){
        var modelprovider = new com.ibm.rqm.planning.web.ui.internal.view.table.SavedQueriesModelProvider({
            key: SavedQueriesKey,
            title: SQProviderTitle,
            shortLabel: SQBCLabel,
            parentView: this,
            explorerView: this.explorerView,
            tableViewer: this.tableViewer,
            runQuery: dojo.hitch(this, "runQuery"),
            onExplorerNodeSelected: dojo.hitch(this, "onExplorerNodeSelected")
        });
        return modelprovider;
    },

    _constructCategoryModelProvider: function(){
        var modelprovider = new com.ibm.rqm.planning.web.ui.internal.view.table.CategoryModelProvider({
            key: CategoriesKey,
            title: CatProviderTitle,
            shortLabel: CatBCLabel,
            exploreArtifactsLabel: this.exploreArtifactsLabel,
            relatedArtifactsLabel: this.relatedArtifactsLabel,
            
            // only allow detail mode in the main query page. Secondary
            // list like dialog will be disabled.
            supportArtifactExploring: this.supportArtifactExploring && this.isQueryPage,
            artifactType: this.explorableArtifactType,
            parentView: this,
            explorerView: this.explorerView,
            tableViewer: this.tableViewer,
            runQuery: dojo.hitch(this, "runQuery"),
            buildCategoriesFilters: dojo.hitch(this, "buildCategoriesFilters"),
            getArtifactChildren: dojo.hitch(this, "runExplorerQuery"),
            onExplorerNodeSelected: dojo.hitch(this, "onExplorerNodeSelected"),
            relatedContainerConstructor: dojo.hitch(this, "constructRelatedContainer"),
            relatedContainerLoader: dojo.hitch(this, "loadRelatedContainer"),
            updateHeader: dojo.hitch(this, function(title) {
                if (this.viewHeaderNode) {
                 	var uriArgs = CommonUtil.getArgsFromUri(window.location.hash);
                 	var groupByFilterLabel = "";
                	if(uriArgs.groupByFilterLabel){
                		groupByFilterLabel = decodeURIComponent(uriArgs.groupByFilterLabel);
                		var label = groupByFilterLabel.split(",");
                		groupByFilterLabel = label.join(" = ");
                	}
                    var filterTitle = "";
                    if(groupByFilterLabel){
                    	filterTitle = this.brStr ? ("(" + this.brStr + " > " + groupByFilterLabel + ")") : "";
                    }else{
                    	filterTitle = this.brStr ? ("(" + this.brStr + ")") : "";
                    }
                    this.viewHeaderNode.setTitle(title || this.queryName);
                    this.viewHeaderNode.setSubTitle(filterTitle);
                }
            })
        });
        return modelprovider;
    },

    buildCategoriesFilters: function(item) {
        var filters = [item.typeItemId + ":" + item.itemId];
        var parent = item.parent;
        
        // walk up the tree to include all parent category values in the
        // filter.
        while(parent && parent.type == ModelNodeTypes.CATEGORY_VALUE_TYPE) {
            filters.push(parent.typeItemId + ":" + parent.itemId);
            parent = parent.parent;
        }
        return filters;
    },
        
    runExplorerQuery: function(item, onComplete) {
         var filters = {};
        if (item.queryToken) {
            filters.queryToken = item.queryToken;
            filters.pageSize = item.pageSize;
            filters.page = item.page;
        }
        else {
            var curFilter = this.filterArgs || {};
            var curCat = this.buildCategoriesFilters(item);
            this.selectedCategory = curFilter.categories = curCat;
            filters = this._prepareQueryArgs(null);
            
            filters.categories = this.selectedCategory;
            if (filters.queryToken) {
                delete filters.queryToken;
            }
            filters.pageSize = 10;
        }
        var callback = {
            self: this,
            curItem: item,
            success: function(resultsDTO){
                var results = this.self._processExplorerQueryResult(this.curItem, resultsDTO);
                onComplete(results);
            },
            failure: function(error){
                this.self._failureExplorerQueryResult(error);
                onComplete([]);
            }
        };
        var responseHandler = new this._searchResponseHandlerType(callback, "success", "failure");          
        this._searchFunction(responseHandler, filters);
    },

    getLabelForArtifactType: function(type) {
        switch(type) {
            case Constants.MODEL_TYPE_PLAN:
                return Constants.getMessage("labelNTestPlans");
            case Constants.MODEL_TYPE_CASE:
                return Constants.getMessage("labelNTestCases");
        }
        return type;
    },
    
    _processExplorerQueryResult: function(item, resultsDTO)
    {
        var results = resultsDTO.results;
        var items = [];
        var len = results ? results.length : 0;
        for (var i = 0; i < len; i++) {
            var iitem = results[i];
            items.push({
                iconClass: this.getExplorerIconClass(iitem),
                type: ModelNodeTypes.ARTIFACT_TYPE,
                name: iitem.name,
                label: iitem.id + ":" + iitem.name,
                id: iitem.id,
                itemId: iitem.versionableItemId,
                children: this.getArtifactExplorableContainers(iitem)
            });
        }
        items.sort(name_sort);
        
        var curPage = resultsDTO.currentPage + 1;
        var totalPages = resultsDTO.totalPages;
        var nRemainPages = totalPages - curPage;
        var queryToken = resultsDTO.queryToken;
        var pageSize = resultsDTO.size;
        var totalSize = resultsDTO.totalSize;
        
        // show all other pages as a folder, but only show the 
        // page folder for the first query, i.e., from the category.
        // The page folder item would call this as well to fetch its
        // children.
        var typeLabel = this.getLabelForArtifactType(this.explorableArtifactType);
        if (item.type == ModelNodeTypes.CATEGORY_VALUE_TYPE && nRemainPages) {
            for (var i = curPage; i < totalPages; i++) {
                var startPos = i * pageSize;
                var span = (i == (totalPages - 1)) ? (totalSize - startPos) : pageSize;
                items.push({
                    iconClass: null,
                    page: i,
                    type: ModelNodeTypes.ARTIFACT_PAGE_TYPE,
                    label: Constants.getMessage("labelNextNItems", span, typeLabel),
                    queryToken: queryToken,
                    itemId: queryToken + (i+1),
                    pageSize: pageSize,
                    children: []
                });
            }
        }
        this._onQueryComplete(item, resultsDTO);
        return items;
    },
    
    _failureExplorerQueryResult: function(error)
    {
        
    }, 
    
    onExplorerNodeSelected: function(item) {
        if(this.viewHeaderNode) {
            var currentProvider = this.explorerView.get("currentProvider");
            this.brStr = currentProvider.getSelectionBreadCrumbDisplayString(item);
            this.updateBreadCrumb(this.brStr);
        }
        
        // save the item. If this selection caused a run query
        // we want to update the item's label with the query's
        // result count.
        this.currSelectedItem = item;
    },
    
    updateBreadCrumb: function(brStr) {
        // update the breadcrumb label with filter info
    	var groupByFilterLabel = this.groupByFilterLabel;
     	if(!groupByFilterLabel){
     		var uriArgs = CommonUtil.getArgsFromUri(window.location.hash);
	    	if(uriArgs.groupByFilterLabel){
	    		groupByFilterLabel = decodeURIComponent(uriArgs.groupByFilterLabel);
	    		var label = groupByFilterLabel.split(",");
	    		groupByFilterLabel = label.join(" = ");
	    	}
     	}
        var filterTitle = "";
        if(groupByFilterLabel){
        	filterTitle = this.brStr ? ("(" + this.brStr + " > " + groupByFilterLabel + ")") : "";
		}else{
			filterTitle = this.brStr ? ("(" + this.brStr + ")") : "";
		}
        if (this.viewHeaderNode) {
            // list view does not show filter info as breadcrumb but append the info to the page title
            this.viewHeaderNode.setTitle(this.title || this.queryName);
            this.viewHeaderNode.setSubTitle(filterTitle);
        }
        // update page breadcrumb entry info so next page will have filter info in the breadcrumb for this page
        this.breadcrumbLabel = this.queryName + " " + filterTitle;
        if (this.breadcrumbEntry) {
            // reset the breadcrumb entry of the page and let current provider update if necessary
            this.breadcrumbEntry.label = this.queryName;
            if (!this.originalBreadcrumbEntryHref) {
                this.originalBreadcrumbEntryHref = this.breadcrumbEntry.href;
            } else {
                this.breadcrumbEntry.href = this.originalBreadcrumbEntryHref;
            }
        }
        if(this.groupByFilterLabel)
        	this.groupByFilterLabel = "";
        if(this.groupByFilterField)
        	this.groupByFilterField = "";
        if(this.groupByFilterFieldValue)
        	this.groupByFilterFieldValue = "";
    },
    
    getExplorerIconClass: function(item)
    {
        // abstract method for subclass to supply the artifact's icon for the tree node in the explorer.
        return null;
    },
    
    getArtifactExplorableContainers: function(item)
    {
        // abstract method for subclass to supply children related artifact containers for an the artifact
        return null;        
    },
    
    constructRelatedContainer: function(item, onComplete)
    {
        // abstract method - constructs the actual related artifact containers for the selected node. 
    },
    
    loadRelatedContainer: function(item, container)
    {
        // abstract method - loads the item's content into a created container. 
    },
    
    // Get the folder item id of a folder item. See com.ibm.rqm.planning.web.ui.internal.view.table.ExplorableTableFilterProvider
    //
    _getModelFolderId: function(folder, scope) {
        var filterProvider = this.tableViewer.filterAttrProvider;
        return filterProvider.getModelFolderId(folder, scope);
    },
    
    // A listener after a new query is saved.  The saved queries in the filter provider is updated.
    // Update the explorer tree to pick up the updated model.
    onSavedQuerySaved: function(query) {
        if (this.showQueryFilters) {
            var explorerContent = this.explorerView.get("treeView");
            var tree = null;
            var model = null;
            // not all views are tree views. We can have non-tree
            // selector views.
            if (explorerContent && explorerContent.tree) {
                // check if the new query already has a tree node i.e. save is overwriting an existing query
                tree = explorerContent.tree;
                model = tree ? tree.model : this.savedQueriesModelProvider.model;
                var exisintgQueryNode = tree.getNodesByItem(query.itemId);
                if (exisintgQueryNode && exisintgQueryNode.length) {
                    var exisintgQuery = exisintgQueryNode[0] ? exisintgQueryNode[0] : null;
                    if (exisintgQuery && exisintgQuery.item) {
                        var queryItem = exisintgQuery.item;
                        // update the query of the existing item
                        queryItem.query = query;
                        model.onChange(queryItem);
                        this._invalidateParentItem(queryItem.parent, model);
                    }
                }
            }
            var currentProvider = this.explorerView.get("currentProvider");
            if (currentProvider && currentProvider.key == SavedQueriesKey) {
                // update the tree only if saved queries provider is up
                var folder = query.folder;
                // get the tree node of the query scope i.e. personal or shared
                var scopeNode = tree ? tree.getNodesByItem(query.scope) : null;
                if (scopeNode && scopeNode.length && scopeNode[0]) {
                    // get the item of the tree node
                    var scopeItem = scopeNode[0].item;
                    if (folder) {
                        var folderId = this._getModelFolderId(folder, query.scope);
                        var siblings = scopeItem.children;
                        var folderExists = false;
                        if (siblings) {
                            for (var i = 0; i < siblings.length; i++) {
                                if (siblings[i].itemId == folderId) {
                                    // folder already exists, invalidate folder
                                    this._invalidateParentItem(siblings[i], model);
                                    folderExists = true;
                                    break;
                                }
                            }
                        }
                        if (!folderExists) {
                        	// new folder does not exist, invalidate the scope to show the folder
                        	this._invalidateParentItem(scopeItem, model);
                        }
                    } else {
                        // no folder is used for this query, invalidate the scope
                        this._invalidateParentItem(scopeItem, model);
                    }
                    // select the new query node to expand the new folder
                } else {
                // something is wrong if a query has no scope, reload the tree
                this.explorerView.reloadContent();
                }
            }
            // Use the update action for the action dispatcher to expand the panel and select the query.
            // Use forceUpdate in case the query is already selected.  Need to select again to run the
            // query with the updated filters or establish a new column display settings profile.
            this.updateSelectedSavedQuery(query.scope, query.folder, query.itemId, query.title, /* forceUpdate */ true);
        }
    },
    
    // A listener after a saved query is deleted.  The saved queries in the filter provider is updated.
    // Update the explorer tree to pick up the updated model.
    onSavedQueryDeleted: function(parentItem, queryItem) {
        var tree = this.explorerView.get("treeView").tree;
        // get the selected item in the tree
        var selectedItem = tree.get("selectedItem");
        if (selectedItem) {
            // remove the selection of the deleted query to clear the filters
            if (queryItem.itemId == selectedItem.itemId) {
                // this method can only be invoked from the saved queries explorer so
                // its provider must be the current one
                this.savedQueriesModelProvider.clearFilter();
            }
        }
        var itemToInvalidate = parentItem;
        // check if the parent folder, if there is one, is deleted because it is empty
        var emptyParent = true;
        for (var key in parentItem.queries) {
            emptyParent = false;
            break;
        }
        var model = tree.model;
        model.onDelete(queryItem);
        if (emptyParent) {
            // invalidate the parent's parent if the parent is a folder to remove the empty parent
            if (parentItem.type == ModelNodeTypes.SAVED_QUERY_CONTAINER_TYPE && parentItem.parent && parentItem.parent.children) {
                itemToInvalidate = parentItem.parent;
                model.onDelete(parentItem);
            }
        }
        this._invalidateParentItem(itemToInvalidate, model);
    },

    // A listener after a saved query is promoted or removed from the application menu.
    onSavedQueryInMenuChanged: function(parentItem, queryItem) {
        // get the tree node
        var tree = this.explorerView.get("treeView").tree;
        var treeNode = tree.getNodesByItem(queryItem.itemId);
        if (treeNode && treeNode.length) {
            // set the state based on the updated query in the item
            treeNode[0].set("stateIcon", queryItem.query.inMenu ? "saved-queries-icon-in-menu" : "");
        } else {
            // the tree node of the item that initiated the action is not found
            // this should not happen but just in case, recreate the same item
            var model = tree.model;
            model.onChange(queryItem);
            this._invalidateParentItem(parentItem, model);
        }
    },

    onSavedQueryEdited: function(parentItem, queryItem, newQuery, originalQuery) {
        var tree = this.explorerView.get("treeView").tree;
        // get the selected item in the tree
        var selectedItem = tree.get("selectedItem");
        var needUpdateSelection = selectedItem && selectedItem.itemId == queryItem.itemId;
        if (needUpdateSelection) {
            tree.set("selectedItem", null);
        }
        if (newQuery.scope == originalQuery.scope && newQuery.folder == originalQuery.folder) {
            // location did not change, only the name of the queryItem is changed.
            queryItem.label = newQuery.title;
            queryItem.query = newQuery;
            var model = tree.model;
            model.onChange(queryItem);
            this._invalidateParentItem(parentItem, model);
        } else {
            // location is changed, remove the old queryItem and create a new one
            this.onSavedQueryDeleted(parentItem, queryItem);
            this.onSavedQuerySaved(newQuery);
        }
        // restore the selection if it was selected before the edit
        if (needUpdateSelection) {
            setTimeout(dojo.hitch(this, function() {
                this._selectSavedQueryNode({
                    sqScope: newQuery.scope,
                    sqFolder: newQuery.folder,
                    sqItemId: newQuery.itemId
                });
            }), 0);
        }
    },
    
    _invalidateParentItem: function(parentItem, model) {
        // invalidate the children list for getChildren to regenerate
        parentItem.children = null;
        if (model) {
            model.getChildren(parentItem, function(children){
                model.onChildrenChange(parentItem, children);
            });
        }
    },
    
    selectSavedQueryOnExplorerOpen: function() {
        if (this.selectQueryOnExplorerOpen) {
            delete this.selectQueryOnExplorerOpen;
            setTimeout(dojo.hitch(this, "_selectSavedQueryNode", this.filterQueryID), 0);
        }
    },
    
    // update action to update the page that is already in the selected tab in the front
    updatePage: function(actionArgs) {
        if (actionArgs.updateAction) {
            switch (actionArgs.updateAction) {
                case  "update-saved-query-selection":
                    var scope = actionArgs.sqScope;
                    var folder = actionArgs.sqFolder || null;
                    var itemId = actionArgs.sqItemId;
                    var title = actionArgs.sqTitle;
                    if(actionArgs.groupByFilterLabel){
                		this.groupByFilterLabel = decodeURIComponent(actionArgs.groupByFilterLabel);
                		var label = this.groupByFilterLabel.split(",");
                		this.groupByFilterLabel = label.join(" = ");
                	}
                    this.groupByFilterField = actionArgs.groupByFilterField;
                    this.groupByFilterFieldValue = actionArgs.groupByFilterFieldValue;
                    
                    if (scope && itemId) {
                        this.updateSelectedSavedQuery(scope, folder, itemId, title, (this.groupByFilterLabel || this.previousGroupFilterOn) ? true : false);
                    }
                    if(this.groupByFilterField){
                    	this.previousGroupFilterOn = true;
                    }else{
                    	this.previousGroupFilterOn = false;
                    }
                    break;
                case "clear-filter-selection":
                    this.clearFilter();
            }
        }
    },
    
    // This wrapper method is the update action for the action dispatcher when a
    // saved query is selected from the application sub-menu or external links
    // such as from a dashboard viewlet.  This makes sure that the saved queries
    // are initialized in the filterAttrProvider while initializing the table view.
    //
    updateSelectedSavedQuery: function(scope, folder, itemId, title, forceUpdate) {
        var filterProvider = this.tableViewer.filterAttrProvider;
        
        // make sure we don't hang waiting, not more than 10 seconds
        if (this.saveQueriesWaitingCount) {
            this.saveQueriesWaitingCount++;
        } else {
            this.saveQueriesWaitingCount = 1;
        }
        
        if (filterProvider.loadingQueries && this.saveQueriesWaitingCount < 50) {
            // still loading, retry
            setTimeout(dojo.hitch(this, function() {
                this.updateSelectedSavedQuery(scope, folder, itemId, title, forceUpdate);
            }), 200);
        } else {
            delete this.saveQueriesWaitingCount;
            this._updateSelectedSavedQuery(scope, folder, itemId, title, forceUpdate);
        }
    },
    
    // Besides bringing the view all artifacts tab to the front, this update action
    // is invoked to select and run the saved query.
    //
    _updateSelectedSavedQuery: function(scope, folder, itemId, title, forceUpdate) {
        var queryArgs = {
            sqScope: scope,
            sqFolder: folder,
            sqItemId: itemId,
            sqTitle: title
        };

        // find the saved query
        var filterProvider = this.tableViewer.filterAttrProvider;
        var query = filterProvider.findQuery(queryArgs);
        
        if (this.explorerView.isInitialized) {
            // explorer view is initialized i.e. the explorer panel has been opened with children views
            // initialized but the children views may not be all loaded
            var currentProvider = this.explorerView.get("currentProvider");
            var savedQueriesTreeView = this.explorerView.providerViews[SavedQueriesKey];

            if (savedQueriesTreeView.init) {
                // save queries tree view is already loaded
                if (!currentProvider || (currentProvider && currentProvider.key != SavedQueriesKey)) {
                    // the saved queries explorer is not the current provider
                    this.explorerView.set("viewOpen", true); // make sure the explorer panel is opened

                    // open the save queries tree view and notify the container to set to current provider
                    savedQueriesTreeView.set("state", "open", true /* animate */, true /* notify*/);
                    if (query) {
                        // select the tree node
                        setTimeout(dojo.hitch(this, "_selectSavedQueryNode", queryArgs, forceUpdate), 0);
                    } else {
                        // query is not found so no node to select
                        this.showPhantomQuery(queryArgs);
                    }
                } else {
                    if (query) {
                        // the saved queries explorer is the current provider
                        if (currentProvider.explorerOpen) {
                            // the explorer panel is open, open the tree view
                            savedQueriesTreeView.set("state", "open");
                            // select the tree node
                            setTimeout(dojo.hitch(this, "_selectSavedQueryNode", queryArgs, forceUpdate), 0);
                        } else {
                            // the explorer panel is closed, can not set select the tree node to focus
                            // notify the onExplorerOpen handler to do the select node
                            this.selectQueryOnExplorerOpen = true;
                            this.filterQueryID = queryArgs;
                            this.explorerView.set("viewOpen", true);  // open the explorer panel
                        }
                    } else {
                        // clear selected node if there is any no matter explorer panel is opened or closed
                        if (savedQueriesTreeView.get("selectedItem")) {
                            // this is clearFilter from the model provider which we opted to skip running
                            // the default view all query becasue we are going to run the phontom query
                            currentProvider.clearFilter(true);
                        }
                        this.showPhantomQuery(queryArgs);
                    }
                }
            } else {
                // save queries tree view is not yet loaded
                if (query){
                    // set the args to trigger selecting the saved query on the tree load complete
                    this.initFilterQuery = true;
                    this.filterQueryID = queryArgs;
                }
                this.explorerView.set("viewOpen", true);  // make sure the explorer panel is open
                // open the save queries tree view to load the tree and notify the container to set to current provider
                savedQueriesTreeView.set("state", "open", true /* animate */, true /* notify*/);
                
                if (!query) {
                    this.showPhantomQuery(queryArgs);
                }
            }
        } else {
            // explorer panel has not been opened to initialized
            if (query) {
                // set the args to trigger selecting the saved query on the tree load complete
                this.initFilterQuery = true;
                this.filterQueryID = queryArgs;

                // set saved queries explorer as the current provider
                this.explorerView.set("currentProvider", this.savedQueriesModelProvider);

                // expand the explorer to expose and load the tree
                this.explorerView.set("viewOpen", true);
            } else {
                // no need to open the explorer panel since we are only setting filters
                this.showPhantomQuery(queryArgs);
            }
        }
    },
    
    // A saved query is invoked but it is not found for the current user.  The saved query
    // may be deleted or the saved query is from a viewlet that is set by other user that
    // is not in the user's scope.
    //
    showPhantomQuery: function(queryArgs) {
        var query = null;
        if (queryArgs.sqItemId) {
            // saved query from a view should cache the query object by the itemId so
            // we can get the filters of query
            query = dojo.getObject("PHANTOM-QUERY-"+queryArgs.sqItemId);
        }
        
        var filterProvider = this.tableViewer.filterAttrProvider;
        if (query && query.filters) {  // make sure there are filters
            // phantom query found from other user
            this.savedQueriesModelProvider.setAndRunQuery(query, true);
            // get the query params from the query object if it is not already set as uri params
            // a viewlet might only store the itemid and the query object
            queryArgs.sqTitle = queryArgs.sqTitle || query.name || query.title;
            queryArgs.sqScope = queryArgs.sqScope || query.scope;
            queryArgs.sqFolder = queryArgs.sqFolder || query.folder;
            filterProvider.setPhantomSelectedQuery(queryArgs);
        } else {
            // query is not found, probably deleted and brought up from view history
            filterProvider.setPhantomSelectedQuery(null);
            
            // we might have cleared the filter UI without running the default query,
            // so clear it here to run the defaultquery
            var currentProvider = this.explorerView.get("currentProvider");
            if (currentProvider) {
                // not using clearFilter() from the list view to check selection because the
                // selection might be cleared assuming we are going to run a phantom query
                currentProvider.clearFilter();
            } else {
                // no provider is loaded yet so just run query with no filters 
                this.runQuery(null, true);
                // also clear the saved query params in the page uri to reflect the no filter query result
                var currentUri = window.location.hash;
                var uriParams = CommonUtil.getArgsFromUri(currentUri);
                var subAction = uriParams.subAction;
                if (subAction) {
                    var newUri = CommonUtil.getArtifactListViewUri(subAction);
                    var displayUri = CommonUtil.getDispatcherSubActionUri(subAction);
                    dojo.publish(GlobalEvents.UpdateTabUri, [currentUri, newUri, displayUri]);
                }
            }

            var messageKey = "msgQueryNotFound" + (queryArgs.sqTitle ? "WithName" : "");
            var message = Constants.getMessage(messageKey, queryArgs.sqTitle);
            var detail = Constants.getMessage("msgDeletedQueryDetail");
            
            // Use a sticky message to make sure the message won't get refreshed away by
            // other async tasks.  It will be cleared when any action is invoked.
            this.showStickyStatusInfo(message, detail);
        }
    },

    // Clear the selection of the explorer tree and all filters.  It is also the update
    // action for the action dispatcher when the view all artifacts action is selected
    // from the application menu with the view all artifacts tab already opened.
    // 
    clearFilter: function() {
        var currentProvider = this.explorerView.get("currentProvider");
        if (currentProvider) {
            // if no current provider => no filter applied => nothing to clear
            var view = this.explorerView.get("treeView");
            // get the selected item in the tree
            var selectedItem = view.get("selectedItem");
            if (selectedItem) {
                // no need to clear if there is nothing selected in the tree
                currentProvider.clearFilter();
            }
        }
    },
    
    // savedQuery expects params as sqScope, sqFolder and sqItemId.  The "sq" prefix is used in the
    // action dispatcher as filtered attributes.
    //
    _selectSavedQueryNode: function(savedQuery, forceUpdate) {
        var savedQueriesTreeView = this.explorerView.providerViews[SavedQueriesKey];
        var selectedItem = savedQueriesTreeView.get("selectedItem");

        // select node only if there is no selection or something else is selected
        if (!selectedItem || (selectedItem && selectedItem.itemId != savedQuery.sqItemId)) {
            // compose the path to the node
            var path = [FiltersQueriesClient.MODELROOT, savedQuery.sqScope];
            if (savedQuery.sqFolder) {
                var folderId = this._getModelFolderId(savedQuery.sqFolder, savedQuery.sqScope);
                path.push(folderId);
            }
            path.push(savedQuery.sqItemId);
            // clear the current selected item if there is any
            if (selectedItem) {
                savedQueriesTreeView.set("selectedItem", null);
            }
            this.selectNode(path, dojo.hitch(this, "_selectSavedQueryNodeAction"));
        } else if (forceUpdate && selectedItem && (selectedItem.itemId == savedQuery.sqItemId)) {
            // node is already selected but need to force update, just invoke the callback
            // for selecting the node
            this._selectSavedQueryNodeAction(selectedItem);
        }
    },

    _selectSavedQueryNodeAction: function(selectedItem) {
        if (selectedItem) {
            var provider = this.explorerView.get("currentProvider");
            provider.onSelect(selectedItem, dojo.clone(this.groupByFilterLabel), dojo.clone(this.groupByFilterField), dojo.clone(this.groupByFilterFieldValue));
        }
    },
    
    // Summary
    //   Programmatically select a tree node and invoke the call back on the node
    //
    selectNode: function(path, callback) {
        var tree = this.explorerView.get("treeView") ? this.explorerView.get("treeView").tree : null;
        if (!tree) {
            // there is no current tree view i.e. no current provider
            // the consumer of the method should make sure the explorer is in proper state 
            return;
        }
        // Setting the path will select the new node but it is deferred until the tree has finished loading.
        // No need for the listener if there is no call back.
        if (callback) {
            // select the tree node
            var pathDeferred = tree.set("path", path);
            var self = this;
            dojo.when(pathDeferred, function() {
                var node = tree.get('selectedNode');
                if (node && node.item){
                    callback(node.item);
                } else {
                    self.onError({message:Constants.getMessage("msgErrorRunningQuery")});
                }
            });
        }
    },

    onError: function(error) {
        if (error) {
            var errMsg = error.message ? error.message : Constants.getMessage("unknownUIError");
            if (this.viewHeaderNode) {
                this.viewHeaderNode.setStatus(errMsg, ViewHeaderLayout.ERROR);
            }
            console.dir(error);
        }
    },
    
    // Set a status message in the view header that won't get cleared excepted by hideStickyStatusInfo()
    // or explicit user click to close the status box.  This is to avoid async tasks from
    // unconditionally clear the status after loaded the list view.
    //
    showStickyStatusInfo: function(message, detail) {
        if (!this.stickyStatusHandler) {
            // set a handler to track status box being closed
            this.stickyStatusHandler = dojo.connect(this.viewHeaderNode, "clearStatus", dojo.hitch(this, function() {
                // set the message again if others clear the message
                if (this.stickyStatus) {
                    this.viewHeaderNode.setStatus(this.stickyStatus, ViewHeaderLayout.INFO, detail);
                }
            }));
            // set a handler to track user closed the status box
            this.userCloseStatusHandler = dojo.connect(this.viewHeaderNode, "_onMessageClose", dojo.hitch(this, function() {
                if (this.stickyStatus) {
                    delete this.stickyStatus;
                }
                dojo.disconnect(this.userCloseStatusHandler);
                delete this.userCloseStatusHandler;
                this.hideStickyStatusInfo();
            }));
        }
        this.stickyStatus = message;
        this.viewHeaderNode.setStatus(this.stickyStatus, ViewHeaderLayout.INFO, detail);
    },
    
    // clear the status message and remove the handler to keep the status message
    //
    hideStickyStatusInfo: function() {
        // remove the handler and clear the status message
        if (this.stickyStatusHandler) {
            dojo.disconnect(this.stickyStatusHandler);
            delete this.stickyStatusHandler;
        }
        if (this.stickyStatus) {
            delete this.stickyStatus;
        }
        setTimeout(dojo.hitch(this, function() {
            this.viewHeaderNode.clearStatus();
        }), 200);
    },
    
    // set or clear the status message if the list view is showing a phantom query
    setPhantomSelectedQuery: function(query) {
        if (this.viewHeaderNode) {
            if (query) {
                var messageKey = "msgQueryNotFound" + (query.sqTitle ? "WithName" : "");
                var message = Constants.getMessage(messageKey, query.sqTitle);
                var detail = Constants.getMessage("msqOthersQueryDetail");
                this.showStickyStatusInfo(message, detail);
            } else {
                this.hideStickyStatusInfo();
            }
        }
    },
    
    /*
     * Brings up the change table settings dialog. This allows tables shown in other
     * context without the header action to change the display settings.
     * The user's selection will be passed to the callback method as follows:
     *  
     *  onOk(settingsValue)
     *  settingsValue = {
     *     'columnsProfile': columnsProfile,
     *     'customSettings': customSettings
     *  }
     */
    showExternalColumnSettingsOptions: function(onOk) {
        this.tableViewer.showExternalColumnSettingsOptions(onOk);
    },
    
    /*
     * Returns the default settings of the table appropriate for external source showing
     * the display settings.
     */
    getDefaultColumnSettingsOptions: function() {
        return this.tableViewer.getDefaultColumnSettingsOptions();
    },
    
    /*
     * Change the table's settings returned by the method's showExternalColumnSettingsOptions
     * settings must be:
     *  settings = {
     *     'columnsProfile': columnsProfile,
     *     'customSettings': customSettings
     *  }
     */
   changeExternalColumnSettingsOptions: function(settings) {
        this.tableViewer.changeExternalColumnSettingsOptions(settings);
    },
    
    /*
     * Provides the table's query filters and saved queries in a single dialog. This dialog
     * is used to support changing filters & saved queries for the table's content in a 'non-conventional'
     * table context. This is for displaying table without the table header, etc, e.g., in the base table
     * viewer's DASHBOARD trim style. This shows a dialog with switchable filtered by attributes or
     * by saved queries. The returned query object should be run with by the next method, 'runExternalFiltersQueriesOptions'.
     * 
     * 
     * The user's selection will be passed to the callback method as follows:
     *  onOk(queryValue)
     *      queryValue = {
     *          type: one of: 'filter-query', or 'filter-attribute'
     *          value: for query {label: 'query's name', itemId: 'query's itemId'}
     *          value: for attribute: {filters: {actual filter objects}}
     *      }
     * 
     * The optional chooserArgs will customize the ExternalFiltersQueriesChooser.  Some attributes included:
     *   title                  - set to override the default "Select filters"
     *   disableFilters         - hide the Filter By Attributes panel if set to true
     *   includePersonalQueries - personal queries is included by default, set to false to override
     *   getMessageBox          - method to return args for jazz.ui.MessageBox to show a message
     */
    showExternalFiltersQueriesOptions : function(defaultQuery, onOk, /* optional */ chooserArgs) {
        var args = {
            parent: this,
            defaultQuery: defaultQuery,
            onOk: onOk
        };
        if (chooserArgs) {
            dojo.mixin(args, chooserArgs);
        }
        var chooser = new com.ibm.rqm.planning.web.ui.internal.view.table.ExternalFiltersQueriesChooser(args);
        chooser.open();
    },
    
    // Using the saved query item id in the filterObj, look up the filters of the saved query as the param
    // to invoke the specified callback.
    //
    getQueryFilters: function(filterObj, callback, /* optional */onError) {
        var filterProvider = this.tableViewer.filterAttrProvider;
        var queryId = filterObj.itemId;
        var handler = new ServiceResponseHandler({
            success: function(response){
                var foundQuery = false;
                if (dojo.isArray(response) && response.length > 0) {
                    for (var i = 0; i < response.length; i++) {
                        var query = response[i];
                        if (query.itemId == queryId) {
                            foundQuery = true;
                            var filters = FiltersQueriesClient._getFilters(query.values);
                            callback(filters);
                            break;
                        }
                    }
                }
                if(!foundQuery && onError) {
                    onError(Constants.getMessage("msgQueryNotFoundWithName", filterObj.label));
                }
            },
            error: function(e){
                if (onError) {
                    onError(Constants.getMessage("errFetchQuery"));
                }
            }
        }, "success", "error");
        var args = {
            artifactType: filterProvider.artifactTypeQueryKey,
            projectAreaItemId: this.projectAreaItemId,
            itemIds: [queryId]
        };
        FiltersQueriesClient.getQueries2(handler, args);
    },
    
    /*
     * Use this method to run the query object produced by the method 'showExternalFiltersQueriesOptions'.
     * If the filter type is a saved query, this method will get the query's filters first to run. Client
     * should provide an onError function to handle any errors that could happen in this method.
     */
   runExternalFiltersQueriesOptions: function(filterObj, updateOnly, onError) {
        var filterProvider = this.tableViewer.filterAttrProvider;

        // the filter type is query, get the saved query's filters
        // and we run it.
        if (!filterObj.filters && (filterObj.type == FilterTypeQuery)) {
            this.getQueryFilters(filterObj, dojo.hitch(this, function(filters) {
                filterProvider.filterRunActionInvoked('run', filters, updateOnly);
            }), onError);
        }
        else {
            // just plain filters, pass the filters directly to the run method.
            var filterValues = filterObj.filters;
            
            // always cause the table to do relayout. External running of the filter options
            // could be due to change of columns as well. 
            filterProvider.filterRunActionInvoked('run', filterValues, updateOnly);
        }
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractNonPagingExplorableTableView", 
             [com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableViewMixin,
              AbstractFilteredTableView],
{
    // sigh...  there are still table list view with filters and saved queries that are not using paging

    // @override AbstractFilteredTableView
    //
    _initRunQuery: function(){
        if (this.filterQueryID && this.filterQueryID.sqItemId && this.showQueryFilters) {
            this.updateSelectedSavedQuery(this.filterQueryID.sqScope, this.filterQueryID.sqFolder, this.filterQueryID.sqItemId, this.filterQueryID.sqTitle);
        } else {
            this.inherited(arguments);
        }
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableView", 
             [com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableViewMixin,
              AbstractFilteredPagingTableView],
{
    // @override AbstractFilteredTableView - to handle synchronization of running the query from
    // the table's filters and the explorer filter.
    runQuery: function(filterArgs, updatePageOnly, fromExplorer) {
        var fromInlineFilter = !fromExplorer && filterArgs;
        
        // If the query is initiated from inline filter and we're using explorer view,
        // process and syncing with the explorer's panel filters.
        if(this.explorerView && fromInlineFilter) {
            if (this.explorerView.isInitialized) {
                var currentProvider = this.explorerView.get("currentProvider");
                if (currentProvider && dojo.isFunction(currentProvider.suspendExplorerQuery)) {
                    // the query is being invoked from table's advanced filters, relinquish
                    // the explorer controls
                    currentProvider.suspendExplorerQuery();
                }
            }

            // check to see if any providers is capable of syncing with its inline
            // filter. Send a message to it to seed its filters. The provider doesn't
            // have to be in view to handle this. We want to seed the value in cases
            // the user opens it later.
            var providers = this.explorerView.get('modelProviders');
            if(providers) {
                for(var id in providers) {
                    var provider = providers[id];

                    // if the provider is capable of syncing with the inline filter values
                    // send it the inline filter args.
                    if(provider.syncWithInlineFilter) {
                        var filterName = provider.inlineFilterName;
                        var filter = filterName && filterArgs[filterName] ? filterArgs[filterName] : null;
                        provider.syncWithInlineFilter(filter);
                    }
                }
            }
        }
        this.inherited(arguments);
    },
    
    // Work Item 96983: Overriding from AbstractFilteredTableView in order for the filters to re-use the same category model than the table.
    _getCategoryValuesLoader: function() {
        this.inherited(arguments);
        this.catValuesLoader.model =  (this.provider && this.provider.getCategoryModel) ? this.provider.getCategoryModel() : null;
        return this.catValuesLoader;
    },
    
    _getParentCategoryValuesLoader: function() {
        this.inherited(arguments);
        this.catValuesLoader.parentModel =  this.provider ? this.provider.getParentCategoryModel() : null;
        return this.catValuesLoader;
    },
        
    // @override AbstractFilteredPagingTableView to augment categories filtering in the explorer view
    _prepareQueryArgs: function(filterArgs, updatePageOnly) {
        var args = this.inherited("_prepareQueryArgs", arguments);
        
        if (this.explorerView) {
            var currentProvider = this.explorerView.get("currentProvider");
            
            // if the provider has a need to augment the query args, call it.
            if (currentProvider && currentProvider.augmentQueryArgs) {
                args = currentProvider.augmentQueryArgs(args);
            }
        }
        return args;
    },

    // @override AbstractFilteredTableView
    //
    _initRunQuery: function(){
        if (this.filterQueryID && this.filterQueryID.sqItemId && this.showQueryFilters) {
            this.updateSelectedSavedQuery(this.filterQueryID.sqScope, this.filterQueryID.sqFolder, this.filterQueryID.sqItemId, this.filterQueryID.sqTitle);
        } else {
            this.inherited(arguments);
        }
    }
    
});

})();