/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.AbstractFilteredTableViewlet");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractExplorableTableView");
dojo.require("com.ibm.asq.common.web.client.internal.ConfigurationManagementClient");
dojo.require("com.ibm.team.configuration.web.context.ContextViewletMixin");
dojo.require("jazz.app.context");

/**
 * The abstract filtered table viewlet hosts an artifact table viewer of the type AbstractExplorableTableView
 * which supports both saved queries and query filters. The subclass is responsible to create the specific artifact
 * table and configure the columns to display, etc. The artifact table would be configured by default to not 
 * run the default queries, which most likely to return all contents without filters. The user must set the
 * filters, or select a saved query to run to view contents. It has two major areas:
 *  
 *  1) the table content area which hosts an artifact table viewer of the type AbstractExplorableTableView that
 *     subclass must create.
 *     
 *  2) a message area which shows a message to inform the user that the viewlet table needs configuration,
 *     i.e., to set the filter, or to select a saved query. The message will be shown under 2 conditions:
 *        - no filters, or saved queries is selected.
 *        - the queries doesn't produce any results.
 *     The table content area would be hidden when the message area is shown and vice versa.  
 * 
 */

(function() {
var TableConstants          = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ServiceResponseHandler  = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
var AbstractViewlet         = com.ibm.asq.common.web.ui.internal.view.viewlets.AbstractViewlet;
var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var ItemNameRegistry        = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var AlertDialog             = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var ConfigurationManagementClient = com.ibm.asq.common.web.client.internal.ConfigurationManagementClient;
var ContextViewletMixin 	= com.ibm.team.configuration.web.context.ContextViewletMixin;
var getCurrentPA            = jazz.app.context.get;
var ProcessClient           = com.ibm.team.process.web.client.internal.ProcessClient;
var FiltersQueriesClient    = com.ibm.asq.common.web.client.internal.FiltersQueriesClient;

var TypeQuery               = "filter-query";
var LabelNoFilters          = Constants.getMessage('labelNoFilters');
var LabelAttributes         = Constants.getMessage('labelAttributes');
var LabelByQuery            = Constants.getMessage('labelBySavedQuery');
var LabelLoading            = Constants.getMessage('labelLoading');

var PREF_SCOPE_ITEM         = "scopeItem";
var PREF_SCOPE_PA           = "PROJECT_AREA";
var IsFF3                   = dojo.isFF >= 3 && dojo.isFF < 4;

/*
 * The provider class to provide the table query/filter custom preference selection to the viewlet
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.FilteredTableQueryProvider", 
             CustomPreferenceProvider,
{
    parentViewlet: null,
    prefFilterId: null,
    
    constructor: function(args) {
        dojo.mixin(this, args);    
    },
    
    getLabel: function(value){
        // there's a filter/query created, show its details.
        if (value != -1) {
            var queryObj = dojo.fromJson(value);
            if (queryObj.type == TypeQuery) {
                return LabelByQuery + queryObj.label;
            }
            else {
                return LabelAttributes;
            }
        }
        return LabelNoFilters;
    },
    
    getLoadingLabel: function() {
        return LabelLoading;
    },
     
    // Compose the chooserArgs based on the scope that the viewlet is in to disable
    // personal queries if the dashboard scope is shared - not personal dashboard.
    //
    composeChooserScopeArgs: function() {
        var isSharedDashboard = this.parentViewlet.isSharedDashboard();
        var chooserArgs = {
            includePersonalQueries: isSharedDashboard ? false : true
        };
        if (isSharedDashboard) {
            chooserArgs.getMessageBox = function() {
                return {
                    filterType: TypeQuery,
                    summary: Constants.getMessage("msgSelectPublicQuery"),
                    type: jazz.ui.MessageBox.STATUS_TYPES.INFO
                };
            };
        }
        return chooserArgs;
    },
    
    createChooser: function(onOk, settings, viewlet){
        // if there's no project yet, show a message. Can't show preferences
        // without a project context.
        if(!this.parentViewlet.checkHasProjectContext()) {
            return;
        }
        // call the viewlet's method to ensure we've a backing table context before
        // using it.
        this.parentViewlet._useBackingTableForPreferences(dojo.hitch(this, function(tableViewer){
            var preferences = settings.preferences;
            var filters = preferences[this.parentViewlet.prefFilterId];
            var filtersObj = null;
            if (filters != -1) {
                filtersObj = dojo.fromJson(filters);
            }
            var chooserArgs = this.composeChooserScopeArgs();
            tableViewer.showExternalFiltersQueriesOptions(filtersObj, dojo.hitch(this, function(queryValue){
                // store the query value in the viewlet's preference
                var jsonVal = dojo.toJson(queryValue);
                onOk(jsonVal, this.prefFilterId);
            }), chooserArgs);
        }));
    }
});


/*
 * The provider class to provide the table settings custom preference selection to the viewlet
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.FilteredTableSettingsProvider", 
             CustomPreferenceProvider,
{
    parentViewlet: null,
    prefSettingsId: null,
    
    constructor: function(args) {
        dojo.mixin(this, args);
    },
    
    getLabel: function(value){
        var viewer = this.parentViewlet.tableViewer || this.parentViewlet.tempBackingTableViewer;
        
        // no backing table viewer yet, can't show any display settings
        if(!viewer && !this.parentViewlet._isBackingTableReady()) {
            return "&nbsp;";
        }
        var tableViewer = viewer.tableViewer;
        var provider = tableViewer.getContentProvider();
        var settings = (value && value != -1) ? dojo.fromJson(value) : viewer.getDefaultColumnSettingsOptions();
        var columnsProfile = settings.columnsProfile;
        var len = columnsProfile.length;
        var label = "";
        var nSet = len;
        
        // insert the custom settings label first.
        if (settings.customSettings) {
            var customLabel = this.parentViewlet._getTableCustomSettingsLabel(settings.customSettings);
            if (customLabel) {
                label += customLabel;
            }
        }
        var curSet = 0;
        for (var i = 0; i < len; i++) {
            var index = columnsProfile[i].index;
            var isSettable = provider.getIsSettingChangeableForColumn(index);
            if (isSettable) {
                var title = provider.getColumnHeading(null, index);
                label += title;
                
                // add a comma to separate the display columns,
                // but not on the last column.
                curSet++;
                if (curSet < nSet) {
                    label += ', ';
                }
            }
        }
        return label;
    },
    
    getLoadingLabel: function() {
        return LabelLoading;
    },

    createChooser: function(onOk, settings, viewlet) {
        // if there's no project yet, show a message. Can't show preferences
        // without a project context.
        if(!this.parentViewlet.checkHasProjectContext()) {
            return;
        }
        
        // call the viewlet's method to ensure we've a backing table context before
        // using it.
        this.parentViewlet._useBackingTableForPreferences(dojo.hitch(this, function(tableViewer){
             tableViewer.showExternalColumnSettingsOptions(dojo.hitch(this, function(newSettings){
                var jsonVal = dojo.toJson(newSettings);
                onOk(jsonVal, this.prefSettingsId);
            }));
        }));
    }
});

/*
 * The abstract base viewlet class that hosts an artifact table.
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.AbstractFilteredTableViewlet",    
             [AbstractViewlet, ContextViewletMixin],
{
    templateString: '<div>' +
                    '  <div dojoAttachPoint="_contentArea">' +
                    '  </div>' +
                    '  <div dojoAttachPoint="errorMessageNode" class="asq-progress-area" style="display:none">' +
                    '    <div class="asq-error-image">' +
                    '      <span dojoAttachPoint="errorLabelNode" class="asq-progress-label"></span>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div style="display:none;padding:4px;" dojoAttachPoint="_message">' +
                    '    <span>${labelEnterFilters}</span>' +
                    '    <a href="javascript:{}" dojoAttachEvent="onclick:_openPreferenceSettings">${labelSelectFilters}</a>' +
                    '  </div>' +
                    '</div>',
    
    // subclass must override these attributes to supply specific table's context.
    labelEnterFilters: null,
    labelSelectFilters: null,    
    prefFilterId: null,
    prefSettingsId: null,
    prefItemPerPageId: null,
        
    // Implements Viewlet's public method to initialize the viewlet
    init: function(){
        this.inherited(arguments);
        
        // hook the text filter box to handle free text filtering.
        if (this.getSite().getFilterBox()) {
            this.getSite().getFilterBox().onChange = dojo.hitch(this, "_textFilterChanged");
            this.filterText = null;
        }
        
        var providerMap = this._prefrenceProviders = {};
        var tableQueryProvider = this.getTableQueryProvider();
        providerMap[this.prefFilterId] = new tableQueryProvider({
            parentViewlet: this,
            prefFilterId: this.prefFilterId
        });
        providerMap[this.prefSettingsId] = new com.ibm.rqm.planning.web.ui.internal.view.viewlets.FilteredTableSettingsProvider({
            parentViewlet: this,
            prefSettingsId: this.prefSettingsId
        });
        
        // initialize the project area context. If we are not using the default context,
        // setup the project area context.
        this.projectAreaContext = this.getProjectArea();
        // Need to validate if the user can access the project area set for the viewlet if it is
        // not the current project area of the application
        var isProjectAreaSet = this.projectAreaContext && this.projectAreaContext.itemId;
        this.projectAreaValidated = !isProjectAreaSet || this.projectIsAppCurrent() ? true : false;
        
        // don't truncate the title. If the secondary title is long
        // the primary title would get truncated to only several chars.
        var context = this.getSite().context;
        if (context) {
            context.truncateTitle = false;
        }
    },
    
    getFormattedUrl:function(){
		var formattedURL = this.getURLContext()+"/web/console/"+ encodeURIComponent(this.getProjectArea().name);
		if(this.configurationItemId){
			formattedURL += "/" + this.configurationItemId;
		}
		return formattedURL;
	},
    
	// Resolve the LC and GC from the selected config of the ContextViewletMixin widget.
	//
    resolveConfigContext: function(callback){
        // reset configuration attributes
        this.gcConfigValue = null;
        this.configurationItemId = null;
        
        var self = this;
        var contextDeferred = this.getContextAsync();
        contextDeferred.then(dojo.hitch(this, function(context) {
            if (context) {
                if (context.localConfigurationUri) {
                    // get item id of the local config at the end of the uri
                    var contexts = context.localConfigurationUri.split("/");
                    var configurationItemId = contexts[contexts.length - 1];
                    if(!context.isUserDefined && !this.isRemote()){
                        // using default "current config" from CCBB in a QM dashboard
                        var currentProjectArea = getCurrentPA();
                        if(currentProjectArea && this.projectAreaContext 
                                && currentProjectArea.itemId != this.projectAreaContext.itemId
                                && !context.globalConfigurationUri){
                            // current config is LC which can't be part of some other project
                            this.displayInvalidContextForProjectArea();
                            return;
                        }
                    }
                    // set the LC but can be undefined if selected context is GC
                    this.configurationItemId = configurationItemId;
                }
                
                // set the GC - the ContextViewletMixin will use the selected LC to implicitly select the GC
                this.gcConfigValue = context.globalConfigurationUri;
                if (this.gcConfigValue && this.projectAreaContext && !this.configurationItemId) {
                    // get the LC from the GC if it is not included in the config context
                    var handler = {
                        success: function(config) {
                            if(config){
                                self.configurationItemId = config.itemId;
                                callback();
                            }
                        },
                        failure: function(e){
                            if(e){
                                self._handleRunQueryErrors(e.message);
                            }
                        }
                    };
                    var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
                    ConfigurationManagementClient.getGlobalToLocalConfiguration2(responseHandler, this.projectAreaContext.itemId, this.gcConfigValue);
                    return;
                }
                callback();
            } else {
                // selected project area is not PLE enabled without any config context, continue using project default config
                this.handleRefreshWithDefaultConfig(dojo.hitch(this, function() {
                    callback();
                }), dojo.hitch(this, function(error) {
                    this._handleRunQueryErrors(error.message);
                }));
            }
        }),
        dojo.hitch(this, function(error) {
            this._handleRunQueryErrors(error.message);
        }));
    },
    
   // Refresh with the updated config resolved first.
    //
    refreshWithConfiguration: function() {
        if (this.projectAreaContext) {
            this.resolveConfigContext(dojo.hitch(this, function() {
                if (this.tableViewer) {
                    // only the config setting is changed, just update the GC and LC
                    // of the existing table view and its table viewer
                    this.tableViewer.gcConfigValue = this.tableViewer.tableViewer.gcConfigValue = this.gcConfigValue;
                    this.tableViewer.configurationItemId = this.tableViewer.tableViewer.configurationItemId = this.configurationItemId;
                }
                this.refresh();
            }));
        } else {
            this.refresh();
        }
    },
    
    // Implements Viewlet's public method to handle the refresh action.
    // Validate if the user can access the project area to avoid unnecessary configuration
    // look up that throws exception regarding configuration and not on the root cause
    //
    refresh: function() {
    	 if (this.projectAreaValidated) {
            this.refreshImpl();
        } else {
            var handler = {
                success: dojo.hitch(this, function(pa) {
                    this.projectAreaValidated = true;  // need to validate once only
                    this.refreshImpl();
                }),
                failure: dojo.hitch(this, function(error) {
                    var name =  this.projectAreaContext.name || ItemNameRegistry.getName(this.projectAreaContext.itemId);
                    this.projectAreaContext.accessError = error;
                    this._handleRunQueryErrors(error.message);
                })
            };
            var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
            ProcessClient.getProjectAreaByUUIDWithLimitedMembers({
                processAreaItemId: this.projectAreaContext.itemId,
                maxMembers: 0  // just want to check access don't waste time to fetch members
            }, responseHandler);
        }
    },
    
    // Implementation of the actual refresh action.  Subclass should override this method
    // instead of refresh() to inherited the capability in refresh().
    //
    refreshImpl: function() {	
        // create the artifact table if it has not been created yet. But make sure
        // that we've a scope context.
        if (!this.tableViewer && this.projectAreaContext) {
            var createCallback = dojo.hitch(this, function(table){
                // initialize this.tableViewer only if not yet initialized
                if (table && !this.tableViewer) {
                    // connect to the table method that shows the query results,
                    // if there's no results returned, we want to show the message
                    // and the link to let the user to configure the filter
                    this.connect(table, "showQueryResults", dojo.hitch(this, "_returnResultCheck"));
                    this.tableViewer = table;
                    dojo.place(table.domNode, this._contentArea);
                }
            });
            // resolve the config info for the table viewer
            this.resolveConfigContext(dojo.hitch(this, function() {
                // Some subclasses async load required class and return nothing.  They require
                // initialize in a callback.
                var table = this._createTableViewer(createCallback);
                // The callback here will only run once and skip if table is not yet defined.
                createCallback(table);
            }));
        }
        else {
            this._update();
        }
    },
    
    getTableQueryProvider: function(){
    	return com.ibm.rqm.planning.web.ui.internal.view.viewlets.FilteredTableQueryProvider;
    },

    // Handle Pre 4.0 M8 saved settings preferences format. 
    _handleOldColumnSettings: function(columnSettings){
        var len = columnSettings.length;
        var profiles = [];
        
        for (var i = 0; i < len; i++) {
            var profile = columnSettings[i];
            
            if (!profile.show) 
                continue;
            
            // fix typo in saved preference... for migration...
            // we have a typo when saving the viewlet preferences in
            // the profile key.. need to correct this. 
            if (profile.indiex != undefined) {
                profile.index = profile.indiex;
                delete profile.indiex;
            }
            profiles.push(profile);
        }
        return profiles;
    },
    
    // Summary:
    //   common host table creation arguments. Subclass must includes these arguments to properly
    //   configure the host table viewer for viewlet display.  
    //          
    _getHostTableCreateArgs: function(configurationItemId){
        var settings = this.getPreference(this.prefSettingsId);
        var tableArgs = {
            trim: TableConstants.TRIM_DASHBOARD,
            showFloatingStatusBar: false
        };
        
        // disable the xml binding for text clipping with ellipsis in remote
        // viewlet & if the browser is FF3. Using the binding in this situation
        // would case the text not render at all. The text would still be clipped
        // but just without the ellipsis.
        //
        if(IsFF3 && this.isRemote()) {
            tableArgs.useFF3NoBindingCSS = true;
        }
        
        // restore the settings preferences from the viewlet's preference store and don't rely
        // on the tableViewer's preferences. The tableViewer's preferences is per user, if the
        // user is not the user who added the viewlet to the project dashboard, they wouldn't see
        // those preferences.
        if(settings) {
            settings = dojo.fromJson(settings);
            var profiles = settings.columnsProfile || null;
            
            // if the viewlet preferences was saved in pre 4.0 M8 format, handle it.
            if(!profiles && settings.columnSettings) {
                profiles = this._handleOldColumnSettings(settings.columnSettings);
            }
            tableArgs.columnsProfile = profiles;
            if(settings.customSettings) {
                tableArgs.customSettings = settings.customSettings;
            }
        }
        
        // pass the config info to the table
        tableArgs.gcConfigValue = this.gcConfigValue;
        tableArgs.configurationItemId = this.configurationItemId;
        
        var pa = this.tempProjectContext || this.projectAreaContext;
        var initCallback = this._createTempBackingTable ? dojo.hitch(this, function() {
                this._tempBackingTableAvaiable = true;
                if(this.waitingDialog) {
                    this.waitingDialog.close();
                    delete this.waitingDialog;
                }
                if(this.preferenceQueued) {
                    this.preferenceQueued(this.tempBackingTableViewer);
                    delete this.preferenceQueued;
                }
            }) : dojo.hitch(this, "_update");
        
        // also pass in the root url context.
        pa.rootUrl = this.getURLContext();
        var args = {
            showQueryFilters: true,
            isSingleSelect: false,
            enableRowSelection: false,
            showExplorer: false,
            rowsSelectable: false,
            pageSize: this._getItemsPerPage(),
            
            // specify the project context for our
            // queries
            projectAreaContext: pa,
            
            // don't run the initQuery
            _runOnOpen: false,
            doRunInitQuery: false,
            
            // when the table completed its initialization, runs
            // the _update method to initiate the query and display
            // of the result data in the table.
            onInitializeCompleted: initCallback,
            gcConfigValue: this.gcConfigValue,
            tableArgs: tableArgs
        };
        if(this.configurationItemId){
        	args.configurationItemId = this.configurationItemId;
        }
        return args;
    },
    
    // Summary:
    //   Abstract method must be implemented by subclasses to create the 
    //   concrete table viewer for the viewlet. The artifact table viewer 
    //   to be created must be a subclass of the AbstractExplorableTableView 
    //   that supports both saved queries & queries filters
    //
    _createTableViewer: function() {
        console.debug("Subclass must implement this method: _createTableViewer & create a table of type AbstractExplorableTableView!!!");
    },

    // Summary:
    //   switch showing the message area to select filters, or to show the content
    //   if there's content to display.
    //
    _updateContentDisplay: function(showContent) {
        // show the message if there is no filters
        dojo.style(this._message, "display", (!showContent) ? "" : "none");
        dojo.style(this._contentArea, "display", (showContent) ? "" : "none");
        
        // show the project area's title as well.
        if(this.projectAreaContext && this.projectAreaContext.itemId) {
            var name =  this.projectAreaContext.name || ItemNameRegistry.getName(this.projectAreaContext.itemId);
            if (name) {
                this.getSite().setSecondaryTitle("(" + name + ")");
            }
        }          
        if(showContent && this.tableViewer && this.tableViewer.tableViewer) {
            this.tableViewer.tableViewer.refresh();
        }
    },
    
    // Summary:
    //   Configures the artifact table with the viewlet's save preferences and runs the
    //   current filters/query preference.
    //
    _update: function() {
        var filters = this.getPreference(this.prefFilterId);
        var hasContext = (filters != -1 && this.projectAreaContext);

        // clear error message area.
        this._clearErrorMessage();
        
        // show the content area if there is filters set.
        this._updateContentDisplay(hasContext);
        
        // if there is a valid context, i.e., project scope & filter/query 
        // saved in the preference, run it.
        if (hasContext) {
            var filtersObj = dojo.fromJson(filters);            
            var itemsPerPage = this.getPreference(this.prefItemPerPageId);
            var tableViewer = this.tableViewer.tableViewer;
            var provider = tableViewer.getContentProvider();

            // set the filter text in the content provider, the query will pick up
            // this search text to pass into the query and set the page size
            provider.filterText = this.filterText || null;
            this.tableViewer.pageSize = itemsPerPage;
            this.tableViewer.tableViewer.setContentPageSize(itemsPerPage);
            
            // check if it is in a shared dashboard and enforce to use shared query if a query is used
            if (this.isSharedDashboard() && filtersObj.query && filtersObj.query.scope && filtersObj.query.scope != FiltersQueriesClient.publicScopeId) {
                this._handleRunQueryErrors(Constants.getMessage("msgSharedQueryRequired", filtersObj.label));
                return;
            }
            
            // call the host table's run method to execution the filters
            this.tableViewer.runExternalFiltersQueriesOptions(filtersObj, /*not updateOnly, do relayout */false, dojo.hitch(this, '_handleRunQueryErrors'));
        }        
    },
  
    /**
     * For viewlets, the setting is stored as "parentArtifactCategories (for TCER specifically), 
     * we need to transform it to "categories" instead, the table will do the rest.  
     */
    _cleanupCategoryFilters: function(filterObj){
    	if(filterObj){
    		for(var key in filterObj){
    			if(key.indexOf("parentArtifactCategories")!= -1){
    				var nkey =  key.replace("parentArtifactCategories", "categories");
    				filterObj[nkey] = filterObj[key];
    				delete filterObj[key];
    			}
    		}  
    	}
    	return filterObj;
    },

    _clearErrorMessage: function() {
        dojo.style(this.errorMessageNode, "display", "none");
        this.errorLabelNode.innerHTML = "";        
    },
    
    _handleRunQueryErrors: function(msg) {
        dojo.style(this.errorMessageNode, "display", "");
        this.errorLabelNode.innerHTML = msg;
        this._returnResultCheck();
    },
    
   _returnResultCheck: function() {
        var content = this.tableViewer && this.tableViewer.tableViewer ? this.tableViewer.tableViewer.getContent() : null;
        var hasContent = content && content.length > 0;

        // show the content area if there's content.
        this._updateContentDisplay(hasContent);
     },
    
    // Summary:
    //    Implements the viewlet method to return our custom preferences' provider
    //
    getPreferenceProvider: function(/* string */ preferenceId) {
       return this._prefrenceProviders[preferenceId];
    },

    // Summary:
    //    Implements the viewlet method to handle preferences changes
    //
    settingsChanged: function(oldSettings, newSettings) {
        var prefs = newSettings.preferences;
        var filters = prefs[this.prefFilterId];
        var settings = prefs[this.prefSettingsId];
        var itemsPerPage = prefs[this.prefItemPerPageId];
        var tableViewer = this.tableViewer;
        var pageSizeChanged = (tableViewer) ? (itemsPerPage != this.tableViewer.pageSize) : true;
        var configChanged = oldSettings.configurationContext != newSettings.configurationContext;
        
        // commit any pending preferences reset
        this.commitPendingResetPref();
        
        // the project scopt had changed, throw away our table and refresh to recreate
        if(oldSettings.scopeItemId != newSettings.scopeItemId) {
            // update the project context and recreate the
            // backing table viewer
            this.projectAreaContext = {
                name: ItemNameRegistry.getName(newSettings.scopeItemId),
                itemId: newSettings.scopeItemId
            };
            if(this.tempProjectContext) {
                delete this.tempProjectContext;
            }
            this._clearTempBackingTable();
            if (tableViewer) {
                tableViewer.destroyRecursive();
                this.tableViewer = null;
            }
            setTimeout(dojo.hitch(this, "refreshWithConfiguration"));
            return;
        }
        
        if(settings && settings != -1 && tableViewer) {
            var settingObj = dojo.fromJson(settings);
            
            // we are storing the column settings in our viewlet's to have 
            // consistent layout where ever it's displayed. Not need to save
            // the settings into the user's pref.
            tableViewer.changeExternalColumnSettingsOptions(settingObj, /*don't save*/false);
        }
        // update the display if anything changed, or we don't have a 
        // table viewer yet.
        if(filters != -1 || pageSizeChanged || settings != -1 || !tableViewer || configChanged) {
            this.refreshWithConfiguration();
        }
    },
    
     settingsCanceled: function(canceledSettings, restoredSettings) {
        if (this.pendingResetPref) {
            delete this.pendingResetPref;
        }
    },

    _getItemsPerPage: function() {
        return this.getPreference(this.prefItemPerPageId);
    },
    
    // Summary:
    //   Monitor's the viewlet's free form text filter box entry. Applies
    //   text filter to our query's text filter.
    //
    _textFilterChanged: function(filterText) {
        // free form filter text changed, set the filter text
        // and re-run the query.
        if(this.filterText != filterText) {
            this.filterText = filterText;
            this.refresh();
        }
    },
    
    _getTableCustomSettingsLabel: function(customSettings) {
        // subclass tables that support custom settings need to
        // override this method supply additional labels for the custom
        // settings.
        return null;    
    },
    
    _openPreferenceSettings: function(evt) {
        if(evt) {
            dojo.stopEvent(evt);
        }
        this.getSite().toggleSettings();
    },
    
    _clearTempBackingTable: function() {
        if(this.tempBackingTableViewer) {
            this.tempBackingTableViewer.destroyRecursive();
            delete this.tempBackingTableViewer;
        } 
        if(this._createTempBackingTable) {
            delete this._createTempBackingTable;
        }
        if(this._createTempBackingTable != undefined) {
            delete this._createTempBackingTable;
        }
    },
    
    _isBackingTableReady: function() {
        return this._createTempBackingTable && this._tempBackingTableAvaiable;
    },
    
    _useBackingTableForPreferences: function(clientMethod) {
        // the backing table isn't ready yet, queue up the 
        // callback and call it when the table is initialized
        if(this._createTempBackingTable && !this._tempBackingTableAvaiable) {
            this.preferenceQueued = clientMethod;

            // show the waiting dialog to block any further clicking...
            this.waitingDialog = new AlertDialog({
                hideOk: true,
                message: Constants.getMessage("msgWaitSwitchProjectContext"),
                primaryTitle: ""                
            });
            this.waitingDialog.open();
        }
        else {
            clientMethod(this.tempBackingTableViewer || this.tableViewer);
        }
    },
    
    checkHasProjectContext: function() {
        var pa = this.tempProjectContext || this.projectAreaContext;
         if(!pa || pa.accessError) {
            var msg = new AlertDialog({
                message: Constants.getMessage("msgSelectProjectContext"),
                primaryTitle: ""                
            });
            msg.open();
        }
        return pa != null;
    },
    
    // @override to intercep scope level changes
    //
    settingChanging: function(/* string */ settingId, /* object */ workingCopy) {
        // clear pending reset of this settingId if there is one
        this.clearPendingResetPref(settingId);
        
        if(settingId == PREF_SCOPE_ITEM && workingCopy.scope == PREF_SCOPE_PA) {
            // allow other pref settings to reset from changing project area
            this.onProjectAreaChanging();
            
            var pa = this.tempProjectContext || this.projectAreaContext;
            
            // the user had changed the project scope. Create a new
            // backing table viewer for the new project scope. Or we 
            // don't have a tableviewer yet, just create the temp backing
            // table to serve the preferences.
            if (!pa || pa.itemId != workingCopy.scopeItemId || !this.tableViewer) {
                this.tempProjectContext = {
                    itemId: workingCopy.scopeItemId
                };
                this._clearTempBackingTable();
                this._createTempBackingTable = true;
                this._tempBackingTableAvaiable = false;
                // Some subclasses load the table viewer class asynchronously
                // So initialize the 'tempBackingTableViewer" variable in callback for those cases
                this.tempBackingTableViewer = this._createTableViewer(dojo.hitch(this, function(table){
                	this.tempBackingTableViewer = table;
                }));
            }
        }
    },
    
    // Changing the project area might invalidate other preference settings.  We don't reset
    // the preference right away until the user committed the changes in settingsChanged().
    //
    // Subclass can override using the method addPendingResetPref()
    //
    onProjectAreaChanging: function() {
        // reset the filters preference
        this.addPendingResetPref(this.prefFilterId, "-1");
    },
    
    // Add a pending reset preference and update the UI label of the setting.
    //
    addPendingResetPref: function(prefKey, value) {
        if (!this.pendingResetPref) {
            this.pendingResetPref = {};
        }
        this.pendingResetPref[prefKey] = value;
        // update the ui
        this.getSite().viewletAdapter._settingsPanel._updateCustomPreferenceLabel(prefKey, value);
    },
    
    // Remove the pending reset if the setting is explicitly changed in settingChanging().
    //
    clearPendingResetPref: function(prefKey) {
        if (this.pendingResetPref && this.pendingResetPref[prefKey]) {
            delete this.pendingResetPref[prefKey];
        }
    },
    
    // Use in settingsChanged() to commit the pending reset.
    //
    commitPendingResetPref: function() {
        if (this.pendingResetPref) {
            // there are pending preferences to reset
            for (var key in this.pendingResetPref) {
                this.setPreference(key, this.pendingResetPref[key]);
            }
            delete this.pendingResetPref;
        }
    },
    
    // @override to clean up any temp backing table that we might have
    // created during a settings change session.
    settingsCanceled: function(/* object */ canceledSettings, /* object */ restoredSettings) {
        this._clearTempBackingTable();
    }      
});

})();
