/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.viewlets.TestArtifactViewlet");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.viewlets.AbstractFilteredTableViewlet");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView");
dojo.require("jazz.core.loader");

(function() {
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;
var AbstractFilteredTableViewlet = com.ibm.rqm.planning.web.ui.internal.view.viewlets.AbstractFilteredTableViewlet;
var ItemNameRegistry		= com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var ImageCache				= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility					= com.ibm.rqm.planning.web.ui.internal.util.Utility;
var FiltersQueriesClient 	= com.ibm.asq.common.web.client.internal.FiltersQueriesClient;
var PrefTypeId				= "com.ibm.rqm.planning.web.testartifact-viewlet-preference-type";
var PrefQueryId             = "com.ibm.rqm.planning.web.testartifact-viewlet-preference-query";
var PrefDisplayId           = "com.ibm.rqm.planning.web.testartifact-viewlet-preference-display";
var PrefItemPerPageId       = "com.ibm.rqm.planning.web.testartifact-viewlet-preference-resultsperpage";
var LabelEnterFilters       = Constants.getMessage("labelNoTestArtifactFilterPrompt");
var LabelSelectFilters      = Constants.getMessage("labelSelectTestArtifactFilter");

/*
 * The provider class to provide the table query/filter custom preference selection to the viewlet
 */
dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.TestArtifactFilteredTableQueryProvider", 
             com.ibm.rqm.planning.web.ui.internal.view.viewlets.FilteredTableQueryProvider,
{
    createChooser: function(onOk, settings, viewlet){
        // if there's no project yet, show a message. Can't show preferences
        // without a project context.
        if(!this.parentViewlet.checkHasProjectContext()) {
            return;
        }
        // call the viewlet's method to ensure we've a backing table context before
        // using it.
        this.parentViewlet._useBackingTableForPreferences(dojo.hitch(this, function(tableViewer){
            var preferences = settings.preferences;
            var filters = preferences[this.parentViewlet.prefFilterId];
            var filtersObj = null;
            if (filters != -1) {
                filtersObj = dojo.fromJson(filters);
            }
            // add title and disableFilters parameters
            var chooserArgs = {
                title: Constants.getMessage("titleSelectQueries"),
                disableFilters: true
            };
            dojo.mixin(chooserArgs, this.composeChooserScopeArgs());
            tableViewer.showExternalFiltersQueriesOptions(filtersObj, dojo.hitch(this, function(queryValue){
                // store the query value in the viewlet's preference
                var jsonVal = dojo.toJson(queryValue);
                onOk(jsonVal, this.prefFilterId);
            }), chooserArgs);
        }));
    }
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.viewlets.TestArtifactViewlet",    
             AbstractFilteredTableViewlet,
{
    // @override artifact view specific attributes
    labelEnterFilters: LabelEnterFilters,
    labelSelectFilters: LabelSelectFilters,    
    prefFilterId: PrefQueryId,
    prefSettingsId: PrefDisplayId,
    prefItemPerPageId: PrefItemPerPageId,
    artifactNameMap: {},
    artifactIconMap: {},
    
    // @override
    postCreate: function(){
    		this.inherited("postCreate", arguments);
    		
    		this.artifactNameMap[Constants.MODEL_TYPE_PLAN] 	= Constants.getMessage("labelTestPlan");
    		this.artifactNameMap[Constants.MODEL_TYPE_CASE] 	= Constants.getMessage("labelTestCase");
    		this.artifactNameMap[Constants.MODEL_TYPE_SCRIPT] 	= Constants.getMessage("labelTestScript");
    		this.artifactNameMap[Constants.MODEL_TYPE_SUITE] 	= Constants.getMessage("labelTestSuite");
    		this.artifactNameMap[Constants.MODEL_TYPE_TESTCASE_EXECUTION_RECORD] 	= Constants.getMessage("labelTestExecutionRecord");
    		this.artifactNameMap[Constants.MODEL_TYPE_SUITE_EXECUTION_RECORD] 		= Constants.getMessage("labelSuiteExecutionRecord");
    		this.artifactNameMap[Constants.MODEL_TYPE_RESULT] 			= Constants.getMessage("labelTestcaseExecutionResult");
    		this.artifactNameMap[Constants.MODEL_TYPE_SUITE_LOG] 		= Constants.getMessage("labelSuiteResult");
    		
    		
    		this.artifactIconMap[Constants.MODEL_TYPE_PLAN] = ImageCache.TEST_PLAN;
    		this.artifactIconMap[Constants.MODEL_TYPE_CASE] = ImageCache.TEST_CASE;
    		this.artifactIconMap[Constants.MODEL_TYPE_SCRIPT] 	= ImageCache.TEST_SCRIPT;
    		this.artifactIconMap[Constants.MODEL_TYPE_SUITE] 	= ImageCache.TEST_SUITE;
    		this.artifactIconMap[Constants.MODEL_TYPE_TESTCASE_EXECUTION_RECORD] 	= ImageCache.EWI;
    		this.artifactIconMap[Constants.MODEL_TYPE_SUITE_EXECUTION_RECORD] 		= ImageCache.EXECUTION_SUITE;
    		this.artifactIconMap[Constants.MODEL_TYPE_RESULT] 			= ImageCache.EXECUTION_RESULT;
    		this.artifactIconMap[Constants.MODEL_TYPE_SUITE_LOG] 		= ImageCache.EXECUTION_RESULT;
    },
    
    // @override AbstractFilteredTableViewlet
    settingsChanged: function(oldSettings, newSettings) {
    	// commit any pending preferences reset
        this.commitPendingResetPref();
        
    	var oldPre = oldSettings.preferences;
    	var newPre = newSettings.preferences;
    	// if(oldPre[PrefQueryId] != newPre[PrefQueryId]){
    		var artifactType = newPre[PrefTypeId];
    		var queryJson = dojo.fromJson(newPre[PrefQueryId]);
    		this.getSite().setTitle(this.artifactNameMap[artifactType] + ' : ' + (queryJson.label ? queryJson.label : Constants.getMessage("labelAttributes")));
    	// }
    	
    	if(oldPre[PrefTypeId] != newPre[PrefTypeId]){
    		// var artifactType = newPre[PrefTypeId];
    		this.getSite().setIcon(this.artifactIconMap[artifactType]);
    		
        	this._clearTempBackingTable();
        	var tableViewer = this.tableViewer;
            if (tableViewer) {
                tableViewer.destroyRecursive();
                this.tableViewer = null;
            }
            this.settingWorkingCopy = null;
            // If project scope is same, then refresh and return
            if(oldSettings.scopeItemId == newSettings.scopeItemId) {
	            setTimeout(dojo.hitch(this, "refresh"));
	            return;
            }
        }
    	
    	this.inherited("settingsChanged", arguments);
    },
    
    _returnResultCheck: function(hasContent){
    	this.inherited("_returnResultCheck", arguments);
    	var queryPref = this.getPreference(PrefQueryId);
		if (queryPref != "-1"){
			var queryObj = dojo.fromJson(queryPref);
			var query = queryObj.query;
            if(query){
            	var artifactType = this.getPreference(PrefTypeId);
			
            	var title = this.artifactNameMap[artifactType] + ' : ' + (queryObj.label ? queryObj.label : Constants.getMessage("labelAttributes"));
            	title += "(" + this.tableViewer.totalSize + ")";
            	this.getSite().setTitle(title);
            }
		}
    },
    
    // @override AbstractFilteredTableViewlet
    settingChanging: function(/* string */ settingId, /* object */ workingCopy) {
        this.inherited("settingChanging", arguments);
        
        if(settingId == PrefTypeId) {
        	var tableViewer = this.tableViewer;
        	if (tableViewer) {
                tableViewer.destroyRecursive();
                this.tableViewer = null;
            }
        	this.settingWorkingCopy = workingCopy;
        	
			
			this._clearTempBackingTable();
            this._createTempBackingTable = true;
            this._tempBackingTableAvaiable = false;
                
            this.addPendingResetPref(PrefQueryId, "-1");
            this.addPendingResetPref(PrefDisplayId, "-1");
        		
            this._createTableViewer(dojo.hitch(this, function(table){
            	this.tempBackingTableViewer = table;
            }));
            var prefsToUpdate = [];
            prefsToUpdate.push(PrefQueryId);
            return prefsToUpdate;
        }
    },
    
    getTableQueryProvider: function(){
    	return com.ibm.rqm.planning.web.ui.internal.view.viewlets.TestArtifactFilteredTableQueryProvider;
    },
    
    getTitleHyperlinkURL: function() {
    	var queryPref = this.getPreference(PrefQueryId);
		if (queryPref != "-1"){
			var queryObj = dojo.fromJson(queryPref);
            var query = queryObj.query;
            if(!query){
            	return null;
            }
			var artifactType = query.type;
			var args = {};
			args.sqScope = query.scope;
            args.sqItemId = query.itemId;
	        args.sqTitle = encodeURIComponent(query.title || query.name);
	        args.isPublic = (query.scope == FiltersQueriesClient.publicScopeId);
	        args.inDropdown = artifactType;
			if (query.folder) {
                args.sqFolder = encodeURIComponent(dojo.clone(query.folder));
            }
			var formattedURL = this.getFormattedUrl();
			var url = null;
			switch(artifactType) {
			case FiltersQueriesClient.TEST_PLAN_QUERY_KEY:
				url = formattedURL+ Utility.getTPlanListingPageURI(args);
				break;
			case FiltersQueriesClient.TEST_CASE_QUERY_KEY:
				url = formattedURL+ Utility.getTCaseListingPageURI(args);
				break;
			case FiltersQueriesClient.TEST_SCRIPT_QUERY_KEY:
				url = formattedURL+ Utility.getTScriptListingPageURI(args);
				break;
			case FiltersQueriesClient.TEST_SUITE_QUERY_KEY:
				url = formattedURL+ Utility.getTSuiteListingPageURI(args);
				break;
			case FiltersQueriesClient.TER_QUERY_KEY:
				url = formattedURL+ Utility.getTCERListingPageURI(args);
				break;
			case FiltersQueriesClient.SER_QUERY_KEY:
				url = formattedURL+ Utility.getTSERListingPageURI(args);
				break;
			case FiltersQueriesClient.TER_TC_RESULT_QUERY_KEY:
				url = formattedURL+ Utility.getTCResultListingPageURI(args);
				break;
			case FiltersQueriesClient.TER_TS_RESULT_QUERY_KEY:
				url = formattedURL+ Utility.getTSResultListingPageURI(args);
				break;
			}

			// cache the query so the list view can get to the filter attributes if
			// the query is private from other user
			dojo.setObject("PHANTOM-QUERY-"+query.itemId, query);

			return url;
		}
		return null;
    },

    // @override AbstractFilteredTableViewlet
    _createTableViewer: function(callback){
        var args = this._getHostTableCreateArgs();
        args.noPersonalQueries = false;;
        args.showHeaderInDashBoardViewlet = true;
        args.showFloatingStatusBar = false;
        
        var type = this.getPreference(PrefTypeId);
        
        if(this.settingWorkingCopy){
        	type = this.settingWorkingCopy.preferences[PrefTypeId];
        	args.tableArgs.columnsProfile = null;
        }
        
        var tpArgs = dojo.mixin(args, {
            //viewProfile: 'dashboard-trace',
            showAspectViews: true,
            showExplorer: false,
            rowsSelectable: false,
            supportExecutionStatus: false,
            tableViewModeId: "general",
            isInEditor: true
        });
        
        this.getSite().setIcon(this.artifactIconMap[type]);
        
        var tableConstructorName = null;
        
        switch (type) {
            case Constants.MODEL_TYPE_CASE:
                tableConstructorName = "com.ibm.rqm.planning.web.ui.internal.view.table.TestCaseTableView";
                tpArgs.noAspectAction = true;
                tpArgs.tableId="DASHBOARD-TESTCASE";
                tpArgs.profile="DASHBOARD_VIEWLET";
                break;
            case Constants.MODEL_TYPE_SCRIPT:
                tableConstructorName = "com.ibm.rqm.planning.web.ui.internal.view.table.TestScriptTableView";
                tpArgs.tableId="DASHBOARD-TESTSCRIPT";
                tpArgs.profile="DASHBOARD_VIEWLET";
                break;
            case Constants.MODEL_TYPE_SUITE:
                tableConstructorName = "com.ibm.rqm.execution.web.ui.internal.view.table.TestSuiteTableView";
                tpArgs.tableId="DASHBOARD-TESTSUITE";
                tpArgs.profile="DASHBOARD_VIEWLET";
                break;
            case Constants.MODEL_TYPE_TESTCASE_EXECUTION_RECORD:
                tableConstructorName = "com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionWorkItemTableView";
                tpArgs.isInDashboard = true;
                tpArgs.profile="DASHBOARD_VIEWLET";
                tpArgs.tableId="DASHBOARD-TESTCASEEXECUTIONRECORD";
                break;
            case Constants.MODEL_TYPE_SUITE_EXECUTION_RECORD:
                tableConstructorName = "com.ibm.rqm.execution.web.ui.internal.view.table.SuiteExecutionRecordTableView";
                tpArgs.profile="DASHBOARD_VIEWLET";
                tpArgs.tableId="DASHBOARD-TESTSUITEEXECUTIONRECORD";
                break;
            case Constants.MODEL_TYPE_RESULT:
                tableConstructorName = "com.ibm.rqm.execution.web.ui.internal.view.table.ExecutionLogTableView";
                tpArgs.profile="DASHBOARD_VIEWLET";
                tpArgs.tableId="DASHBOARD-TESTCASERESULT";
                break;
            case Constants.MODEL_TYPE_SUITE_LOG:
                tableConstructorName = "com.ibm.rqm.execution.web.ui.internal.view.table.SuiteLogTableView";
                tpArgs.profile="DASHBOARD_VIEWLET";
                tpArgs.tableId="DASHBOARD-TESTSUITERESULT";
                break;
            case Constants.MODEL_TYPE_PLAN:
            default:
                tableConstructorName = "com.ibm.rqm.planning.web.ui.internal.view.table.TestPlanTableView";
            	tpArgs.tableId="DASHBOARD-TESTPLAN";
            	tpArgs.profile="DASHBOARD_VIEWLET";
                break;
        }
        
        var tableConstructor = dojo.getObject(tableConstructorName);
        
        if (dojo.isObject(tableConstructor)) {
        	var table = new tableConstructor(tpArgs);
        	if(callback)
        		callback(table);
        	return table;
        } else {
        	jazz.core.loader.load_async(tableConstructorName, dojo.hitch(this, function(){
        		tableConstructor = dojo.getObject(tableConstructorName);
	            var table = new tableConstructor(tpArgs);
	            if(callback)
	            	callback(table);
	        }));
        }
    }
});

})();
