/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2015. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableView");

dojo.require("dojo.date");

dojo.require("com.ibm.asq.common.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TextCellRenderer");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.process.web.client.internal.common.TeamAreaClient");
dojo.require("com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient");
dojo.require("com.ibm.rqm.process.web.ui.internal.view.extension.IterationSelector");
dojo.require("com.ibm.rqm.process.web.ui.internal.view.timebox.PlanDetailsDialog");


(function(){
    var ImageCache 						= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var TableConstants 					= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;        
    var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;    
    var ServiceResponseHandler 			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var AbstractFilteredPagingTableView = com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredPagingTableView;
    var Utility 						= com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var SaveParticipant 				= com.ibm.asq.common.web.ui.internal.model.SaveParticipant;
    var TeamAreaClient					= com.ibm.rqm.process.web.client.internal.common.TeamAreaClient;
    var ACTION_EDIT						= "Edit";
    var ACTION_PLAN_DETAILS				= "Details";
    var OslcConfigParamName             = com.ibm.asq.common.web.client.internal.Constants.OSLC_CONFIG_PARAM_NAME;
    
    dojo.declare("IterationListTableView.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        constructor: function(args){
            dojo.mixin(this, args);
        },
        
        
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        },
        
        tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, actionId)
        {
            if (this.parentView && this.parentView.requirementHeaderColumnActionInvoked) {
                this.parentView.requirementHeaderColumnActionInvoked(tableViewer, columnIndex, actionId);
            }         
        }   
      });
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableViewProvider", 
    		com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
    	
        constructor: function(viewer){
            this.tableViewer = viewer;
            
            this._columnHeadings = [];
            this._columnIdNames = [];
            this._columnWidths = [];
            
            this.COLUMNS = {                    
                    NAME: this.toColumn(Constants.getMessage("labelName"), 24, false, TableConstants.ALIGN_LEFT, "RenderType.TreeEmulation", 'name', true),
                    DESCRIPTION: this.toColumn(Constants.getMessage("labelDescriptionNoColon"), 22, false, TableConstants.ALIGN_LEFT, "RenderType.Text", 'description', true),
                    POINTS: this.toColumn(Constants.getMessage("labelPointsTimeBox"), 7, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'points', true),
                    PLANNED_DEFECTS: this.toColumn(Constants.getMessage("labelExpectedDefects"), 10, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'planned_defects', true),
                    ACTUALSTARTDATE: this.toColumn(Constants.getMessage("labelStartDateTimeBox"), 10, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EXPLICIT_DATE, 'startdate', true),
                    ACTUALENDDATE: this.toColumn(Constants.getMessage("labelEndDateTimeBox"), 10, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_EXPLICIT_DATE, 'endDate', true),
                    PLANNEDDURATION: this.toColumn(Constants.getMessage("labelDurationTimeBox"), 10, false, TableConstants.ALIGN_LEFT, TableConstants.RENDER_TYPE_TEXT, 'duration', true),
                    ACTIONS: this.toColumn(Constants.getMessage("labelActions"), 7, false, TableConstants.ALIGN_LEFT, "RenderType.MultiAction", 'actions', true)                    
                };      
            this.COLUMNS.ALL = [this.COLUMNS.NAME,this.COLUMNS.DESCRIPTION, this.COLUMNS.POINTS, this.COLUMNS.PLANNED_DEFECTS, this.COLUMNS.ACTUALSTARTDATE, this.COLUMNS.ACTUALENDDATE, this.COLUMNS.PLANNEDDURATION, this.COLUMNS.ACTIONS];
        },
        
        toColumn: function(name, width, groupable, alignment, type, rowName, isVisible, dbName) {
            return {
                name: name,
                width: width,
                groupable: groupable,
                alignment: alignment,
                type: type,
                rowName: rowName,
                isVisible: isVisible,
                dbName: dbName ? dbName : rowName
            };
        },  
                
        getNumColumns: function(content){            
            return this.COLUMNS.ALL.length;            
        },
                
        getColumnHeading: function(content, index){
            return  this.COLUMNS.ALL[index].name;
        },
                        
        getColumnIdName: function(columnIndex){
        	var column = this.COLUMNS.ALL[columnIndex];
            return column.dbName || column.rowName;
        },
       
        getIsColumnVisible: function(atColumnIndex){
        	return true;
        },
           
        getColumnHeadingAlignment: function(atColumnIndex){
        	
        	return this.COLUMNS.ALL[atColumnIndex].alignment;
        },
        
        getIsColumnGroupable: function(content, atColumnIndex){
            return false;
        },
        
        getIsColumnSortable: function(content, atColumnIndex){        
             return false;   
        },       
        
        getColumnWidth: function(content, atColumnIndex){
        	return this.COLUMNS.ALL[atColumnIndex].width;
        },
     
        getContentObjects: function(content){
            return content;
        },
        
        getRowDataId: function(rowData){          
            return "_iterationtableview" + rowData.itemId;
        },
                
        getCellDisplayType: function(rowData, atColumnIndex){
            
        	 return this.COLUMNS.ALL[atColumnIndex].type;
        },
        
        getCellAlignment: function(rowData, atColumnIndex){
            return TableConstants.ALIGN_LEFT;
        },
           
        getCellDisplayData: function(rowData, atColumnIndex){
        	var column = this.COLUMNS.ALL[atColumnIndex];
        	
            var val = "";
            switch (column.rowName) {
            case this.COLUMNS.NAME.rowName:
            	val = Utility.getIterationLabel(rowData);
                   break;
            case this.COLUMNS.DESCRIPTION.rowName:
            	val = rowData.phaseDescription;
            	break;
            case this.COLUMNS.POINTS.rowName:
            	val = rowData.expectedTotalPoints;            	
            	break;
            case this.COLUMNS.PLANNED_DEFECTS.rowName:
            	val = rowData.expectedDefects;
            	break;
            case this.COLUMNS.ACTUALSTARTDATE.rowName:
            	var date = rowData.plannedStartDate;            	
            	val = date?date:"";
            	break;
            case this.COLUMNS.ACTUALENDDATE.rowName:            	
            	var date = rowData.plannedEndDate;
            	val = date?date:"";
            	break;
            case this.COLUMNS.PLANNEDDURATION.rowName:
            	var startDate = rowData.plannedStartDate;
            	var endDate =  rowData.plannedEndDate;            	 
            	if(startDate && startDate  != "" && endDate && endDate != ""){
            		startDate =  new Date(startDate);
            		endDate = new Date(endDate);
            		var diff = dojo.date.difference(startDate, endDate, "day") + 1;
            		val = Constants.getMessage("labelDurationUnit", diff);
            	}else{
            		val = Constants.getMessage("labelDurationUnit", "0");
            	}               	
            	break;
            		           
            }
            return val;
        },
        
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
        {  
        	var column = this.COLUMNS.ALL[atColumnIndex];
            var val = {};      
            switch(column.rowName){
            case this.COLUMNS.NAME.rowName:
            	 val.label = rowData.label;            	 			
         		 var icon = this.getIterationIcon(rowData);         		         		 
            	 val.decorateImage = icon;
            	 val.alt = rowData.label;
            	break;
            case this.COLUMNS.ACTIONS.rowName:
            	var editAction = {
            		image:    ImageCache.EDIT,             
            		alt: Constants.getMessage("labelEdit"),	
            		actionId: ACTION_EDIT
            	};
            	var detailsAction = {
            		image:  ImageCache.PLAN_DETAILS, 
            		alt: Constants.getMessage("labelPlanDetails"),
            		actionId: ACTION_PLAN_DETAILS
            	};
            	val.actions = [editAction, detailsAction];       	
            	break;           
            }    		
            return val;
        },
                
        getIterationIcon: function(item){
			if(item){
 				if(item.current){
 					if(item.archived  || item.isFake){
 						return this._getFoundationImageUri("archived-iteration-current_obj.gif");   
 					}
 					return this._getFoundationImageUri("current_iteration.gif");         					 					
 				} else { 					
 					if(item.archived || item.isFake){
 						return this._getFoundationImageUri("milestone_archived_obj.gif");
 					}
 					return this._getFoundationImageUri("milestone_obj.gif");
 				}
 			}			
			return "";
		},
		
		_getFoundationImageUri:function(imgName){              	
        	var base =  dojo.moduleUrl("com.ibm.team.process.web" , "ui/internal/admin/projects/images/").toString();      			
        	return base + imgName;
        }
                            
    });
    
        
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableViewCellModifier", 
    		com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,{

    	constructor: function(args){
    		dojo.mixin(this, args);
    		this.inherited('constuctor',arguments);      		
    	},
    	
    	cellActionInvoked: function(actionId, rowObject, columnIndex)
    	{
    		if(actionId == ACTION_PLAN_DETAILS){
    			this._handlePlanDetailsClick(rowObject);    			
    		} else if(ACTION_EDIT){    		    			
    			var dialog = new com.ibm.rqm.process.web.ui.internal.view.extension.IterationEditor({
    						iteration: rowObject,
    						parentTableView: this.parentTableView
    					});    			
    		}    		
    	}, 
    	
    	_handlePlanDetailsClick: function(iteration){
    		var iterationCopy = dojo.clone(iteration); //planDetailsDialog reformats the "context" and we don't want that
    		var planDetailsDataObj = iteration.testPhaseDetails || iteration.phaseDetails;
    		var planDetailsDataArray = null;
    		var errorMessage=null;
    		var self = this;
    		if (iteration && (!iteration.startDate || !iteration.endDate || iteration.startDate == "" || iteration.endDate == "" )){
    			errorMessage = Constants.getMessage("labelDetailsError");  //TODO UA help with this message
    		}
    		if(this.parentTableView && this.parentTableView.parentSection && this.parentTableView.parentSection.isSectionDirty()){
    			errorMessage = Constants.getMessage("msgUnsavedChange");    			
    		}
    		
    		if (!errorMessage) {
    			if (planDetailsDataObj) {
    				planDetailsDataArray = new Array();
    				var len = planDetailsDataObj? planDetailsDataObj.length: 0;
    				for (var i = 0; i <len; i++) {
    					var detailObject = planDetailsDataObj[i];
    					var date;
    					if(detailObject.dateMilliseconds!= 0){
    						date = new Date(detailObject.dateMilliseconds);
    					} else {
    						date = com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString(planDetailsDataObj[i].date.split("T")[0]);
    					}    					
    					planDetailsDataArray.push([date, planDetailsDataObj[i].pointsAttempted, planDetailsDataObj[i].pointsCompleted, planDetailsDataObj[i].comment]);
    				}
    			}                
    			//TODO: rework planDetailsDialog, there a re a bunch of things that we don't need anymore
    			this.planDetails = new com.ibm.rqm.process.web.ui.internal.view.timebox.PlanDetailsDialog({
    				okCallback: dojo.hitch(self.parentTableView, "_planDetailsCallback"),
    				isEditable: function() { return self.parentTableView.editable; }, // GridContainerHelper expects isEditable to be a function.
    				startDate: iteration.startDate,
    				endDate: iteration.endDate,
    				testPhaseItemId: iteration.phaseItemId,
    				iterationItemId: iteration.itemId,
    				testPlanId: this.parentTableView.versionablePlanItemId,
    				planDetails: planDetailsDataArray ? planDetailsDataArray : null,
    				teamAreaEnabled: true,
    				context: iterationCopy,
    				iterationItem: iteration
    			});
    		}else{
    			var dialogArgs = {
    					'message': errorMessage
    			};
    			var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(dialogArgs);
    			alertDialog.open();
    		}
    	}
    });

    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.EmulationTreeCellRenderer", 
    		com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TextCellRenderer,
    		{    
    	
    	renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint){
    		var node = document.createElement("div");
    		var treeNode =  document.createElement("span");
    		var hasImage = this._hasDecorateImage(extendedDisplayInfo);
    		var hasLevel =  rowObject.level > 0; 

    		this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint - (hasImage ?  30 : 0));

    		if(hasLevel){
    			var level = rowObject.level;

    			for(var i = 0; i <level; i++){  
    				var icon = document.createElement("img");
    				icon.src = dojo.moduleUrl("dojo", "resources/blank.gif");  
    				    				
    				dojo.style(icon,{ //standard icon size, not need to move to CSS
    					height:"16px",
    					width: "16px"
    				}); 				    				
    				treeNode.appendChild(icon);
    			}
    		}    		
    		// render an image in front of the text if extended data specified an decorative image.
    		if (hasImage) {
    			var imageRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_IMAGE);    			
    			node = imageRenderer.addImageRendering(node, viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
    		}

    		if(rowObject.current){
    			dojo.addClass(node, "currentIterationLabel");
    		}
    		// this field is updateable, put a wrapper around the dom node.
    		if (extendedDisplayInfo && extendedDisplayInfo.isLoading != undefined && extendedDisplayInfo.inputProvider != undefined) {
    			var w = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.UpdateableTextCell({
    				'isLoading': extendedDisplayInfo.isLoading,
    				'inputProvider': extendedDisplayInfo.inputProvider,
    				'providerData': {
    					'rowObject': rowObject,
    					'columnIndex': columnIndex
    				}
    			}, node);
    			_dojo.addClass(w.domNode, TableConstants.CELL_UPDATEABLE_MARKER_CLASS_NAME);
    		}
    		node.insertBefore(treeNode, node.firstChild);
    		return node;
    	}
    		});
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.MultiActionCellIterationRenderer", 
    		com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,{    
//be careful  too many actions may degrade performance, this is intended for few actions( 1 -3 )    	
    	
    	renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    	{
    		if(!viewer.isEditable() || rowObject.isFake){
    			return document.createElement("span");   
    		}
    		var imgNodes =  [];
    		var imageCellRenderer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer();
    		var mainNode =  document.createElement("a");
    		
    		var parentRow = viewer.tableRows["_iterationtableview" + rowObject.itemId];    		
    		if(extendedDisplayInfo && 
    				extendedDisplayInfo['actions']) {

    			var actions = extendedDisplayInfo['actions'];
    			
    			var len = (actions && actions.length)?actions.length: 0;
    			var lastAnchor = null;
    			for(var i = 0; i < len; i++){
    				var action = actions[i];

    				mainNode.setAttribute("tabindex", "0");
    				mainNode.setAttribute("href", "javascript:{}");    				

    				var node =imageCellRenderer.renderCell(viewer, cellData, action, cellModifier, rowObject, columnIndex, cellWidthHint);    				
    				var anchor = document.createElement("a");
    				anchor.setAttribute("tabindex", "0");
    				anchor.setAttribute("href", "javascript:{}");

    				anchor.className = "image-action";    				
    				if(!rowObject.archived){
    					this.connect(anchor, "onclick", dojo.hitch(
    							{	'cellModifier': cellModifier,
    								'rowObject': rowObject,
    								'columnIndex':columnIndex,
    								'tooltip': this._tooltip,
    								'actionId':action.actionId
    							},this._actionClicked));      
    				} 
    				dojo.style(node, {
    					"paddingRight": "10px",
    					"display": "none"
    				});
    				imgNodes.push(node);    	
    				anchor.appendChild(node);
    				lastAnchor = anchor;
    				mainNode.appendChild(anchor);    				
    			}   
    			if (lastAnchor) {
    				this.connect(lastAnchor, "onblur", dojo.hitch(this, "_handleMouseOutTr", imgNodes));
    			}
    			if(!rowObject.archived){
    				//Special style requirement for 57767
    				this.connect(parentRow, "onmouseover", dojo.hitch(this, "_handleMouseOverTr",imgNodes));
    				this.connect(parentRow, "mouseleave", dojo.hitch(this, "_handleMouseOutTr", imgNodes));
    				this.connect(mainNode, "onfocus", dojo.hitch(this, "_handleMouseOverTr",imgNodes));
    			}
    			return mainNode;
    		}
    		else {
    			var node = document.createElement("a");
    			var divNode = document.createElement('div');
    			node.appendChild(divNode);

    			node.setAttribute("href", "javascript:{}");
    			dojo.addClass(node, "table-cell-link");
    			this._renderCellText(viewer, cellData, extendedDisplayInfo, divNode, cellWidthHint);

    			this.connect(node, "onclick", dojo.hitch(
    					{	'cellModifier': cellModifier,
    						'rowObject': rowObject,
    						'columnIndex':columnIndex,
    						'tooltip': this._tooltip
    					},this._actionClicked));            

    			return node;
    		}
    	},
    	_handleMouseOverTr: function(imgNodes, event){    	
    		var len = imgNodes.length;
    		for(var i=0; i< len; i++){
    			var obj = imgNodes[i];
    			dojo.style(obj,"display","");
    		}    	
    	},
    	
    	_handleMouseOutTr: function(imgNodes, event){    	 	    	 	
    		var len = imgNodes.length;
    		for(var i = 0; i< len; i++){
    			var obj = imgNodes[i];
    			dojo.style(obj, "display", "none");
    		}   
    	},
    	
    	_actionClicked: function(e){    		
    		dojo.stopEvent(e);

    		if(this.tooltip)
    			this.tooltip.close();

    		if(this.cellModifier)
    			this.cellModifier.cellActionInvoked(this.actionId, this.rowObject, this.columnIndex);
    	}

    });
    
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableView", [dijit._Widget, dijit._Templated, AbstractFilteredPagingTableView], {
        templateString:'<div>'+ 
        				'<table style="width:100%" >'+
        			    '<tr><td>'+
        					'<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>'+
        				'</td></tr>'+
        				'</div>',

        teamAreaUUId: "",               
        currentIterationItemId:"",    
        allIterations: [], //Temporal Array of all iterations, normally will be empty after initialization is complete  
        planItemId: "",       				
        widgetsInTemplate: true,        
        pendingSaveDetails: [],
        newIterationCounter:0,
        onInitializationCompleted: null,
        
        constructor: function(args){			
			this.pendingSaveDetails = [];
			this.allIterations = [];
			this.newIterationCounter = 0;
			this.currentIterationItemId =  null;
			this._showArchived = false;			
        },
                        
        postCreate: function(){        
        	this.editable = (this.parentSection)?this.parentSection.isEditable(): true;
        	this.teamAreaUUId = (this.teamAreaUUId)?this.teamAreaUUId:"";
        	this.currentIterationItemId = this.currentIterationItemId?this.currentIterationItemId:"";
        	this.versionablePlanItemId  = this.versionablePlanItemId?this.versionablePlanItemId:"";
            this.inherited("postCreate", arguments);                                     
            var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
                isCompactMode: !this.printable,
                allowAdjustSettings: true,
                _addPreferenceSettingsAction: function(){            	
            	},
            	editable: this.editable,
                enableExtraHeader: true,
                showGroupFilterControls: false,
                extraHeaderTitle: Constants.getMessage("labelTestScheduleTable"),
            	displayInPopup: false,
            	tableId: "RQM-ITERATIONS-VIEWER",
                autoSaveSettings: true                               
            });            
            this._tableViewerNode.appendChild(tableViewer.domNode);            
            var tableProvider = new com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableViewProvider(tableViewer);            
            tableViewer.setContentProvider(tableProvider);                        
            tableViewer.registerCellRenderer("RenderType.MultiAction", new com.ibm.rqm.process.web.ui.internal.view.extension.MultiActionCellIterationRenderer());
            tableViewer.registerCellRenderer("RenderType.TreeEmulation", new com.ibm.rqm.process.web.ui.internal.view.extension.EmulationTreeCellRenderer());
            tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("RQM-KEY-PROC-TIMEBOX-TITLE")));
            
            var cellModifier = new com.ibm.rqm.process.web.ui.internal.view.extension.IterationListTableViewCellModifier({
            	parentTableView:this
            	});
            tableViewer.setCellModifier(cellModifier);            
            
            tableViewer.setTableListener(new IterationListTableView.TableListener({
                parentView: this
            }));
            var label = document.createElement("label");
            label.innerHTML = Constants.getMessage("labelParentIterationColon");
            
            dojo.style(label, {
            	color: "#666666",
            	fontWeight: "normal"
            	});            
                                               
            tableViewer.extraHeaderActions.appendChild(label);            
            tableViewer.addExtraHeaderAction("select-iteration", Constants.getMessage("labelSelectIteration"), null,  "", dojo.hitch(this, "_openIterationDialog"));
            tableViewer.addExtraHeaderAction("select-iteration", Constants.getMessage("labelCleanIteration"), null,  "", dojo.hitch(this, "_cleanIteration"));
            
            //bypass normal addaction for table since table doesn't support 2 state button.
            this._actionsBar = new jazz.ui.SimpleToolbar();            
            this._actionsBar.addButton(this, this._displayTeamAreaArchived, {
            	label: Constants.getMessage("labelShowArchivedSchedules"),
            	imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_archived_obj.gif"), 
            	buttonType: jazz.ui.SimpleToolbar.ButtonTypes.TWO_STATE
            });

            this._actionsBar.addButton(this,this._displaySettings,{
            	label: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_changeSettings"),
            	imgSrc: dojo.moduleUrl("com.ibm.asq.common.web","ui/internal/images/icons/elcl16/managetable.gif")
            });
            tableViewer.actionsAreaNode.appendChild(this._actionsBar.domNode);
            dojo.style( this._actionsBar.domNode, "float", "right");
            
            this.tableViewer = tableViewer;            
            this.tableViewer.showLoadingProgress();
            
            var parentEditor =  (this.parentSection && this.parentSection.getParentEditor())? this.parentSection.getParentEditor(): null;            
            var isSnapshot = this.isSnapshot =  (parentEditor && parentEditor.isSnapshot());
            if(isSnapshot){
            	this._getInitialContentForSnapshots(parentEditor);
            } else {            	
            	this._getInitialContent();
            }
        },
        
        _displaySettings:function(){
       	 var defaultOkHandler = dojo.hitch(this.tableViewer, "_handleSettingsChanged");
       	 this.tableViewer.openTableSettingsDialog(defaultOkHandler);
       },
        
        _displayTeamAreaArchived: function(){        	
        	this._showArchived =  !this._showArchived;        	
        	this.refresh(this.content);
        },
        
        _openIterationDialog: function(){
        	var iterationSelector =  new com.ibm.rqm.process.web.ui.internal.view.extension.IterationSelector({
        		parent:this,
        		teamAreaUUId: this.teamAreaUUId
        	});
        },
        
        _cleanIteration: function(){
        	this.setCurrentIteration();
        	this.setDirty(true);
        },
                
        _getInitialContentForSnapshots: function(parentEditor){
        	var artifact =  parentEditor.getArtifact();
        	var versionable = artifact? artifact.versionableItemId: null;          	
        	if(versionable){
            	var responseHandler = new ServiceResponseHandler(this, "_setIterations", "_error");
            	var args = {
            			itemId: versionable
            	};
        		com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient.getIterationsLocal(responseHandler, args);
        	}
        },
        
        _getInitialContent: function(){          
        	//get the team area dev line or the default development line
        	if(this.teamAreaUUId && this.teamAreaUUId!="" && this.teamAreaUUId!="Unassigned"){        		        		
        		var responseHandler = new ServiceResponseHandler(this, "setInitialTimeLine", "_error");
    			TeamAreaClient.getTeamAreaByUUIDWithLimitedMembers(responseHandler, {
    				processAreaItemId: this.teamAreaUUId,
    				maxMembers: 1
    			});    			
        	} else {
        		var devlineResponseHandler = new ServiceResponseHandler(this, "_findCurrentDevelopmentLine", "_error");
        		TeamAreaClient.getProjectDevelopmentLines(devlineResponseHandler, null, null, this.configurationItemId);						
        	}
        },
        
        setInitialTimeLine:  function(response){        	
        	if(response && response.developmentLine){
        		var timeLine = response.developmentLine;
        		this._timeLineItemId = timeLine.itemId;
        		
        		
        		//Not sure why, but sometimes the service is breaking with certain uuids, so let's avoid using it for now
        		/*		var args = {
        				includeArchived: true,
        				uuid:timeLine.itemId
        		};
        		var responseHandler = new ServiceResponseHandler(this,"_setIterations", "_error");
        		com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient.getIterationsLocal(responseHandler, args);*/
        	} /*else */ {        		
        		var responseHandler = new ServiceResponseHandler(this, "_setIterations", "_error");
        		var args = {
        				includeArchived:true, 
        				uuid: this.teamAreaUUId				
        		};			
        		TeamAreaClient.getTeamAreaIterations(responseHandler, args);        	
        	}        	        

        },
        
        _findCurrentDevelopmentLine:function(lines){
        	var len = lines.length;
			for(var i=0;i<len;i++){
				var devLine = lines[i];
				if(devLine.projectTimeline){
					var args = {
							includeArchived: true,
							uuid:devLine.itemId
					};
					if (this.configurationItemId) {
					    args[OslcConfigParamName] = this.configurationItemId;
					}
					this._timeLineItemId = devLine.itemId;
					var responseHandler = new ServiceResponseHandler(this,"_setIterations", "_error");
					TeamAreaClient.getIterations(responseHandler, args);
					return;
				}
			}
			//Should not happend, but if we are here means that we are no development lines were found, so we should display a warning in the future?
			this._updateLegacyAttributes([]);
        },
        
        
        _setIterations: function(response){
        	this.allIterations = response;     
        	this.currentIteration = this._searchIteration(this.currentIterationItemId, this.allIterations, true);        	   
        	if(!this.currentIteration && (this.currentIterationItemId && this.currentIterationItemId!="")){
        		//something is amiss here, we have an iteration item id, but we couldn't find the iteration, this means that user directly edited/removed 
        		//the iterations or changed timeline, so we will proceed to display a warning message and prepare a fake display in order to allow deletion of phases.
        		this.prepareSemiFakePresentationForInvalidIteration();        		        		        	
        	} else {
        		this.updateLegacyPhases(this.currentIteration);
        	}        	
        },
        
        updateLegacyPhases: function(iteration){         	
        	if(this.versionablePlanItemId && this.versionablePlanItemId!=""){
        		var params = {
        				testPlanId: this.versionablePlanItemId,
        				iterationId: iteration? iteration.itemId: this.currentIterationItemId,
        				includeArchived: true,
        				returnInMillis: true
        		};
                if (this.configurationItemId) {
                    params[OslcConfigParamName] = this.configurationItemId;
                }
        		var _self = this;        		
        		var handler = {
        				success: function(response){
        					_self._legacyPhases = response;
        					var currentIteration = _self.currentIteration;
        					if((!currentIteration && iteration) || (iteration && currentIteration && currentIteration.itemId != iteration.itemId)){
        						//meaning we came with a new iteration from IterationSelector, so we need to clean up
        						_self.setCurrentIteration(iteration);        						
        					} else {
        						_self._updateLegacyAttributes();
        					}
        				},
        				error: function(){
        					_self._error();
        				}
        		};
        		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
        		com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient.getTestPhasesDTO(responseHandler, params);
        	} else {
        		//new TP
        		this._legacyPhases = [];
        		var currentIteration = this.currentIteration;
				if((!currentIteration && iteration) || (iteration && currentIteration && currentIteration.itemId != iteration.itemId)){
					//meaning we came with a new iteration from IterationSelector, so we need to clean up
					this.setCurrentIteration(iteration);        						
				} else {
					this._updateLegacyAttributes();
				}
        	}
        },                
        
        _updateLegacyAttributes: function(){              	        
        	var iteration = this.currentIteration;     	
        	if(!iteration &&  this.isSnapshot){
        		//If this is a snapshot and we couldn't get a root iteration then probably we are dealing with 3.x schedule snapshots, 
        		//so we will try to force the display  
        		this.setCurrentIterationNoRoot();
        	} else {
        		this.setCurrentIteration(iteration, this._legacyPhases, true);        	
        	}
        	delete this.allIterations; //we don't need this anymore.
        	
        	if(this.onInitializationCompleted && dojo.isFunction(this.onInitializationCompleted)){
        		this.onInitializationCompleted();
        		this.onInitializationCompleted = null;
        	}
        },  
        
        setCurrentIterationNoRoot: function(){
        	this.content =  [];
        	var len = this.allIterations?this.allIterations.length:0;
        	for(var i=0; i<len; i++){
        		var iteration = this.allIterations[i];
        		this._prepareItemPresentation(iteration, this.content, 0,  this._legacyPhases);
        	}        	      
        	  	
        	this.refresh(this.content);
        	this.tableViewer.endLoadingProgress();
        	this.tableViewer.show();
        },
        
        setCurrentIteration:function(iteration, legacy, initialProcess){
        	var changedIteration = false;
        	
        	    
        	if(        			
        		//we don't have a current iteration set
        		(!this.currentIterationItemId || this.currentIterationItemId == "") ||
        		//we have an iteration set  but the user changed it
        		(iteration && iteration.itemId != this.currentIterationItemId) ||
        		//delete current iteration
        		(!iteration &&  (this.currentIterationItemId && this.currentIterationItemId!="")))
        	{        	        	        	
        		this.currentIterationItemId = iteration? iteration.itemId: "";        	
        		changedIteration = true;
        		legacy = this._legacyPhases;
        	}        	
        	
        	//A weird case specific to child plans: You have a parent iteration set, iteration exists in timeline 
        	//but we don't have any test phases created, we must prompt user to clear schedule somehow
        	//TODO: a most appropiate message
        	if(initialProcess && iteration && iteration.itemId == this.currentIterationItemId && !legacy && !iteration.isNew){
            	this.parentSection.setParentEditorWarning(Constants.getMessage("labeldevLineNotMatch"));
        	}
        	        	       		       		
        	var oldContent = this.content;  
        	var content;
        	//merge content, iterations->Phases
        	if(iteration){
        		content = this.content =  this._prepareItemPresentation(iteration, [], 0,  legacy);
        	} else  if(initialProcess && this.getRootPhaseStatus() == 1){
        		this.parentSection.setParentEditorWarning(Constants.getMessage("labeldevLineNotMatch"));
        		//build a fake iteration
        		this._prepareItemPresentationforNullRoot(legacy);      
        		return;
        	} else {
        		content = this.content = [];
        	}
        	
        	if(changedIteration){
        		//If a new iteration was selected first we need to locate phases that need to be deleted        		
        		var len = oldContent? oldContent.length: 0;
        		for(var i=0; i < len; i++){
        			var item = oldContent[i];        			
        			//If we had any previous change or the phase was edited remove it from participants
    				this._removeSaveParticipant(item);
        			if(!this.containsIteration(item.itemId, content) && !item.isNew && item.phaseItemId){
        				//If the item is not new and it's not currently being used in the new set of iterations
        				//then we must delete it
        				this._addSaveParticipant(item, true);        			
        			}
        		}

        		//now we need to check if a new phase needs to be created/updated
        		var contLen = content? content.length: 0;
        		for(var j=0; j < contLen; j++){
        			var item = content[j];        			
        			if(item.hasDeliverable)
        				this.updateIteration(item);

        		}
        	}        		

        	this.refresh(content);
        	this.tableViewer.endLoadingProgress();
        	this.tableViewer.show();
        },  
        
        /* @return
         * -1 : phase is archived  
         *  0 : phase was not found
         *  1 : phase exists and it's not archived
         */ 
        getRootPhaseStatus: function(){
        	var len = this._legacyPhases?this._legacyPhases.length: 0;
        	for(var i=0; i< len; i++){
        		var phase = this._legacyPhases[i];
        		if(this.currentIterationItemId == phase.iterationItemId){
        			if(phase.archived)
        				return -1;
        			else 
        				return 1;
        		}
        	}
        	return 0;
        },
        
        containsIteration: function(itemId, array){
        	var len = array? array.length: 0;
        	for(var i=0; i< len; i++){
        		if(array[i].itemId == itemId){
        			return true;
        		}
        	}
        	return false;
        },
        
        updateIteration:function(iterationItem, skipSave){
        	if(!iterationItem.phaseItemId){
        		iterationItem.isNew = true; //if we don't have a phaseId then surely is a new phase
        		iterationItem.phaseItemId = ++this.newIterationCounter;        	 
        	}
        	var iteration = this._searchIteration(iterationItem.itemId);        	
        	dojo.mixin(iteration, iterationItem);                	
        	if(!skipSave)
        		this._addSaveParticipant(iteration);
        	this.refresh(this.content);
        },

        setTeamArea:function(teamAreaUUId){
        	if(this.teamAreaUUId!= teamAreaUUId){
        		this.teamAreaUUId =  teamAreaUUId;
        		if(teamAreaUUId && teamAreaUUId != ""  && teamAreaUUId != Constants.UNASSIGNED_ITERATION_CONSTANT_VALUE){        			
        			var responseHandler = new ServiceResponseHandler(this, "getAssociatedTimeline", "_error");
        			TeamAreaClient.getTeamAreaByUUIDWithLimitedMembers(responseHandler, {
        				processAreaItemId:teamAreaUUId,
        				maxMembers:1
        			});
        		} else{
        			var devlineResponseHandler = new ServiceResponseHandler(this, "getAssociatedTimeline", "_error");
        			TeamAreaClient.getProjectDevelopmentLines(devlineResponseHandler);
        		}
        	}
        },
        
        prepareSemiFakePresentationForInvalidIteration: function(){        	
        	var parentEditor =  (this.parentSection && this.parentSection.getParentEditor())? this.parentSection.getParentEditor(): null;
        	var params = {
    				testPlanId: this.versionablePlanItemId,
    				iterationId: this.currentIterationItemId,
    				includeArchived: true,
    				returnInMillis: true
    		};
            if (this.configurationItemId) {
                params[OslcConfigParamName] = this.configurationItemId;
            }
    		var _self = this;        		
    		var handler = {
    				success: function(response){            					
    					_self._prepareItemPresentationforNullRoot(response, true);
    				},
    				error: function(){
    					_self._error();
    				}
    		};
    		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
    		com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient.getTestPhasesDTO(responseHandler, params);
        	if(parentEditor){        			        		
        		parentEditor.setHeaderStatus(Constants.getMessage("titleWarning"), Constants.HEADER_STATUS_WARNING, Constants.getMessage("labeldevLineNotMatch"));
        	}
        },
                
        getAssociatedTimeline: function(response){
        	var timeLineItemId;
        	if(response && response.developmentLine){
        		timeLineItemId = response.developmentLine.itemId;        		
        	} else if( response && response.length > 0){
        		var len  = response.length;
        		for(var i=0; i< len; i++){
        			var devLine = response[i];
        			if(devLine.projectTimeline){
        				timeLineItemId = devLine.itemId;
        				break;
        			}
        		}
        	}
        	if(this._timeLineItemId != timeLineItemId){ //different timelines, we must clean table
    			this._timeLineItemId = timeLineItemId;
    		//	this.setCurrentIteration(); 
    			this.setDirty(true);
    		}
        },
        
        refresh: function(content){        	
        	var intermediateContent = this.filterArchivedIterations(content);        	
            this.tableViewer.setContent(intermediateContent?intermediateContent:[]);       
            this.tableViewer.refresh();
        },
        
        filterArchivedIterations: function(initialContent){
        	var len = initialContent?initialContent.length:0;
        	var finalContent = [];
        	for(var i=0; i< len; i++){
        		var item = initialContent[i];
        		if(this._showArchived || !item.archived){
        			finalContent.push(item);
        		}
        	}
        	return finalContent;
        },
        getContent: function(){
            return this.tableViewer.getContent();
        },

        setDirty:function(b){
        	this.parentSection.setDirty(b);
        },
        
        getCurrentParentIterationItemId:function(){
        	return (this.currentIterationItemId && this.currentIterationItemId!="")? this.currentIterationItemId:null;
        },
        
        _prepareItemPresentation:function(iteration, savedIterations, level, legacy){
			if(!savedIterations){
				var savedIterations = [];
				level = 0;				
			}
			iteration.level = level;
			if(iteration.hasDeliverable){
				var mixinLegacyAttribs;
				if(legacy && legacy.length > 0){
					mixinLegacyAttribs = this._searchLegacyIteration(iteration, legacy);   
				}			
				if(mixinLegacyAttribs){
					dojo.mixin(iteration, mixinLegacyAttribs);
				} else {
					iteration.plannedStartDate = iteration.startDate? com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString(iteration.startDate): null;
					iteration.plannedEndDate = iteration.endDate? com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString(iteration.endDate):null;
				}			
				savedIterations.push(iteration);
			}
								
			var len = iteration.iterations? iteration.iterations.length: 0;						
			for(var i = 0; i < len; i++){
				var childIteration = iteration.iterations[i];
				this._prepareItemPresentation(childIteration, savedIterations, (level + 1), legacy);
			}	
			return savedIterations;
		},
		
		_prepareItemPresentationforNullRoot: function(legacy, flagAsFake){
			var len = legacy?legacy.length: 0;
			this.content =  [];
			
			for(var i=0; i< len; i++){
				var phase = legacy[i];
				var fakeIteration = {
						label: phase.name,
						itemId: phase.iterationItemId,
						startDate: (phase.startDate && phase.startDate!="")? new Date(Number(phase.startDate)):"",
						endDate: (phase.endDate && phase.endDate!="")? new Date(Number(phase.endDate)):"",
						hasDeliverable: true,
						isFake: flagAsFake
				};				  
				this._prepareItemPresentation(fakeIteration, this.content, 0,  [phase]);
			}
			this.refresh(this.content);
        	this.tableViewer.endLoadingProgress();
        	this.tableViewer.show();			        	        	    
		},
		
		_findParticipant: function(obj){
			var itemId = obj.phaseItemId;
			var saveContext = this.parentSection.getSaveContext();						
			var iter = saveContext._participants.getIterator();
			while (!iter.atEnd()) {
				var participant = iter.get();
				if (participant.getItemType() == "com.ibm.rqm.process.TestPhase" && 
						participant.getVersionableItemId() == itemId) {
					return participant;
				}
			}
			return false;
		},
		
		_removeSaveParticipant: function(obj) {
			var participant = this._findParticipant(obj);			
			if (participant) {
				var saveContext = this.parentSection.getSaveContext();
				saveContext.removeParticipant(participant);
			}
		},
		
		_addSaveParticipant: function(iterationItem, deleteItem){		
			var self = this;
			var isNew = iterationItem.isNew;			
			var itemId = iterationItem.phaseItemId;
			if (!this.editable) {
				return;
			}		
			
			var saveParticipant = this._findParticipant(iterationItem);
			if(!saveParticipant){
				saveParticipant = new SaveParticipant();				
			}

			if(!this.newIterationCounter){
				this.newIterationCounter = 0;
			}			
			  

			saveParticipant.getItemId = function () { 
				return self.parentSection.getParentEditor().getVersionableItemId(); 
			};

			saveParticipant.setVersionableItemId(itemId);

			saveParticipant.getStateId = function(){
				return iterationItem.phaseStateId; 
			};

			saveParticipant.getItemType = function(){
				return "com.ibm.rqm.process.TestPhase";
			};

			saveParticipant.isDirty = function () {
							
				return self.parentSection.isSectionDirty();
			};	
			saveParticipant.saveFinished = function(){
				//remove from participant list, we are done
				var saveContext = self.parentSection.getSaveContext();
				saveContext.removeParticipant(this);
			};		
			saveParticipant.getFormattedAttributes =  function(){
				return self.getFormattedAttributes(iterationItem);
			};
			saveParticipant.getAttributes = function(){					
												
				var param = {};
				var deleteArray = new Array();
				var addArray = new Array();
				var updateArray = new Array();					
				
				var params = self.getFormattedAttributes(iterationItem);
				
				if(deleteItem){
					params.archived = true;
					deleteArray.push(params);
				} else if (isNew){
					params.archived = false;
					addArray.push(params);
				} else {
					params.archived = false;
					updateArray.push(params);
				}
					
				param.remove = deleteArray;
				param.update = updateArray;
				param.add = addArray;
				return {
					operationJson: dojo.toJson(param)};
			};	  
         	
			var parent = this.parentSection.getParentEditor();
			if (parent != null) {
				this.parentSection.getSaveContext().createReference(saveParticipant, "testPlanId", parent);
			}
			this.parentSection.getSaveContext().addParticipant(saveParticipant);
		},
		
		
		getFormattedAttributes: function(iterationItem){
			
			/* 
			 * -------------------------------------------------------------------------
			 * Iteration      |       Phase                |    IterationItem(Internal)
			 * -------------------------------------------------------------------------
			 * startDate      |       startDate  		   |    startDate  
			 * endDate        |       endDate 		       |    endDate
			 * N/A            |       expectedStartDate	   |    plannedStartDate
			 * N/A            |       expectedEndDate 	   |    plannedEndDate
			 * ------------------------------------------------------------------------- 
			 */

			var self = this;			
			var expectedStartDate  = (iterationItem.plannedStartDate && iterationItem.plannedStartDate != "")?(new Date(Number(iterationItem.plannedStartDate))):"";  					
			var expectedEndDate = (iterationItem.plannedEndDate && iterationItem.plannedEndDate != "")?(new Date(Number(iterationItem.plannedEndDate))):"";      							
			var expectedDuration = (expectedStartDate != "" && expectedEndDate != "")?(Constants.getMessage("labelDurationUnit", dojo.date.difference(expectedStartDate, expectedEndDate))): "0";							
			var params = {						
					name: iterationItem.label,
					description: iterationItem.phaseDescription? iterationItem.phaseDescription: "",
					expectedDuration: expectedDuration, 
					itemId: iterationItem.phaseItemId,
					expectedTotalPoints: iterationItem.expectedTotalPoints?iterationItem.expectedTotalPoints:0,
					expectedValidityRate:iterationItem.expectedValidityRate?iterationItem.expectedValidityRate:0,
					expectedDefects: iterationItem.expectedDefects?iterationItem.expectedDefects:0,
					expectedStartDate: (expectedStartDate && expectedStartDate!="")?"UTCMillis"+expectedStartDate.getTime():"",
					expectedEndDate: (expectedEndDate && expectedEndDate!="")?"UTCMillis"+expectedEndDate.getTime():"",
					iterationItemId: iterationItem.itemId,
					tableType: "phase",						
					testPlanId: (self.versionablePlanItemId && self.versionablePlanItemId!="")?self.versionablePlanItemId: null
			};		
			return params;
		},
		
		
		//DeepSearch will look recursivelly on .iteration children 
		_searchIteration:function(itemId, array, deepSearch){
        	var content = (array)?array:this.content;
        	var len = content?content.length:0;
        	for(var i = 0; i<len; i++){        		
        		if(content[i].itemId == itemId){
        			return content[i];
        		} else if (content[i].iterations && deepSearch){
        			var iteration = this._searchIteration(itemId, content[i].iterations, deepSearch);
        			if(iteration){
        				return iteration;
        			}
        		}
        	}
        },

        _searchLegacyIteration:function(iterationItem, array){

        	/* 
        	 * 
        	 * -------------------------------------------------------------------------
        	 * Iteration      |       Phase                |    IterationItem(Internal)
        	 * -------------------------------------------------------------------------
        	 * startDate      =        startDate			=     startDate  
        	 * endDate        =        endDate				=     endDate
        	 * N/A            |    expectedStartDate     	=    plannedStartDate
        	 * N/A            |    expectedEndDate          =    plannedEndDate
        	 *  
        	 */
        	var len = array? array.length: 0;        	
        	for(var i = 0; i<len; i++){
        		var phase = array[i];        		
        		if(phase.iterationItemId == iterationItem.itemId || phase.id == iterationItem.id){          			
        			var plannedStartDate = (phase.expectedStartDate && phase.expectedStartDate != "")?
        					new Date(Number(phase.expectedStartDate))
        						:"";
        			var plannedEndDate = (phase.expectedEndDate && phase.expectedEndDate != "")?
        					new Date(Number(phase.expectedEndDate))
        						:"";        		
        			return {
        					phaseItemId:phase.itemId? phase.itemId: null,
        					expectedTotalPoints: phase.expectedTotalPoints,
        					phaseStateId: phase.stateId,
        					expectedValidityRate: phase.expectedValidityRate,
        					expectedDefects:  phase.expectedDefects,        					
        					phaseDescription: phase.description,
        					plannedStartDate: (plannedStartDate!= "")? plannedStartDate:"",
        					plannedEndDate:(plannedEndDate!= "")?plannedEndDate:"",
        					testPhaseDetails: phase.testPhaseDetails
        					};
        		}
        	}
        },
               
        _planDetailsCallback: function(data){
        	this.setDirty(true);        	        
        	var detailsIndex = this._searchSavedDetails(data.context.phaseItemId);
        	
        	this.updateIteration({
        		itemId:data.context.itemId,
        		phaseDetailJsonString: data.phaseDetailJsonString,
        		expectedTotalPoints: data.expectedTotalPoints,
        		phaseItemId: data.itemId
        	});
        	
        	  	        			
        	var detailsUpdateItem =  {        	
        			phaseItemId: data.context.phaseItemId,
                    phaseDetailJsonString: data.phaseDetailJsonString
         	};
        	if(detailsIndex != -1){        		
        		this.pendingSaveDetails.push(detailsUpdateItem);
        	} else{
        		this.pendingSaveDetails[detailsIndex + 1] =  detailsUpdateItem;
        	}        	
        },
        
        //this way of saving phasedetails is really weird, we should really consider restructuring how this works 
        //and include phasedetails into drawservice probably
        followupSaveOperation: function(){        	
        	this.tableViewer.showLoadingProgress();
        	var len = this.pendingSaveDetails.length;
        	var _self = this;        	
        	for(var i = 0; i < len; i++){
        		var phasePendingSaveObject = this.pendingSaveDetails[i];        		
        		var saveParticipant = this._findParticipant(phasePendingSaveObject);
        		var attribs = saveParticipant.getFormattedAttributes();
        		attribs.phaseDetailJsonString =  phasePendingSaveObject.phaseDetailJsonString;
        		var clbObj = {
        				success: function(response){
        					_self._reinitialize();
        				},

        				failure: function(){     
        					this.tableViewer.endLoadingProgress();
        		        	this.tableViewer.show();
        					console.debug("IterationListTableView - Failed saving details " + i +" of "+ len  );
        				}
        		};

        		var responseHandler = new ServiceResponseHandler(clbObj, "success", "failure");
        		com.ibm.rqm.process.web.client.internal.timebox.TimeboxClient.postUpdateTestPhase(responseHandler, attribs);
        	} 
        	if(len == 0){        		
        		this._reinitialize();
        	}        	
        },
        
        _reinitialize:function(){
        	this.pendingSaveDetails = [];        	
        	var versionable = (this.parentSection.getParentEditor() && this.parentSection.getParentEditor().getVersionableItemId())?this.parentSection.getParentEditor().getVersionableItemId():"";
        	this.versionablePlanItemId = versionable;
        	if(this.currentIterationItemId && this.currentIterationItemId != "" && this.versionablePlanItemId && this.versionablePlanItemId!=""){
            	this._getInitialContent();
            } else {
            	this.tableViewer.endLoadingProgress();
            	this.tableViewer.show();
            }        	
        },
        _searchSavedDetails:function(phaseItemId){
        	var len = (this.pendingSaveDetails && this.pendingSaveDetails.length > 0)? this.pendingSaveDetails.length : 0;
        	
        	for(var i = 0; i < len; i++){
        		if(this.pendingSaveDetails[i].itemId == phaseItemId){
        			return i;        			
        		}
        	}
        	return -1;
        },
        
        getCheckedRows: function(){
            return this.tableViewer.getCheckedRows();
        } 
        
    });
                
    
})();
