/****************************************************************************** 
* Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalWidget");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.process.web.client.internal.approval.ProcessClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.ESignOperationDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.StateTransitionConstraintsUtil");

(function(){ // module container
	var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;

    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;    
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var ProcessClient = com.ibm.rqm.process.web.client.internal.approval.ProcessClient;
    var SaveParticipant = com.ibm.asq.common.web.ui.internal.model.SaveParticipant;
    var ConstraintsUtil = com.ibm.rqm.planning.web.ui.internal.util.StateTransitionConstraintsUtil;
    var Session = com.ibm.team.repository.web.client.session;
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.extension.ReviewSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString:
        	"<div>"+
        	"<div style='overflow-x:auto;' dojoAttachPoint=\"_reviewSection\"></div>"+
        	"<div style='overflow-x:auto;' dojoAttachPoint=\"_commentsSection\"></div>"+
			"<div dojoAttachPoint=\"loadingNode\" class=\"loading-animation\"></div>"+
			"</div>",
        
        constructor: function(args) {
			this._initialized = true;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);       
            this.getParentEditor().registerApprovalSection(this);
            this.addWorkItemSupport(null, this.getParentEditor()?this.getParentEditor().isEditable():this.isEditable());
            this._initialize();
        },
        
        setCompositeDTO:function(compositeDTO){
        	this.compositeDTO = compositeDTO;
        },
        
		//@Override
        isEditable: function() {
        	
            // Formal Review is editable if artifact is locked

            if(this.isPrintView && this.isPrintView()){
        		return false;
        	}
        	
			if (this.isSnapshot()) {
				return false;
			}
			
			if (this.isArchived()) {
				return false;
			}
			var parentEditor = this.getParentEditor();
			if (dojo.isFunction(parentEditor.isReadOnlyCompare) && parentEditor.isReadOnlyCompare()) {
			    return false;
			}
			var isMerging = dojo.isFunction(parentEditor.isMerging) && parentEditor.isMerging();
			if (isMerging && !parentEditor.diffMergeController) {
			    // parent editor is in the process of merging and it is the read-only merge source
			    return false;
			}
			
			if(this.isReadOnlyConfiguration()) {
				return false;
			}
			
			//in case of ready-only license, this section must be editable to viewers
            var op = PermissionsClient.getStoredPermission(PermissionsClient.OP_PROCESS_APPROVAL);
            if (op && op.create && op.edit) {
				return true;
			}
			return this.inherited('isEditable', arguments);
        },
        
        isTitleEditable: function(){
        	var parentEditor = this.getParentEditor();
        	if(dojo.isFunction(parentEditor.isReviewSectionTitleEditable)){
        		return parentEditor.isReviewSectionTitleEditable();        		
        	}else{
        		return this.isEditable();
        	}
        },
        
        isSectionRemovable: function(){
        	var parentEditor = this.getParentEditor();
        	if(dojo.isFunction(parentEditor.isReviewSectionRemovable)){
        		return parentEditor.isReviewSectionRemovable();        		
        	}else{
        		return this.isEditable();
        	}
        },
        
        refreshConstraints: function(teamArea) {
            ConstraintsUtil.loadConstraints(Constants.TYPE_FORMAL_REVIEW, teamArea, /* callback */null, /* projectArea */null, /* useCache */ true);                
        },
        
        initialize: function(isEditable){
        	//in case of compare merge we need to reinitialize the section to read only in most cases so we need to override from AAEditor, 
        	//for all other cases we don't need to do anything, _initialize will handle the rest 
        	if(this.getParentEditor && this.getParentEditor().isMerging && this.getParentEditor().isMerging()){
        		this._initialize(false);
        	}
        },
			
        //If refreshAfterSave is called, means we need to refresh our own compositeDTO too
        _initialize: function(refreshAfterSave){        	
            // hopefully our container has done init in an order were we will not
            // make extra constraint loading calls
            this.refreshConstraints(this.getParentEditor().getTeamAreaItemId());
        	dojo.empty(this._reviewSection);
        	dojo.empty(this._commentsSection);
        	dojo.addClass(this.loadingNode,"loading-animation");
        	this.dirty = false; //our dirty flag needs to be different from AAEditorSection
        	this._statusDirty = false;
        	this.currentUser = Session.getAuthenticatedContributor().itemId;
        	if(this._widget){
        		this._widget.cleanup();
        		this._widget.destroy();
        	}        	
        	 if (this.getParentEditor().getVersionableItemId() != null) {
                 var params = {
                     artifactItemTypeName: this._getItemType(),
                     artifactPackageUri: this._getItemPackageUri(),
                     artifactItemId: this.getParentEditor().getVersionableItemId(),
                     includeArchived: true
                 };
                 if(this.configurationItemId){
                	 params.configurationItemId = this.configurationItemId;
                 }
                 if(!this.compositeDTO || refreshAfterSave == true){
                	 this.refreshParentObject = true;
                	 var responseHandler = new ServiceResponseHandler(this, "_initApproval", "_error");
                     ProcessClient.getApprovalGroupDTO(responseHandler, params); 
                 } else {
                	 this._initApproval(this.compositeDTO);
                 }                 
             }
             else {
                 this._initApproval(null);
             }	            	
        },
        
        _initApproval: function(approvalDTO){      
            if (!this.loadingNode && !this._commentsSection) {
                // do nothing if the parent editor is closed before the server call returned to callback
                return;
            }
            this.compositeDTO = approvalDTO;
            if(this.refreshParentObject && this.notifyApprovalCompositeChanged && dojo.isFunction(this.notifyApprovalCompositeChanged)){
            	 this.notifyApprovalCompositeChanged(approvalDTO);            	 
            }
            if(this.refreshParentObject){
            	delete this.refreshParentObject;
            }
        	dojo.removeClass(this.loadingNode,"loading-animation");
        	var approvalGroups =  (approvalDTO && approvalDTO.approvalGroupResults)? approvalDTO.approvalGroupResults: null;
        	
        	this._widget = new com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalWidget({        		
        		inSection: true,
        	    parentVersionableItemId: this.getParentEditor().getVersionableItemId(),
        		isEditable: this.isEditable(),
        		parentSection: this,
        		parentItemType:this._getItemType(),
        		existingApprovalGroups: approvalGroups        		
        	});
            this.addMangedWidget(this._widget);
        	this._reviewSection.appendChild(this._widget.domNode);        	
      
        	var generalComments  = (approvalDTO && approvalDTO.comments)? approvalDTO.comments : [];
        	this._commentsWidget = new com.ibm.rqm.process.web.ui.internal.view.approval.ApprovalWidget.CommentSection({
        		existingComments: generalComments,        		
        		isMainHeader: true,
        		parentSection: this,
        		isGeneralComment: true,
        		isEditable: this.isEditable(),
        		title: Constants.getMessage("labelGeneralComments")        		
        	});
            this.addMangedWidget(this._commentsWidget);

        	this._commentsSection.appendChild(this._commentsWidget.domNode);           
        	this.initialized = true;

        	// get the constraint configuration data
        	var artifactType = this.getParentEditor().getArtifactType();
            var constraintId = ConstraintsUtil.getESignFormalReviewConstraintId(artifactType);
            var teamArea = this.getParentEditor().getTeamAreaItemId();
           if (this._compositeDTO && this._compositeDTO.configurationData) {
            	ConstraintsUtil.setConfigurationData(this._compositeDTO.configurationData, constraintId, Constants.TYPE_FORMAL_REVIEW, teamArea, dojo.hitch(this, "_setConstraintConfigData"));
            } else {
            	ConstraintsUtil.loadConfigurationData(constraintId, Constants.TYPE_FORMAL_REVIEW, ConstraintsUtil.ESignConfigElementId, /* isSaveOperation */ true, teamArea, dojo.hitch(this, "_setConstraintConfigData"));
            }
        },
        
        _setConstraintConfigData: function(configData) {
            this.constraintConfigData = ConstraintsUtil.parseESignaureConfigurationData(configData);
        },
        
        _getItemType: function() {
            switch (this.getParentEditor().getArtifactType()) {
                case Constants.TYPE_CASE:
                    return "VersionedTestCase";
                case Constants.TYPE_PLAN:
                    return "VersionedTestPlan";
                case Constants.TYPE_SUITE:
                    return "VersionedTestSuite";    
                case Constants.TYPE_RESULT:
                    return "ExecutionResult";    
                case Constants.TYPE_SUITE_RESULT:
                case Constants.MODEL_TYPE_SUITE_LOG:
                    return "ExecutionResultGroup";    
                case Constants.TYPE_SCRIPT:
                    return "VersionedExecutionScript";    
                default:
                    return "";
            }           
        },
        _getItemPackageUri: function() {
            switch (this.getParentEditor().getArtifactType()) {
                case Constants.TYPE_CASE:
                case Constants.TYPE_PLAN:
                case Constants.TYPE_SUITE:
                case Constants.TYPE_SCRIPT:
                    return Constants.PACKAGE_URI;    
                case Constants.TYPE_RESULT:    
                case Constants.TYPE_SUITE_RESULT:
                case Constants.MODEL_TYPE_SUITE_LOG:
                    return Constants.EXECUTION_PACKAGE_URI;    
                default:
                    return "";
            }           
		},

        //@override from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited(arguments);
            
            // need to reinsert work item support.
            this.addWorkItemSupport(null, this.getParentEditor()?this.getParentEditor().isEditable():this.isEditable());
        },
                
        // Note: we should rename this method to avoid confusion with the saveFinished in AbstractArtifactEditorSection
        _saveFinished: function(savedItem){       
        	if(this.behaveAsParticipant){
        		// WI 109298: Comment out not to call saveFinshed() in its parent class where handling to add/remove quality task of section. 
        		// Instead, copying the codes from AbstractArtifactEditorSection.saveFinished() except the codes handling quality tasks.    		
        		// this.inherited('saveFinished', arguments);
        		this.refreshSection();
     			this.setSectionDirty(false);
     			if(this._richText && this._richText.inEditMode){
     				this._richText._editor.focus();
     			}
        		
        		this._initialize(this.isEditable());
        		this.getSaveContext().removeParticipant(this.saveParticipant);
        		this.saveParticipant = null;        		
        	}
        },
		
		//@Overridden from AbstractArtifactEditorSection
        saveRequiresSignature: function() {        	
        	if (!this.initialized) {
//        		console.debug(this.declaredClass + ": esig NOT required. section wasn't used.");
        		return false;
        	}
            if (this.isStatusDirty()){
                var constraint = ConstraintsUtil.getESignFormalReviewConstraintId(this.getParentEditor().getArtifactType());
                if (constraint && ConstraintsUtil.isConstraintEnabled(constraint, Constants.TYPE_FORMAL_REVIEW, this.getParentEditor().getTeamAreaItemId())) {
                    // there is constraint for E-Sign
                    if (this.constraintConfigData) {
                        // check if only approval is required from the config
                        var approvalOnly = (this.constraintConfigData.forTypeApprovalOnly == "true");
                        // parse the approval changes from the approval widget
                        var approvalCmds = this._widget.getFormattedJSON();
                        var hasApproval = false;
                        var approvalActions = [];
                        for (var i = 0; i < approvalCmds.length; i++) {
                            var approvalObj = approvalCmds[i].approvalObj;
                            hasApproval = hasApproval || (approvalObj.approvalType == Constants.TYPE_APPROVER);
                            var name = approvalObj.name;
                            var action = "";
                            var approvers = approvalObj.approvers || [];
                            for (var a = 0; a < approvers.length; a++) {
                                // it is not possible for the same user to be the approver more than once in a group
                            	if(approvers[a].user == this.currentUser){
                            		action = this._getActionDisplayString(approvers[a].status, approvalObj.approvalType);
                            		break;
                            	}
                            }
                            var approvalAction = { "action": action, "name": name };
                            approvalActions.push(approvalAction);
                        }
                        this.constraintConfigData.approvalActions = approvalActions;
                    
                        if (approvalOnly && !hasApproval) {
                            // only approval is required and there is no approval
                            return false;
                        }
                    }
                    // return the config data (true value) to indicate E-Sign is required and provide info to populate the E-Sign dialog
                    return this.constraintConfigData || {};
                }
            }
            //review signing feature disabled
//            console.debug(this.declaredClass + ": esig NOT required. feature disabled.");
            return false;
        },
        
        // get the displaying string for the approval action type
        //
        _getActionDisplayString: function(approvalAction, approvalType) {
            switch (approvalAction) {
                case Constants.TYPE_PENDING:
                    return Constants.getMessage("labelPendingApprovalTable");
                case Constants.TYPE_APPROVED:
                    if (approvalType == Constants.TYPE_REVIEWER) {
                        return Constants.getMessage("labelReviewedApprovalTable");
                    } else {
                        return Constants.getMessage("labelApprovedApprovalTable");
                    }
                case Constants.TYPE_REJECTED:
                    return Constants.getMessage("labelRejectedApprovalTable");
                default:
                    return approvalAction;
            }
        },
        
		validateStateTransition:function(initialState, action, endState, editable, callback)
		{
			return true;
		},
		
		/**
		 * Save formal review/approval group information independent of the artifact save. Use skipSave (=true) to
		 * avoid triggering a save of the parent artifact. This is necessary if the artifact save is after this
		 * approval save. Otherwise, the artifact save would fail due to stale state id. Currently, Test script editor
		 * is doing 2 phase save of the artifact and the approval and it's the only user of this flag.
		 */
		internalFormalReviewSave: function(callback, esignPassword, esignComment, esignReason, esignAction, skipUpdateState, skipSave){			
			this._doSave(callback, esignPassword, esignComment, esignReason, esignAction, skipUpdateState, skipSave);
		},
		
		_doSave: function(callback, esignPassword, esignComment, esignReason, esignAction, skipUpdateState, skipSave){			
			var internalHandlerObj = {
				success: function(item){						
					callback.success(item);						 
				},
				error: function(e)
				{
					var message = (e && e.message)?e.message: Constants.getMessage("labelApprovalStatusSaveError");												
					callback.error(message);
					// maybe new constraints were added
					ConstraintsUtil.loadConstraints(Constants.TYPE_FORMAL_REVIEW, this.getParentEditor().getTeamAreaItemId());
					console.error("ApprovalWidget --- Error saving:"+ (e && e.message)? e.message:"");						
				}
			};
			var generalComment  = this._commentsWidget.getNewComment();			
	
			var responseHandler = new ServiceResponseHandler(internalHandlerObj, "success", "error");
			var args = {
				artifactItemId: this.getParentEditor().getVersionableItemId(),
				artifactItemTypeName: this._getItemType(),
				artifactPackageUri: this._getItemPackageUri(),
				operationJson: this._widget.getFormattedJSON(),
				skipStateUpdate: skipUpdateState
			};			
			if(generalComment){
				args.comment = generalComment.getComment().comment;
			}
            if (esignPassword) args.esignPassword = esignPassword; 
            if (esignComment) args.esignComment = esignComment; 
            if (esignReason) args.esignReason = esignReason; 
            if (esignAction) args.esignAction = esignAction; 
            if (skipSave) args.skipSave = true; 
			ProcessClient.postApprovalGroupDTO(responseHandler, args);			
		},
		
		isDirty: function(){
			return this.dirty;
		},
		
		setApprovalDirty: function(isInSection){
			if(isInSection){
			    this.setSectionDirty(true);				
			}			
			var pEditor = this.getParentEditor();
			pEditor.updateSaveable(true);
			//Insert DirtyItemInfo data, just make this call once
			if( dojo.isFunction(pEditor.postDirtyItemInfo) && !pEditor.isNew() && !this.dirty){
				pEditor.postDirtyItemInfo();
			}
			this.dirty = true;
			dojo.publish(com.ibm.asq.common.web.ui.internal.util.GlobalEvents.UpdateTabLabel, [this._uri ? this._uri : window.location.hash, "", "", true]);
		}, 
		
		setDirty: function(){
			this.setApprovalDirty(true);
			if(dojo.isFunction(this.getParentEditor) && dojo.isFunction(this.getParentEditor().setDirty)){			
				this.getParentEditor().setDirty(true);
			}
		},
		
		isStatusDirty: function(){
		    return this._statusDirty;
		}, 
		
	    setStatusDirty: function(){
	        this._statusDirty = true;
	    },
				
		_error: function(error){
			console.error(error &&error.message?error.message:"An error occurred on ReviewSection - "+ error);
		},
		
		initializeAsSaveParticipant:function(esignPassword, esignComment, esignReason, esignAction){
			this._addSaveParticipant(esignPassword, esignComment, esignReason, esignAction);		
			this.behaveAsParticipant = true;
		},
		
/*		onActivated: function(){
			this.inherited('onActivated', arguments);
			if(this._widget){
				this._widget.resizeHandler();
			}
		}, */
		
		//Probably will get removed
        _addSaveParticipant: function(esignPassword, esignComment, esignReason, esignAction){    
			var artifactItemId = this.getParentEditor().getArtifact()?this.getParentEditor().getArtifact().itemId + "_approvalSection":"_approvalSection";
			
			var saveParticipant = this.getSaveContext().findParticipant(artifactItemId);
			if(!saveParticipant){
				saveParticipant = new SaveParticipant();				
			}
			            
            saveParticipant.setDirty(false);
            
            saveParticipant.getItemType = function(){
                return "com.ibm.rqm.process.ApprovalRecord";
            };
			
			var self = this;
			saveParticipant.parent = this.getParentEditor();
			saveParticipant.saveContext = this.getSaveContext();
			
			saveParticipant.getParticipantId = function(){
                return artifactItemId;
            };
			saveParticipant.getStateId = function(){
                return self.getParentEditor().getStateId();
            };
			saveParticipant.isDirty = function(){				
				return self.isDirty();
            };
			saveParticipant.getAttributes = function(){
				var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_PROCESS_APPROVAL);								
				var editor = self.getParentEditor();
				var attributes = {
						artifactItemTypeName: self._getItemType(),
						artifactPackageUri: self._getItemPackageUri()
				};				
				var operations =  self._widget.getFormattedJSON();
				var attributes = {
					artifactItemTypeName: self._getItemType(),
					artifactPackageUri: self._getItemPackageUri(),
					operationJson: dojo.toJson(operations)
				};
	            if (esignPassword) attributes.esignPassword = esignPassword; 
	            if (esignComment) attributes.esignComment = esignComment; 
                if (esignReason) attributes.esignReason = esignReason; 
                if (esignAction) attributes.esignAction = esignAction; 

				var generalComment  = self._commentsWidget.getNewComment();
				if(generalComment)
					attributes.comment = generalComment.getComment().comment;
				
				if (editor.isNew()) { 
					this.saveContext.createReference(this, "artifactItemId", this.parent);
				}
				if (!editor.isNew()) {
					attributes.artifactItemId = editor.getVersionableItemId();
				}
				return attributes;
            };            
            saveParticipant.saveFinished =  dojo.hitch(this, "_saveFinished");
            this.getSaveContext().addParticipant(saveParticipant);
            this.saveParticipant = saveParticipant;
        },

        getFormattedJson: function(){
        	if(this._widget){
        		var attributes = this._widget.getFormattedJSON();        		
        		var generalComment  = this._commentsWidget.getNewComment();
				if(generalComment)
					attributes.comment = generalComment.getComment().comment;
				return attributes;
        	}
        },
        
        uninitialize: function(){
        	this.inherited(arguments);
        	this.initialized = false;
        }	
    });
    
})();
